// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.animation;

import fr.unicaen.openmap.*;
import com.bbn.openmap.Layer;
import com.bbn.openmap.gui.OMToolComponent;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.AnimationDSEventManager;
import fr.unicaen.openmap.dataStorage.MonoValuedFilterDataStorage;
import fr.unicaen.openmap.dataStorage.AnimationDataStorageListener;
import fr.unicaen.openmap.gui.ThemaMapFileChooserManager;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.util.ImageExporter;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;

/**
 * Tool which manages Animations.
 *
 * @author jchauvea
 */
public class MetaAnimator extends OMToolComponent implements AnimationDataStorageListener {

    private JComponent gui;
    private Box chainBoxList;
    private List<MonoValuedFilterDataStorage> availableDSList;
    private List<MonoValuedFilterDataStorage> usedDSList;
    private LayerHandler layerHanlder;
    private JButton bShortCut;
    private JButton bNewChain;
    //private JButton bCloseMeta;
    private JPanel toolGUI;
    private JTextField fieldPath;
    private int nbChain = 0;
    private List<AnimationDSChain> chains;
    private JDialog dialog;
    private JPanel toolBar;
    private ArrayList<AnimationDSChainListener> listeners;
    private ArrayList<AnimationDSChainCascade> cascades;
    public static String nbChainsPropertyString = "nbChains";

    public MetaAnimator() {
        super();
        listeners = new ArrayList<AnimationDSChainListener>();
        availableDSList = new ArrayList<MonoValuedFilterDataStorage>();
        usedDSList = new ArrayList<MonoValuedFilterDataStorage>();
        cascades = new ArrayList<AnimationDSChainCascade>();
        chains = new ArrayList<AnimationDSChain>();
        bShortCut = new JButton(Util.loadIcon("Movie24.png"));
        AnimationDSEventManager.addAnimationDataStorageListener(this);
        bShortCut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                popup();
            }
        });
        ComponentSizer.fixComponentSize(bShortCut, new Dimension(28, 28));
        bShortCut.setBackground(Color.GRAY);
        bShortCut.setOpaque(false);
        toolGUI = new JPanel();
        toolGUI.add(bShortCut);
        toolGUI.setBackground(Color.GRAY);
        toolGUI.setOpaque(false);
        bNewChain = new JButton(Util.loadIcon("newAnim.png"));
        bNewChain.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                addChainGUI(initChain(), getNbChain());
            }
        });
        bNewChain.setToolTipText(i18n.get(this, "addAnimation", "Add an animation"));
        ComponentSizer.fixComponentSize(bNewChain, new Dimension(35, 35));
        bNewChain.setBackground(Color.GRAY);
        bNewChain.setOpaque(false);
        bNewChain.setVisible(false);
    }

    public void popup() {
        dialog = new JDialog();
        dialog.getContentPane().add(getGUI());
        dialog.setSize(700, 600);
        dialog.setLocationRelativeTo(bShortCut);
        dialog.setVisible(true);
        bNewChain.setVisible(true);
    }

    @Override
    public void findAndInit(Object someObj) {
        if (someObj instanceof LayerHandler) {
            layerHanlder = (LayerHandler) someObj;
        }
    }

    /**
     * Collect all animations (AnimationDataStorage) by browsing layerhandler's layers
     * @return the number of collected animations
     */
    private int collectAnimationDataStorages() {
        availableDSList.clear();
        List<MonoValuedFilterDataStorage> tmpList = new ArrayList<MonoValuedFilterDataStorage>();
        for (Layer l : layerHanlder.getLayers()) {
            if (l instanceof ThematicDataDisplayLayer) {
                ThematicDataDisplayLayer tddl = (ThematicDataDisplayLayer) l;

                tmpList.addAll(
                        tddl.getDataStorageIterator().getDataStorage().getAnimationDataStorageList());

                for (MonoValuedFilterDataStorage ads : tmpList) {
                    if (!availableDSList.contains(ads) && !usedDSList.contains(ads))
                        availableDSList.add(ads);
                }

            }
        }
        return availableDSList.size();

    }

    public Component getGUI() {
        if (gui == null) {
            collectAnimationDataStorages();
            gui = new JPanel(new BorderLayout());
            JPanel wrap = new JPanel(new BorderLayout());
            gui.add(wrap, BorderLayout.NORTH);
            toolBar = new JPanel(new BorderLayout());
            Box box = Box.createVerticalBox();
            final JPanel wrapAnimExport = new JPanel();
            wrapAnimExport.add(bNewChain);
            toolBar.add(wrapAnimExport, BorderLayout.WEST);
            //ComponentSizer.fixComponentSize(wrapAnimExport, new Dimension(40, 40));
            final JToggleButton exportButton = new JToggleButton(Util.loadIcon("Export16.png"));
            wrapAnimExport.add(exportButton);

//            bCloseMeta = new JButton(Util.loadIcon("closearrow.png"));
//            bCloseMeta.addActionListener(new ActionListener() {
//
//                public void actionPerformed(ActionEvent arg0) {
//                    dialog.dispose();
//                }
//            });

            final JPanel panelExport = new JPanel();
            fieldPath = new JTextField();
            fieldPath.setEditable(false);
            JButton bBrowse = new JButton("browse");
            JButton bExport = new JButton("Export");
            exportButton.setBackground(Color.GRAY);
            exportButton.setOpaque(false);
            ComponentSizer.fixComponentSize(exportButton, new Dimension(35,35));
            fieldPath.setText(
                    Environment.get(ThemaMapFileChooserManager.DATA_SAVE_PATH, System.getProperty("user.dir")));
            bBrowse.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent ae) {
                    ThemaMapFileChooserManager.dataLoadFileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                    fieldPath.setText(
                            ThemaMapFileChooserManager.dataLoadDialog("", null) + System.getProperty("file.separator"));
                }
            });

            bExport.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent arg0) {
                    SwingUtilities.invokeLater(new Runnable() {

                        public void run() {
                            new Exporter(fieldPath.getText(), ImageExporter.ImageType.JPG).run();
                        }
                    });

                }
            });
            panelExport.add(fieldPath);
            panelExport.add(bBrowse);
            panelExport.add(bExport);
            exportButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0) {
                    panelExport.setVisible(exportButton.isSelected());
                    //wrapAnimExport.updateUI();
                }
            });
            wrapAnimExport.add(panelExport);
            panelExport.setVisible(false);

//            bCloseMeta.setBorder(null);
//            JPanel wrapCloseMeta = new JPanel();
//            wrapCloseMeta.add(bCloseMeta);
//            bCloseMeta.setBackground(Color.gray);
//            bCloseMeta.setOpaque(false);
//            toolBar.add(wrapCloseMeta, BorderLayout.EAST);
            //ComponentSizer.fixComponentSize(wrapAnimExport, new Dimension(40, 40));
            toolBar.setBorder(BorderFactory.createMatteBorder(1, 1, 0, 1, Color.gray));
            wrap.add(toolBar, BorderLayout.NORTH);
            wrap.add(new JScrollPane(box), BorderLayout.CENTER);
            chainBoxList = Box.createVerticalBox();
            box.add(chainBoxList);
            for (AnimationDSChain ch : chains)
                ch.addAnimationsToAdd();
            initChainsGUI();
        }
        return gui;
    }

    public void fireAnimationDSSelected(AnimationDSChain source, MonoValuedFilterDataStorage ads) {
        for (AnimationDSChainListener adscl : listeners) {
            if (adscl == source)
                continue;
            else
                adscl.animationDSSelected(ads);
        }
    }

    public void fireAnimationDSUnselected(AnimationDSChain source, MonoValuedFilterDataStorage ads) {
        for (AnimationDSChainListener adscl : listeners) {
            if (adscl == source)
                continue;
            else
                adscl.animationDSUnselected(ads);
        }
    }

    @Override
    public Container getFace() {
        if (getUseAsTool()) {
            return toolGUI;
        } else {
            return null;
        }
    }

    @Override
    public boolean getUseAsTool() {
        return true;
    }

    private AnimationDSChain initChain() {
        AnimationDSChain newChain = new AnimationDSChain(this, i18n.get(MetaAnimator.class, "animation", "Animation ") + (getNbChain() + 1));
        chains.add(newChain);
        listeners.add(newChain);
        return newChain;
    }

    public void initChainsGUI() {
        int i = 0;
        for (AnimationDSChain chain : chains) {
            addChainGUI(chain, i++);
        }
    }

    private void addChainGUI(AnimationDSChain chain, int index) {
        if (index > 1) {
            AnimationDSChainCascade cascade = new AnimationDSChainCascade(
                    chains.get(index - 2), chains.get(index - 1));
            chainBoxList.add(cascade.getGUI());
            cascades.add(cascade);
        }
        JComponent chainPanel = chain.getGUI();
        chainPanel.setBorder(BorderFactory.createTitledBorder(chain.getName()));
        chainBoxList.add(chainPanel);
        toolGUI.add(chain.getToolGUI());
        toolGUI.updateUI();
        gui.setVisible(false);
        gui.setVisible(true);

    }

    /**
     * @return the availableDSList
     */
    public List<MonoValuedFilterDataStorage> getAvailableDSList() {
        return availableDSList;
    }

    /**
     * @param availableDSList the availableDSList to set
     */
    public void setAvailableDSList(List<MonoValuedFilterDataStorage> availableDSList) {
        this.availableDSList = availableDSList;
    }

    /**
     * @return the usedDSList
     */
    public List<MonoValuedFilterDataStorage> getUsedDSList() {
        return usedDSList;
    }

    public void removeChain(AnimationDSChain chain) {
//        if (chains.size() == 1) { // we always keep one chain at least
//            return;
//        }
        listeners.remove(chain);
        chains.remove(chain);
        chainBoxList.remove(chain.getGUI());
        //remove cascadePanel
         if (!chains.isEmpty())
            chainBoxList.remove(cascades.get(cascades.size() - 1).getGUI());
        toolGUI.remove(chain.getToolGUI());
        toolGUI.updateUI();
        chainBoxList.setVisible(false);
        chainBoxList.setVisible(true);
    }

    public void animationRemoved(MonoValuedFilterDataStorage ads) {
        availableDSList.remove(ads);
        usedDSList.remove(ads);
        for (int i = 0; i < chains.size(); i++)
            chains.get(i).animationRemoved(ads);
    }

    public void animationAdded(MonoValuedFilterDataStorage ads) {
        availableDSList.add(ads);
        for (AnimationDSChain adsc : chains) {
            adsc.animationAdded(ads);
            if (ads.isSet())
                return;
        }
        ads.setMetaAnimator(this);

    }

    public void animationChanged(MonoValuedFilterDataStorage ads) {
        for (AnimationDSChain adsc : chains)
            adsc.animationChanged(ads);
    }

    public LayerHandler getLayerHandler() {
        return layerHanlder;
    }

    public Layer[] getLayers() {
        return layerHanlder.getLayers();
    }

    public int getNbChain() {
        return chains.size();
    }

    @Override
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        nbChain = Integer.parseInt(properties.getProperty(prefix + nbChainsPropertyString, "1"));
        for (int i = 0; i < nbChain; i++)
            initChain();
        for (AnimationDSChain dsc : chains) {
            dsc.setProperties(prefix, properties);
        }
    }

    @Override
    public Properties getProperties(Properties properties) {
        properties = super.getProperties(properties);
        String prefix = PropUtils.getScopedPropertyPrefix(this);
        properties.put(prefix + nbChainsPropertyString, String.valueOf(chains.size()));
        for (AnimationDSChain dsc : chains) {
            dsc.getProperties(prefix, properties);
        }
        return properties;
    }

    public int getChainRank(AnimationDSChain chain) {
        return chains.indexOf(chain);
    }

    public MonoValuedFilterDataStorage findAnimationDataStorage(String layerName, int index) {
        for (MonoValuedFilterDataStorage ads : availableDSList) {
            if (ads.getLayer().getName().equals(layerName) &&
                    ads.getLayer().getDataStorageIterator().getDataStorage().getAnimationDataStorageList().indexOf(ads) == index)
                return ads;
        }
        for (MonoValuedFilterDataStorage ads : usedDSList) {
            if (ads.getLayer().getName().equals(layerName) &&
                    ads.getLayer().getDataStorageIterator().getDataStorage().getAnimationDataStorageList().indexOf(ads) == index)
                return ads;
        }
        return null;
    }

    class Exporter extends Thread {

        private String fileName = "AnimationExport-";
        private int index = 0;
        private String folder;
        private boolean end;
        private ImageExporter.ImageType type;
        private Action action = new AbstractAction() {

            public void actionPerformed(ActionEvent e) {
                for (AnimationDSChain ch : chains) {
                    if (!ch.nextStep())
                        stopThread();
                }
            }
        };

        public Exporter(String folder, ImageExporter.ImageType type) {
            this.folder = folder;
            this.type = type;
        }

        @Override
        public void run() {
            dialog.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            while (!end) {
                ImageExporter.exportInFile(folder + fileName + index + "." + type.toString().toLowerCase(),
                        type);
                index++;
                action.actionPerformed(null);
            }
        }

        public synchronized void stopThread() {
            end = true;
            dialog.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
    }
}

