// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage;

import java.awt.Component;
import java.util.Properties;
import javax.swing.JComponent;

import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
 
/**
 * Classes implementing this interface are able to deliver DataObject
 * instances.
 * <br/>
 * Standards implementations of this interface over DataStorage are provided 
 * in the package.
 */
public interface DataStorageIterator {

    /**
     * Tells if we can call next();
     */
    public boolean hasNext();
    /**
     * get the next object in the list.
     */
    public DataObject next();
    
    /**
     * Reset to first object.
     */
    public void reset();
    
    /**
     * Do we need this ?
     */
    public boolean isReady();

    /**
     * @return A component to be layed in palette to configure DataStorage
     */
    public Component getGUI();
 
    
    /**
     * @return A component to be layed in palette to display datas 
     */
    public JComponent getDataGUI();


    //try to save
    public Properties getProperties(String prefix,Properties properties);
    //...    
    
     /**
     * We must change this ! Hack ... 
     */    
    public DataStorage getDataStorage();
  
    public int getCurrentIndex();

    /**
     * get info about the model ...
     */
    public String[] getDataColumnNames();

    //public String[] getKeyableDataColumnNames();
    /**
     * get info about the model ...
     */
    public String[] getOMColumnNames();

    /**
     * returns the real data column name given its index
     */
    public String getRealColumnName(int index);

    /**
     * returns the real OM column name given its index
     */
    public String getRealOMColumnName(int index);


    public int getIndexForColumnName(String name);

    public int getIndexForOMColumnName(String name);

    public BoundingBox getBoundingBox();

}
