// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/**
 * SharedDataStorage
 * 
 * Created on 24 July 2010
 *
 * @author  Jacques.Madelaine@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage;

import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.MapHandler;
import com.bbn.openmap.util.PropertyStringFormatException;

import fr.unicaen.openmap.app.ThemaMap;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.propertyEditor.DataStoragePropertyEditor;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;

import java.util.*;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.*;

/**
 * A SharedDataStorage is a DataStorage borrowed from another ThematicDataDisplayLayer
 * pb when the exporter layer is destroyed 
 * pb of multiple DS listene: the SharedDataStorage must handle 2 DataStorageListener..
 */
public class SharedDataStorage extends DataStorage implements DataStorageListener {

    public static final String foreignLayerNamePropertyString = "sharedDS.foreignLayerName";
    // rank of the datastorage in the foreign layer using a Dick path
    public static final String dataStoragePathPropertyString = "sharedDS.dataStoragePath"; 

    /** used to pack all the properties in one property for the SharedDataStoragePropertyEditor */
    public final static String GLOBAL = "sharedDS.global"; 

    /** The original layer of the dataStorage */
    private ThematicDataDisplayLayer foreignLayer;
    
    ///** 
    // * dsl2 is the original DataStorageListener of the foreign dataStorage 
    // * while the current SharedDataStorage (this) is its DataStorageListener.
    // * Events received will be forward to dsl2.
    // * This model is interesting but not implemented as 
    // * we should restore the DataStorageListener of the foreign DataStorage 
    // * when the SharedDataStorage is destroyed, but we have no event for that
    // * even if the method Layer.dispose() exists...
    // */ 
    // private DataStorageListener dsl2;
    	
    public SharedDataStorage(DataStorageListener dsl) {
        super(dsl);
    }

    private DataStorage _dataStorage;

    public DataStorage getDataStorage() {
        return _dataStorage;
    }
   
    public void setDataStorage(DataStorage dataStorage) {
        _dataStorage = dataStorage;
    }

    public void dataStorageEvent(DataStorage source) {
        Debug.message("DataStorage", "[SharedDataStorage:] dataStorageEvent <== " + source);
	// update GUI
	resetDataTableStoragePanel();		    
	
	// forward the event to our layer...
	Debug.message("DataStorage", "[SharedDataStorage:] "+this + " ==> " + dsl);
	dsl.dataStorageEvent(this);

	// if (source != this && dsl2 != null) {
	//     // ... and to the original layer of the dataStorage
	//     Debug.message("DataStorage", "[SharedDataStorage:] "+ this + " ==> " + dsl2);
	//     dsl2.dataStorageEvent(source);
	// }
    }

    public boolean isReady() {
        return getDataStorage() != null && getDataStorage().isReady();
    }

    public DataObject getElementAt(int i) {
        return _dataStorage.getElementAt(i);
    }
  
    public int getCount() {
        return _dataStorage.getCount();
    }

    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        super.setProperties(prefix, properties);
	// System.out.println("**setProperties("+prefix+") \n"+properties.getProperty(prefix+GLOBAL)+"\n\n");
        try { // if called via PropertyInfo, remove "global" from the keys
            Properties global = PropUtils.parsePropertyList(properties.getProperty(prefix + GLOBAL), fr.unicaen.openmap.util.propertyEditor.SharedDataStoragePropertyEditor.propertySeparator, fr.unicaen.openmap.util.propertyEditor.SharedDataStoragePropertyEditor.keyValueSeparator);
            Set<String> keys = global.stringPropertyNames();
            String key = "";
            for (Iterator<String> iterator = keys.iterator(); iterator.hasNext();) {
                key = iterator.next();
                properties.put(prefix + key, global.getProperty(key));
                properties.remove(key); //useless, but clean
            }
        } catch (PropertyStringFormatException exc) {
            System.err.println("Warning: Parse error" + properties.getProperty(prefix + GLOBAL));
        }
	// System.out.println("*** setProperties "+properties.getProperty(prefix + foreignLayerNamePropertyString)+ " " + properties.getProperty(prefix + dataStoragePathPropertyString));
	setForeignLayer(getLayerByName(properties.getProperty(prefix + foreignLayerNamePropertyString)));
	setDataStorage(getDataStorageByPath(properties.getProperty(prefix + dataStoragePathPropertyString)));
	if (getForeignLayer() == null) {
	    final String t_prefix = prefix;
	    final Properties t_properties = properties;
	    //if (getForeignLayer() != null)
		new Thread() { // retry till the foreign layer is built
		    public void run() { 
			Debug.message("DataStorage", "[SharedDataStorage] setProperties runs a thread to wait for "+getForeignLayer());
			try { sleep(500); } catch(java.lang.InterruptedException e) {}
			setProperties(t_prefix, t_properties); }
		}.start();
	}
        initialize();
    }
    
    protected void setForeignLayer(ThematicDataDisplayLayer layer) {
	this.foreignLayer = layer;
	// System.out.println("SharedDataStorage.setLayer("+layer+")");
    }

    public ThematicDataDisplayLayer getForeignLayer() {
	return foreignLayer;
    }

    private void initialize() {
	Debug.message("DataStorage", "[SharedDataStorage] initialize");
	if (! isReady()) {
	    Debug.message("DataStorage", "[SharedDataStorage] can't initialize because dataStorage is not ready !!!!");
	    return;
        }
	//	// resetDataTableStoragePanel();

	// auto send an event that will refresh gui and inform listeners of the change
	dataStorageEvent(this);
    }

    public java.util.Properties getProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        properties = super.getProperties(prefix, properties);
	
	properties.put(prefix + foreignLayerNamePropertyString, (foreignLayer == null ? "" : foreignLayer.getName()));
	
	properties.put(prefix + dataStoragePathPropertyString, getDataStoragePath(foreignLayer, getDataStorage()));

	properties.put(prefix + GLOBAL + "." + foreignLayerNamePropertyString, (foreignLayer == null ? "" : foreignLayer.getName()));
	
	properties.put(prefix + GLOBAL + "." + dataStoragePathPropertyString, getDataStoragePath(foreignLayer, getDataStorage()));
        
       // Management for getPropertyInfo()
        // Note : these properties can't be used once saved in a properties file 
	//            but can be used during getPropertyInfo to load properties from a PropertyEditor.
	properties.put(
                prefix + GLOBAL,
                this);

        return properties;
    }

   /**
     * Provides the essential information in order to initialize this Data Storage.
     */
    public java.util.Properties getPropertyInfo(Properties info) {

	info = super.getPropertyInfo(info);

        PropUtils.setI18NPropertyInfo(i18n,
				      info,
				      SharedDataStorage.class,
				      GLOBAL,
				      "Borrow data storage from", // label
				      "From which layer borrow the data storage?", // tooltip
				      "fr.unicaen.openmap.util.propertyEditor.SharedDataStoragePropertyEditor"
				      );

        info.put(initPropertiesProperty, GLOBAL + " " + getInitPropertiesPropertyString());
	
        return info;
    }

    public String getInitPropertiesPropertyString() {
	return "";//foreignLayerNamePropertyString + " " + dataStoragePathPropertyString;
    }
    
    public static List<ThematicDataDisplayLayer> getThematicDataDisplayLayers() {
	MapHandler mapHandler = ThemaMap.getMapHandler();
        if (mapHandler == null) 
	    return null;

	List<ThematicDataDisplayLayer> layers = new ArrayList<ThematicDataDisplayLayer>();
	LayerHandler lh = (LayerHandler)mapHandler.get("com.bbn.openmap.LayerHandler");
	if (lh == null)
	    return null;
	
	for (Layer l : lh.getLayers()) {
	    if (l instanceof ThematicDataDisplayLayer) {
		// only ThematicDataDisplayLayer have DataStorages
		layers.add((ThematicDataDisplayLayer) l);
	    }
	}
	return layers;
    }

    public static ThematicDataDisplayLayer getLayerByName(String name) {
	if (name == null)
	    return null;

	List<ThematicDataDisplayLayer> layers = getThematicDataDisplayLayers();
	if (layers == null)
	    return null;

	for (ThematicDataDisplayLayer l: layers)
	    if (name.equals(l.getName()))
		return l;
	return null;
    }

    public static String getDataStoragePath(ThematicDataDisplayLayer layer, DataStorage ds) {
	if (layer == null || layer.getDataStorageIterator() == null)
	    return "";
	return "p"+getDataStoragePath(layer.getDataStorageIterator().getDataStorage(), ds, "");
    }

    /** 
     * @parameter path: the path to ds accumulated so far from the layer's root DataStorage to root.  
     */
    private static String getDataStoragePath(DataStorage root, DataStorage ds, String path) {
	if (root == ds)
	    return path;
	
	if (root instanceof RemovableDataStorage) {  
	    if (root instanceof DualDataStorage) {
		String ret = getDataStoragePath(((DualDataStorage)root).getSecondDataStorage(), ds, path+"1");
		if (ret != null) 
		    return ret;
	    }
	    return getDataStoragePath(((RemovableDataStorage)root).getDataStorage(), ds, path+"0");
	}
	return null;
    }

    public DataStorage getDataStorageByPath(String path) {
	if (foreignLayer == null  || foreignLayer.getDataStorageIterator() == null || path == null)
	    return null;
	path = path.substring(1);
	// System.out.println("path <"+path+">");
	return getDataStorageByPath(foreignLayer.getDataStorageIterator().getDataStorage(), path);
    }

    public static DataStorage getDataStorageByPath(ThematicDataDisplayLayer layer, String path) {
	if (layer == null || layer.getDataStorageIterator() == null || path == null)
	    return null;
	return getDataStorageByPath(layer.getDataStorageIterator().getDataStorage(), path);
    }

    private static DataStorage getDataStorageByPath(DataStorage root, String path) {
	if ("".equals(path))
	    return root;
	switch (path.charAt(0)) {
	case '0':
	    // if (! root instanceof RemovableDataStorage)
	    //	break;
	    return getDataStorageByPath(((RemovableDataStorage)root).getDataStorage(), path.substring(1));
	case '1':
	    // if (! root instanceof DualDataStorage)
	    //	break;
	    return getDataStorageByPath(((DualDataStorage)root).getSecondDataStorage(), path.substring(1));
	}
	System.err.println("*** SharedDataStorage error for data storage path "+path);
	return null;
    }

    public JComponent getGUI() {
	JPanel panel = new JPanel();
	panel.add(DataStoragePropertyEditor.provideCustomPanel(this));
        return panel;
    }

    
    public DataTableModel getDefaultDataTableModel() {
	return _dataStorage.getDefaultDataTableModel();
    }

    protected void resetDataTableStoragePanel() {
	DataTableModel dataTableModel = getDefaultDataTableModel();
	if (dtsp == null)
	    dtsp = new DataTableStoragePanel(dataTableModel);
	else{
	    ((DataTableStoragePanel)dtsp).setModel(dataTableModel);
	    ((DataTableStoragePanel)dtsp).fireStructureChanged();
        }
    }

}
