// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.dataStorage.expr;

import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.DataStorageListener;
import fr.unicaen.openmap.dataStorage.expr.gui.RequestFilterDataStoragePanel;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import java.util.ArrayList;
import java.util.Arrays;
import javax.swing.Box;
import javax.swing.JComponent;

/**
 *
 * @author jchauvea
 */
public class RequestFilterDataStorage extends AddColumnDataStorage {

    public ArrayList<Integer> filteredDataIndex = new ArrayList<Integer>();
    public static String ifBegin = "if(";
    public static String ifEnd = ";1;0)";
    protected String exprFilter;

    public enum requestElement {

        EQ("="),
        LTE("<="),
        GTE(">="),
        LT("<"),
        GT(">"),
        PARENTHESIS("( )"),
        AND("AND"),
        OR("OR"),
        NOT("NOT"),
        DIFF("<>");
        private String sign;

        private requestElement(String s) {
            this.sign = s;
        }

        @Override
        public String toString() {
            return sign;
        }
    }

    public RequestFilterDataStorage(DataStorageListener dsl) {
        super(dsl);
    }

    public RequestFilterDataStorage(DataStorageListener dsl, DataStorage dataStorage) {
        super(dsl, dataStorage);
    }

    protected void computeFilteredDataIndex() {
        filteredDataIndex.clear();
        if (treeExpr == null) {
            return;
        }
        DataStorage ds = getDataStorage();
        int count = ds.getCount();
        for (int i = 0; i < count; i++) {
            DataObject data = ds.getElementAt(i);
            if (((Integer) Function.eval(treeExpr, data)) == 1) {
                filteredDataIndex.add(i);
            }
        }
    }

    @Override
    public DataObject getElementAt(int i) {
        return getDataStorage().getElementAt(filteredDataIndex.get(i));
    }

    @Override
    public int getCount() {
        return filteredDataIndex.size(); 
    }

    protected void applyFilter() {

        if (getDataStorage() == null || getDataStorage().getDefaultDataTableModel() == null) {
            return;
        }

        computeFilteredDataIndex();

	dataColumnNames = 
	    Arrays.asList(getDataStorage().getDefaultDataTableModel().getDataColumnNames());
        omColumnNames = 
	    Arrays.asList(getDataStorage().getDefaultDataTableModel().getOMColumnNames());

    }

    @Override
    public void initialize() {
        if (isReady()) {
            applyFilter();
            dsl.dataStorageEvent(this);
        }
    }

    @Override
    public JComponent getGUI() {
        if (gui == null) {
            gui = new RequestFilterDataStoragePanel(this);
        }
        Box b = Box.createVerticalBox();
        b.add(gui);
        b.add(getDataStorage().getGUI());

        return b;
    }

    @Override
    public boolean setExpr(String _expr) {
        if (super.setExpr(_expr)) {
            exprFilter = _expr;
            return super.setExpr(ifBegin + _expr + ifEnd);
        } else
            return false;
    }

    @Override
    public String getExpr() {
        return exprFilter;
    }
}
