// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.dataStorage.gml;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;

import javax.swing.JComponent;
import javax.swing.JPanel;

import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.dataStorage.CSVFileDataStorage;
import fr.unicaen.openmap.dataStorage.DataObject;
import fr.unicaen.openmap.dataStorage.DataStorageListener;
import fr.unicaen.openmap.dataStorage.InformationDataStorage;
import fr.unicaen.openmap.dataStorage.OMDataLocation;
import fr.unicaen.openmap.dataStorage.gml.in.GMLParser;
import fr.unicaen.openmap.dataStorage.gml.out.GMLFactory;
import fr.unicaen.openmap.dataStorage.gml.out.GMLWriter;
import fr.unicaen.openmap.dataStorage.gui.DataTableModel;
import fr.unicaen.openmap.dataStorage.gui.DataTableStoragePanel;
import fr.unicaen.openmap.dataStorage.xmltable.XmlTableDataStorage;
import fr.unicaen.openmap.util.propertyEditor.DataStoragePropertyEditor;
import fr.unicaen.openmap.util.propertyEditor.FileTypeDescription;

public class GMLDataStorage extends /* XmlTableDataStorage */ InformationDataStorage {

    private String gmlFile = "";
    private String xsdFile = "";
    private boolean initialized = false;
    // private GMLWriter writer;
    private String originType;
    private static final String gmlFilePropertyString = "gmlFile";
    private static final String xsdFilePropertyString = "xsdFile";
    private static final String changeXmlFilePropertyString = "gmlFile.change";
    public static final String GML_BY_INFO = "gml.info";
    public static final String XSD_BY_INFO = "xsd.info";
    public static final java.lang.String PARAM_GML = "gml";
    public static final java.lang.String PARAM_XSD = "xsd";

    // public final static String GLOBAL = "GMLDS.global";
    // private DSType originType;
//    public enum DSType {
//
//        SHAPEFILE(1, "shapefile"), POSTGIS(2, "postgis"),
//        GML(3, "gml");
//        protected int id;
//        protected String label;
//
//        DSType(int id, String label) {
//            this.id = id;
//            this.label = label;
//        }
//
//        public int getId() {
//            return this.id;
//        }
//
//        public String getLabel() {
//            return this.label;
//        }
//    }

    public GMLDataStorage(DataStorageListener dsl) {
        super(dsl);
        // writer = new GMLWriter();
        originType = "GMLDataStorage";

    }

    public GMLDataStorage(DataStorageListener dsl, String t) {
        super(dsl);
        // writer = new GMLWriter();
        originType = t;
    }

    public String getName() {
        return "".equals(gmlFile) ? name : stripFileNameWithClassPathDirectories(gmlFile);
    }

    public boolean isReady() {

        return initialized;
    }

    public Properties getPropertyInfo(Properties properties) {
        properties = super.getPropertyInfo(properties);
        /*
         * PropUtils.setI18NPropertyInfo(i18n, properties,
         * XmlTableDataStorage.class, xmlFilePropertyString,
         * xmlFilePropertyString, xmlFilePropertyString,
         * "com.bbn.openmap.util.propertyEditor.FilePropertyEditor" );
         */

        //list = super.getPropertyInfo(list);
//		PropUtils.setI18NPropertyInfo(i18n,
//					      properties,
//					      GMLDataStorage.class,
//					      "gml.info",
//					      "gmlFile", //label
//					      "gmlFile", //tooltip
//					      "fr.unicaen.openmap.util.propertyEditor.GMLDataStoragePropertyEditor"
//					      );
//		PropUtils.setI18NPropertyInfo(i18n,
//			      properties,
//			      GMLDataStorage.class,
//			      "",
//			      "",
//			      "",
//			      null //default editor : textfield
//			      );

        /*PropUtils
        .setI18NPropertyInfo(i18n, properties, GMLDataStorage.class,
        GML_BY_INFO, "File", "Select a GML file.",
        "fr.unicaen.openmap.util.propertyEditor.FileTypePropertyEditor");*/

//		properties.put(initPropertiesProperty,
//		 xmlFilePropertyString
//				GML_BY_INFO);


        /*PropUtils
        .setI18NPropertyInfo(i18n, properties, GMLDataStorage.class,
        XSD_BY_INFO, "File", "Select a XSD file.",
        "fr.unicaen.openmap.util.propertyEditor.GMLDataStoragePropertyEditor");*/

        //properties.put(initPropertiesProperty,
        // xmlFilePropertyString
        //		XSD_BY_INFO);


        PropUtils.setI18NPropertyInfo(i18n,
                properties,
                GMLDataStorage.class,
                GML_BY_INFO,
                "File",
                "Select a GML file.",
                "fr.unicaen.openmap.util.propertyEditor.FileTypePropertyEditor");

        properties.put(initPropertiesProperty,
                //		       xmlFilePropertyString
                GML_BY_INFO);


        return properties;
    }

    public void setProperties(String prefix, java.util.Properties properties) {
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        name = prefix;
        super.setProperties(prefix, properties);

        gmlFile = properties.getProperty(prefix + gmlFilePropertyString,
                properties.getProperty(prefix + GML_BY_INFO, ""));

        xsdFile = properties.getProperty(prefix + xsdFilePropertyString,
                properties.getProperty(prefix + XSD_BY_INFO, ""));

        Debug.message("DataStorage", "[GMLDataStorage] using file:" + gmlFile + ".");

        initialize();
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);

        properties.put(prefix + gmlFilePropertyString, stripFileNameWithClassPathDirectories(gmlFile));

        // Used with getPropertyInfo
        properties.put(prefix + GML_BY_INFO, // csvFilePropertyString,
                new FileTypeDescription(FileTypeDescription.GML_DESCRIPTION,
                gmlFile, FileTypeDescription.GML_PATTERN));

        return properties;
    }

    public void initialize() {

        try {
            GMLParser parser = new GMLParser(gmlFile);
            dataTable.setTable(parser.getDataObjects(),
                    getDataAttributesNamesToDisplay(parser.getColumnNames()),
                    parser.getOMGraphicsNames(),
                    getDataAttributesNamesAliases());

            dataTableModel = new DataTableModel(dataTable);
            resetDataTableStoragePanel(dataTableModel);
            initialized = true;
        } catch (Exception e) {
            e.printStackTrace();
            initialized = false;
        }
        dsl.dataStorageEvent(this);

    }

    /*public String getInitPropertiesPropertyString() {
    return "";
    }*/
    public JComponent getGUI() {
        JPanel panel = new JPanel();
        panel.add(DataStoragePropertyEditor.provideCustomPanel(this));
        return panel;
    }

    public String getGmlFile() {
        return gmlFile;
    }

    public void setGmlFile(String gmlFile) {
        this.gmlFile = gmlFile;
    }

    public String getXsdFile() {
        return xsdFile;
    }

    public void setXsdFile(String xsdFile) {
        this.xsdFile = xsdFile;
    }
}
