// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * DataTableSelectionStoragePanel.java
 *
 * Created on 9 novembre 2005, 23:31
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 */
package fr.unicaen.openmap.dataStorage.gui;

import fr.unicaen.openmap.dataStorage.SelectionDataStorageListener;
import fr.unicaen.openmap.util.Util;
import com.bbn.openmap.util.Debug;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

/**
 * This class is used to display the content of a TableModel.
 *
 */
public class DataTableStoragePanel extends JPanel implements SelectionDataStorageListener {

    protected JTable tableView = null;
    protected DataTableModel model = null;
    // private JPanel dataPanel, formatPanel;

    //    public  DataTableSelectionStoragePanel(JTable table, Vector<Color> colors, int initrow) {
    public DataTableStoragePanel(DataTableModel model) {
        super();
        setDoubleBuffered(true);
        this.model = model;
        try {
            setLayout(new BorderLayout());
            tableView = new JTable(model);
            tableView.setShowHorizontalLines(true);
            tableView.setShowVerticalLines(true);
            tableView.setIntercellSpacing(new Dimension(2, 2));
            tableView.setRowSelectionAllowed(true);
            tableView.setRowHeight(25);
            tableView.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS); //JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS); // AUTO_RESIZE_OFF);
            JScrollPane sp = new JScrollPane(tableView);
            add(sp, BorderLayout.CENTER);
        } catch (Exception e) {
            System.err.println("===> [ " + Util.className(this) + " ] :  Error " + e.toString());
            e.printStackTrace();
        }
    }

    /**
    Change the table model. Call fireStructureChanged()
     */
    public void setModel(DataTableModel model) {
        this.model = model;
        tableView.setModel(model);
        Debug.message("DataStorage", "[DataStorage]*** setModel " + Util.className(model));
        fireStructureChanged();
    }

    /**
     * We have to update the panel, as the data structure has changed. 
     * This method is used when the structure is not the same which
     * means a column or a row has been added or deleted.
     *
     **/
    public void fireStructureChanged() {
        Debug.message("DataStorage", "[DataStorage]*** fireStructureChanged " + Util.className(model));
        ((AbstractTableModel) model).fireTableStructureChanged();

    }

    /**
     * return the JTable used in the panel.
     **/
    public JTable getJTable() {
        return tableView;
    }
}
