// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.dataStorage.gui;

import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.util.Util;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.table.TableModel;

/**
 *
 * @author jchauvea
 */
public class EditableDataTableStoragePanel extends DataTableStoragePanel {

    private DataStorage dataStorage;

    public EditableDataTableStoragePanel(DataStorage ds, DataTableModel model) {
        super(model);
        this.dataStorage = ds;
        try {
            setLayout(new BorderLayout());
            tableView = new EditableDataStorageTable(this);
            tableView.setShowHorizontalLines(true);
            tableView.setShowVerticalLines(true);
            tableView.setIntercellSpacing(new Dimension(2, 2));
            tableView.setRowSelectionAllowed(true);
            tableView.setRowHeight(25);
            tableView.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS); //JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS); // AUTO_RESIZE_OFF);
            JScrollPane sp = new JScrollPane(tableView);
            add(sp, BorderLayout.CENTER);
        } catch (Exception e) {
        }
    }

    public TableModel getModel() {
        return model;
    }

    public DataStorage getDataStorage() {
        return dataStorage;
    }

    public void highlightInDataTable(int index) {

        tableView.setRowSelectionInterval(index, index);
        scrollToRowIndex(index);
        tableView.updateUI();
    }

    private void scrollToRowIndex(int index) {
        JViewport viewport = (JViewport) tableView.getParent();
        // This rectangle is relative to the table where the
        // northwest corner of cell (0,0) is always (0,0).
        Rectangle rect = tableView.getCellRect(index, 0, true);
        // The location of the viewport relative to the table
        Point pt = viewport.getViewPosition();
        // Translate the cell location so that it is relative
        // to the view, assuming the northwest corner of the
        // view is (0,0)
        rect.setLocation(rect.x - pt.x, rect.y - pt.y);
        // Scroll the area into view
        viewport.scrollRectToVisible(rect);
    }
}
