// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * SelectionColorPanel.java
 *
 * Created on 9 novembre 2005, 23:31
 *
 * @author Christophe.Turbout@info.unicaen.fr
 *
 */

package fr.unicaen.openmap.dataStorage.gui;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.dataStorage.SelectionDataStorage;
import fr.unicaen.openmap.dataStorage.SelectionDataStorageListener;
import fr.unicaen.openmap.layer.preLayer.UnicaenLayerListener;
import fr.unicaen.openmap.util.Util;
import java.awt.*;
import java.util.Locale;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * This class is used to display the content of Vector<Color>.
 *
 */
public class SelectionColorPanel extends JPanel implements SelectionDataStorageListener, ChangeListener {
    private JTabbedPane tabs = null;
    private UnicaenLayerListener unicaenLayerListener = null;
    private SelectionDataStorage selectionDataStorage = null;
    
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    protected I18n i18n = new BasicI18n(new Locale("fr","FR"));
    
    public  SelectionColorPanel(SelectionDataStorage selectionDataStorage) {
        super();
        this.selectionDataStorage=selectionDataStorage;
        setDoubleBuffered(true);
        setLayout(new BorderLayout());
        // le JTabpanned
        tabs = new JTabbedPane();
        tabs.addChangeListener(this);
        add(tabs, BorderLayout.CENTER);
        addColors();
    }
    
    private void addColors(){
        // the components
        try {
            for (int i=0;i<selectionDataStorage.getSelectionsSize();i++){
                ColorPanel cp = new ColorPanel(selectionDataStorage,i);
                cp.addUnicaenListener(unicaenLayerListener);
                tabs.addTab(selectionDataStorage.getSelectionName(i),cp);
            }
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    
    
    private void addColors(int selection){
        // the components
        try {
            ColorPanel cp = new ColorPanel(selectionDataStorage,selection);
            cp.addUnicaenListener(unicaenLayerListener);
            String name = selectionDataStorage.getSelectionName(selection);
            tabs.insertTab(name,null,cp,name,selection);
            
        } catch (Exception e){
            System.err.println("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
    /**
     *  Add UnicaenLayerListener to refresh layer();
     */
    public void addUnicaenListener(UnicaenLayerListener ull){
        for(int i=0;i<tabs.getTabCount();i++) ((ColorPanel)tabs.getComponentAt(i)).addUnicaenListener(ull);
        unicaenLayerListener=ull;
    }
    
    /**
     *  Remove the UnicaenLayerListener;
     */
    public void removeUnicaenListener(){
        unicaenLayerListener=null;
    }
     /*
      * We have to update panel, data structue has changed. This method is used when the structure is not the same which
      * means a column are a row has been added or suppressed.
      *
      **/
    public void fireStructureChanged(){
        // remove all Components in the panel
        tabs.removeAll();
        // add all colors ...
        addColors();
    }
    
    public void fireStructureChanged(int selection){
        // remove all Components in the panel
        tabs.remove(selection);
        // add all colors ...
        addColors(selection);
    }
    
    public void fireColumnAdded(int selection,int subselection){
        // get the panel
        ColorPanel panel = (ColorPanel) tabs.getComponent(selection);
        if (panel!=null) panel.addColor(selectionDataStorage.getSubSelectionNames(selection),selectionDataStorage.getSelectionColors(selection),subselection);
        else System.err.println("Impossible de mettre à jour les couleurs panel==null");
    }
    
    /**
     * When state is changed, we need to update SelectionDataStorage to set the Selection.
     **/
    public void stateChanged(ChangeEvent e) {
        int selectedTab = ((JTabbedPane)e.getSource()).getSelectedIndex();
        Debug.message(Util.className(this)," StateChanged colors : "+selectedTab+", CurrentSelection = "+selectionDataStorage.getCurrentSelection());
        if (selectionDataStorage!=null && selectedTab>-1 && selectionDataStorage.getCurrentSelection()!=selectedTab) {
            selectionDataStorage.setActiveSelection(selectedTab);
            JTabbedPane pane = (JTabbedPane) getParent();
            if (pane!=null) {
                SelectionStoragePanel parentPanel = (SelectionStoragePanel) pane.getParent();
                if (parentPanel!=null) parentPanel.setActiveSelection(selectedTab);
            }
            if (unicaenLayerListener!=null) unicaenLayerListener.refreshGraphics();
        }
    }
    
    /**
     * Set active selection. This give us the ability to match the pane selected in selections Panel.
     **/
    public void setActiveSelection(int selection){
        if (tabs.getSelectedIndex()!=selection ) tabs.setSelectedIndex(selection);
    }
}
