// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
/*
 * FileNamePanel.java
 *
 * Created on 8 juin 2007, 15:23
 */

package fr.unicaen.openmap.gui;

import com.bbn.openmap.util.FileUtils;
import com.bbn.openmap.util.Debug;
import java.io.File;

import fr.unicaen.openmap.event.GUIChangeEvent;
import fr.unicaen.openmap.event.GUIChangeListener;
import fr.unicaen.openmap.util.gui.ThemaMapJFileChooser;
import javax.swing.filechooser.FileFilter;

import java.util.regex.*;
import javax.swing.plaf.basic.BasicButtonUI;

/**
 *
 * @author  davy
 */
public class FileNamePanel extends javax.swing.JPanel {
    /**
     * Extract file name from its full path.
     */
    private String basename(String fileName) { 
	if (fileName == null) return "";
        return fileName.substring(fileName.lastIndexOf(File.separator) +1);
    }
    
    private String fileExtPattern;
    private String fileDescription;
    private String filePath;
    private GUIChangeListener changeListener;
    /**
     * Creates new form FileNamePanel
     */
    public FileNamePanel(String fileDescription, String filePath, String pattern, GUIChangeListener changeListener) {
        this.fileDescription = fileDescription;
        this.filePath = filePath;
        this.fileExtPattern = pattern;
        this.changeListener = changeListener;
        if (filePath == null) filePath = "?";
        initComponents();
        chooserButton.setText(fileDescription + " : " + basename(filePath));
        chooserButton.setToolTipText(filePath);
        chooserButton.setUI(new BasicButtonUI());
    }
    
    /**
     * Returns the current filePath.
     */
    public String getFilePath() { return filePath; }
    
    /**
     * Change the filePath.
     */
    public void setFilePath(String filePath) {
        // replace all back slashes with forward slashes to permit
        // safe writing and reading from PrintStreams
        this.filePath = filePath.replace('\\', '/').trim();

	Debug.message("basic", "Setting file path to: " + filePath);
        chooserButton.setText(fileDescription + " : " + basename(filePath));
        chooserButton.setToolTipText(filePath);
    }
    
    private void initComponents() {
        chooserButton = new javax.swing.JButton();

        chooserButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/fr/unicaen/openmap/data/images/folder.png")));
        chooserButton.setText("Description : File Name");
        chooserButton.setToolTipText("Full Path");
        chooserButton.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        chooserButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chooserButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(chooserButton, javax.swing.GroupLayout.DEFAULT_SIZE, 242, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(chooserButton, javax.swing.GroupLayout.DEFAULT_SIZE, 36, Short.MAX_VALUE)
        );
    }
    
    public static class FF extends FileFilter {
        private Pattern pattern;
        private String desc;
        public FF(String patternString, String desc) {
            this.desc = desc;
            pattern = Pattern.compile(patternString);
        }
        public String getDescription() { return desc; }
        public boolean accept(File f) {
            if (f.isDirectory()) return true;
            Matcher matcher = pattern.matcher(f.getName());
            return matcher.matches();
        }
    }
    
    private void chooserButtonActionPerformed(java.awt.event.ActionEvent evt) {
        String _file = ThemaMapFileChooserManager.dataLoadDialog(fileDescription, new FF(fileExtPattern, fileDescription));
        if (_file == null || "".equals(_file)) return;
	setFilePath(_file);
        changeListener.reloadFromGUI(new GUIChangeEvent(this));
    }
    
    private javax.swing.JButton chooserButton;

    public void setDescription(String description) {
	fileDescription = description;
        chooserButton.setText(fileDescription + " : " + basename(filePath));
    }    

    public void setPattern(String pattern) {
	fileExtPattern = pattern;
    }
}
