// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.util.Properties;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;

import com.bbn.openmap.Layer;
import com.bbn.openmap.gui.LayerAddPanel;
import com.bbn.openmap.gui.LayersPanel;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropUtils;

import fr.unicaen.openmap.layer.chartLayer.ChartLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import javax.swing.JLabel;

public class LayerControlButtonPanel extends com.bbn.openmap.gui.LayerControlButtonPanel {

    protected LayerLoadPanel layerLoadPanel;
    static ImageIcon upgif = Util.loadIcon("moveUp.png");
    protected JButton load = null;
    protected boolean loadLayers = true;
    private Thematic selectedThematic = null;
    private Dimension buttonsDim = new Dimension(30, 30);

    protected void createInterface() {
        removeAll();

        setAlignmentX(LEFT_ALIGNMENT);
        setAlignmentY(CENTER_ALIGNMENT);
        //setLayout(new BoxLayout(this, orientation));

        top = new JButton(topgif);
        top.setActionCommand(LayersPanel.LayerTopCmd);
        top.setPressedIcon(topclickedgif);
        top.setToolTipText(i18n.get(LayerControlButtonPanel.class,
                "moveLayerToTop",
                "Move selected layer to top"));
        top.addActionListener(this);
        add(top);

        up = new JButton(Util.loadIcon("moveup.png"));
        up.setActionCommand(LayersPanel.LayerUpCmd);
        //up.setPressedIcon(upclickedgif);
        up.setToolTipText(i18n.get(LayerControlButtonPanel.class,
                "moveLayerUpOne",
                "Move selected layer up one"));
        up.addActionListener(this);
        add(up);

        down = new JButton(Util.loadIcon("movedown.png"));
        //down.setPressedIcon(downclickedgif);
        down.setActionCommand(LayersPanel.LayerDownCmd);
        down.setToolTipText(i18n.get(LayerControlButtonPanel.class,
                "moveLayerDownOne",
                "Move selected layer down one"));
        down.addActionListener(this);
        add(down);
        //ComponentSizer.fixComponentSize(down, buttonsDim);

        bottom = new JButton(bottomgif);
        bottom.setPressedIcon(bottomclickedgif);
        bottom.setActionCommand(LayersPanel.LayerBottomCmd);
        bottom.setToolTipText(i18n.get(LayerControlButtonPanel.class,
                "moveLayerToBottom",
                "Move selected layer to bottom"));
        bottom.addActionListener(this);
        add(bottom);

        if (deleteLayers) {
            JLabel blank = new JLabel(" ");
            add(blank);

            delete = new JButton(deletegif);
            delete.setActionCommand(LayersPanel.LayerRemoveCmd);
            delete.setToolTipText(i18n.get(LayerControlButtonPanel.class,
                    "removeLayer",
                    "Remove selected layer"));
            delete.addActionListener(this);
            delete.setEnabled(false);
            add(delete);
            ComponentSizer.fixComponentSize(delete, buttonsDim);
        }

        if (addLayers && add != null) {
            add(add);
        }
        //super.createInterface();
        //down.setPreferredSize(new Dimension(20, 20));
        //up.setIcon(Util.loadIcon("moveUp.png"));
        //up.setPreferredSize(new Dimension(20, 20));
        ComponentSizer.fixComponentSize(up, buttonsDim);
        ComponentSizer.fixComponentSize(down, buttonsDim);
        /*add.setPreferredSize(new Dimension(20, 20));*/
        // delete.setPreferredSize(new Dimension(20, 20));
        if (add != null) {
            ComponentSizer.fixComponentSize(add, buttonsDim);
        }
        remove(top);
        remove(bottom);
    }

    public void actionPerformed(java.awt.event.ActionEvent e) {
        String command = e.getActionCommand();
        if (Debug.debugging("layerbuttons")) {
            Debug.output("LayersPanel.actionPerformed(): " + command);
        }
        if (command == LayersPanel.LayerTopCmd ||
                command == LayersPanel.LayerBottomCmd ||
                command == LayersPanel.LayerUpCmd ||
                command == LayersPanel.LayerDownCmd ||
                (command == LayersPanel.LayerRemoveCmd &&
                JOptionPane.showConfirmDialog(
                this,
                i18n.get(this, "confirmDelete", "Delete the layer") + " " + selected.getName() + "?",
                i18n.get(this, "deletion", "deletion"),
                JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)) {
            if (selected != null) {
                if (Debug.debugging("layercontrol")) {
                    Debug.output("LayerControlButtonPanel: button firing " + command + " event for " + selected.getName());
                }
                firePropertyChange(command, null, selected);

            } else {
                if (selectedThematic != null) {
                    firePropertyChange(command, null, selectedThematic);
                } else {
                    if (Debug.debugging("layercontrol")) {
                        Debug.output("LayerControlButtonPanel: button firing " + command + " event with no layer selected");
                    }
                }
            }
        } else if (command.equals(LayersPanel.LayerAddCmd)) {
            if (layerAddPanel != null) {
                layerAddPanel.showPanel();
            }
        }
    }

    public void findAndInit(Object someObj) {

        if (someObj instanceof LayerLoadPanel) {
            setLayerLoadPanel((LayerLoadPanel) someObj);
            return;
        }
        if (someObj instanceof LayerAddPanel) {
            setLayerAddPanel((LayerAddPanel) someObj);
        }

        if (someObj instanceof LayersPanel) {
            setLayersPanel((LayersPanel) someObj);
        }

        if (someObj instanceof ThemaMapPanelsManager) {
            addPropertyChangeListener((ThemaMapPanelsManager) someObj);
        }

    }

    public void setLayerLoadPanel(LayerLoadPanel llp) {
        layerLoadPanel = llp;

        if (layerLoadPanel != null) {
            load = new JButton(Util.loadIcon("loadLayer.png"));
            ComponentSizer.fixComponentSize(load, buttonsDim);
            load.setToolTipText(i18n.get(LayerControlButtonPanel.class,
                    "loadLayer",
                    "Load layer(s)"));
            load.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent arg0) {
                    if (layerLoadPanel != null) {
                        layerLoadPanel.showPanel();
                    }
                }
            });
            if (loadLayers) {
                this.add(load);
            }
        } else if (load != null) {
            this.remove(load);
        }

    }

    /**
     * Set the panel that brings up an interface to dynamically add
     * layers.
     */
    public void setLayerAddPanel(LayerAddPanel lap) {
        layerAddPanel = lap;

        if (layerAddPanel != null) {
            add = new JButton(Util.loadIcon("addThema.png"));
            add.setActionCommand(LayersPanel.LayerAddCmd);
            add.setPreferredSize(new Dimension(20, 20));
            add.setToolTipText(i18n.get(LayerControlButtonPanel.class,
                    "addLayer",
                    "Add a layer"));
            add.addActionListener(this);
            if (addLayers) {
                this.add(add, getComponentCount() - 1);
                ComponentSizer.fixComponentSize(add, buttonsDim);
            }
        } else if (add != null) {
            this.remove(add);
        }

    }

    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        String command = pce.getPropertyName();
        Object obj = pce.getNewValue();
        if (Debug.debugging("layercontrol")) {
            Debug.output("LayerControlButtonPanel: receiving PropertyChangeEvent " + pce.getPropertyName());
        }
        if (command == LayersPanelTree.ThematicSelectedCmd && obj instanceof Thematic) {
            selectedThematic = (Thematic) obj;
            selected = null;
            delete.setEnabled(false);
        }
        if (command == LayersPanel.LayerSelectedCmd && obj instanceof Layer) {
            if(selected != null && selected instanceof ChartLayer)//unactiv last selected chartLayer
                ((ChartLayer)selected).setActive(false);
            selected = (Layer) obj;
            selectedThematic = null;
            delete.setEnabled(selected.isRemovable());

            if (Debug.debugging("layercontrol")) {
                Debug.output("LayerControlButtonPanel: got notification that layer is selected: " + selected.getName());
            }
            if(selected instanceof ChartLayer)
                ((ChartLayer)obj).setActive(true);
        } else if (command == LayersPanel.LayerDeselectedCmd) {
            selected = null;
            delete.setEnabled(false);
        }
    }
    public final static String loadLayersProperty = "load";

    public void setProperties(String prefix, Properties props) {
        super.setProperties(prefix, props);
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        loadLayers = PropUtils.booleanFromProperties(props, prefix + loadLayersProperty, loadLayers);
    }

    public Properties getProperties(Properties props) {
        props = super.getProperties(props);
        String prefix = PropUtils.getScopedPropertyPrefix(this);
        props.put(prefix + loadLayersProperty, new Boolean(loadLayers).toString());
        return props;
    }
}
