// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui;

import com.bbn.openmap.MapHandler;
import com.bbn.openmap.gui.WindowSupport;

import fr.unicaen.openmap.event.GUIChangeEvent;
import fr.unicaen.openmap.event.GUIChangeListener;
import fr.unicaen.openmap.layer.utils.LayerLoader;
import fr.unicaen.openmap.util.TooManyLegendException;
import fr.unicaen.openmap.util.propertyEditor.FileTypeDescription;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
/**
 * 
 * @author jchauvea
 *
 */
public class LayerLoadPanel extends com.bbn.openmap.gui.LayerAddPanel/*OMComponentPanel*/ implements ActionListener,
	GUIChangeListener{

	private FileNamePanel fileNamePanel;
	private JPanel panLayers;
	private ArrayList<String> selectedLayers;
	private LayerLoader layerLoader;

	
	public LayerLoadPanel(){
		super();
		selectedLayers = new ArrayList<String>();
	}
	@Override
	public void actionPerformed(ActionEvent e) {
		 if (e.getActionCommand() == "load"){
			 showPanel();
		 }
		 else if(e.getActionCommand() == "loadProperties"){
			 try {
				 File f = new File(fileNamePanel.getFilePath());
				 layerLoader = new LayerLoader(f.toURI().toURL(), layerHandler);
				 String[] layers = layerLoader.findAvailableLayers();
                                 if(layers == null || layers.length == 0){
                                    JOptionPane.showMessageDialog(null, "Aucune couche dans ce fichier", "Erreur", JOptionPane.ERROR_MESSAGE);
                                    return;
                                 }
				 panLayers.removeAll();
				 panLayers.setLayout(new GridLayout(layers.length,1));
				 for(int i = 0; i<layers.length; i++){
					 final JCheckBox cb = new JCheckBox(layers[i]);
					 cb.addItemListener(new ItemListener(){
						public void itemStateChanged(ItemEvent evt) {
							if(cb.isSelected())
								selectedLayers.add(cb.getText());
							else
								selectedLayers.remove(cb.getText());
						}});
					 panLayers.add(cb);
				 }
				 panLayers.setVisible(false);
				 panLayers.setVisible(true);
			} catch (MalformedURLException e1) {
				e1.printStackTrace();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		 }
		 else if(e.getActionCommand() == "import"){
                     try{
			 for(String selection: selectedLayers){
				 layerLoader.initLayer(selection);
			 }
                     }
                     catch(TooManyLegendException ex){
                        JOptionPane.showMessageDialog(this, ex.getMessage(),"",JOptionPane.ERROR_MESSAGE);
                     }
                         selectedLayers.clear();
			 windowSupport.killWindow();
		 }		
	}
	
	@Override
    public void showPanel() {
        createPanel();

        WindowSupport ws = getWindowSupport();

        MapHandler mh = (MapHandler) getBeanContext();
        Frame frame = null;
        if (mh != null) {
            frame = (Frame) mh.get(java.awt.Frame.class);
        }

        if (ws == null) {
            ws = new WindowSupport(this, i18n.get(LayerLoadPanel.class,
                    "title",
                    "Load Layer(s)"));
            setWindowSupport(ws);
        }

        ws.displayInWindow(frame, -1, -1, 480, 250);
    }
    /**
     * Produces a dialog panel to add a layer. If the layers haven't been
     * manually added through createPanel(layers), then the PropertyHandler is
     * consulted and the layer list is built from the layerTypes property.
     */
	@Override
    public void createPanel() {
        removeAll();
        Box wrap = Box.createVerticalBox();
        JLabel labFile = new JLabel(i18n.get(this, "loadfrom", "Load from"));
        fileNamePanel = new FileNamePanel(FileTypeDescription.PROPERTIES_DESCRIPTION, "", FileTypeDescription.PROPERTIES_PATTERN, this);
        JButton bLoad = new JButton(i18n.get(this, "load", "Load"));
        bLoad.setToolTipText(i18n.get(this, "loadtooltip", "Load data from file"));
        JButton bImport = new JButton(i18n.get(this, "import", "Import"));
        bImport.setToolTipText(i18n.get(this, "importtooltip", "Import selected layer(s)"));
        bLoad.setActionCommand("loadProperties");
        bLoad.addActionListener(this);
        bImport.setActionCommand("import");
        bImport.addActionListener(this);
        JPanel panFile = new JPanel();
        panFile.add(labFile);
        panFile.add(fileNamePanel);
        panFile.add(bLoad);
        JPanel panButton = new JPanel();
        panButton.add(bImport);
        panLayers = new JPanel();
        panLayers.add(new JLabel("Aucune couche chargée"));
        JScrollPane scrollPane = new JScrollPane(panLayers);
        scrollPane.setPreferredSize(new Dimension(400,120));
        scrollPane.setBorder(BorderFactory.createTitledBorder("Couches chargées"));
        wrap.add(panFile);
        wrap.add(scrollPane);
        wrap.add(panButton);
        add(wrap);
  
    }


	public void reloadFromGUI(GUIChangeEvent event) {
		// TODO Auto-generated method stub
		
	}

}
