package fr.unicaen.openmap.gui;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import com.bbn.openmap.event.LayerEvent;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;

import fr.unicaen.openmap.event.LayerListener;
import fr.unicaen.openmap.gui.dnd.LayersTreeTransferHandler;
import fr.unicaen.openmap.layer.legendLayer.LegendLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicDataDisplayLayerPopupMenu;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicPopupMenu;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import java.util.Enumeration;
import java.util.HashMap;
import javax.swing.DropMode;
import javax.swing.tree.TreeSelectionModel;

public class LayersTree extends JTree implements TreeSelectionListener,
        ThematicListener, MouseListener, LayerListener{

    protected LayerHandler layerHandler;
    private JComponent layerGUIPanel;
    protected DefaultMutableTreeNode root;
    protected boolean layersLoaded = false;
    protected HashMap<DefaultMutableTreeNode, LayerPopupMenu> popups;
    //private DefaultMutableTreeNode root;
    protected DefaultMutableTreeNode lastThematicDataDisplayLayerNodeSelected;
    protected I18n i18n = Environment.getI18n();
    protected int nLayers = 0;

    public LayersTree(DefaultMutableTreeNode root,
            LayerHandler lh, JComponent comp) {
        super(root);
        this.root = root;
        setEditable(true);
        LayerCellRenderer renderer = new LayerCellRenderer();
        setCellRenderer(new LayerCellRenderer());
        setCellEditor(new LayerCellEditor(this, renderer));
        setDragEnabled(true);
        setDropMode(DropMode.ON_OR_INSERT);
        setTransferHandler(new LayersTreeTransferHandler());
        getSelectionModel().setSelectionMode(
                TreeSelectionModel.CONTIGUOUS_TREE_SELECTION);
        popups = new HashMap<DefaultMutableTreeNode, LayerPopupMenu>();
        addTreeSelectionListener(this);
        addMouseListener(this);
        layerHandler = lh;
        layerHandler.addLayerListener(this);
        layerGUIPanel = comp;
        //final DefaultTreeModel model = (DefaultTreeModel) getModel();

    }

    public LayersTree(DefaultTreeModel model, LayerHandler lh, JComponent comp) {
        super(model);
        layerGUIPanel = comp;
        layerHandler = lh;
        setEditable(true);
        LayerCellRenderer renderer = new LayerCellRenderer();
        setCellRenderer(new LayerCellRenderer());
        setCellEditor(new LayerCellEditor(this, renderer));
        setDragEnabled(true);
        setDropMode(DropMode.ON_OR_INSERT);
        setTransferHandler(new LayersTreeTransferHandler());
        getSelectionModel().setSelectionMode(
                TreeSelectionModel.CONTIGUOUS_TREE_SELECTION);
        popups = new HashMap<DefaultMutableTreeNode, LayerPopupMenu>();
        addTreeSelectionListener(this);
        addMouseListener(this);

    }

    protected void expandTree(DefaultMutableTreeNode rootNode) {

        Enumeration e = rootNode.breadthFirstEnumeration();
        while (e.hasMoreElements()) {
            DefaultMutableTreeNode node =
                    (DefaultMutableTreeNode) e.nextElement();
            if (node.isLeaf()) {
                continue;
            }
            int row = getRowForPath(new TreePath(node.getPath()));
            expandRow(row);
        }
    }

    protected void buildTreeForLayers(DefaultMutableTreeNode pnode, Layer[] layers) {

        for (Layer l : layers) {
            DefaultMutableTreeNode node = new DefaultMutableTreeNode(l);
            pnode.add(node);
            nLayers++;
            if (l instanceof ThematicDataDisplayLayer) {
                popups.put(node, new ThematicDataDisplayLayerPopupMenu(this,
                        (ThematicDataDisplayLayer) l));
                ThematicDataDisplayLayer tddl = (ThematicDataDisplayLayer) l;
                tddl.addThematicListener(this);
                for (int i = 0; i < tddl.getThematicArray().size(); i++) {
                    DefaultMutableTreeNode nodeThema = new DefaultMutableTreeNode(
                            tddl.getThematicArray().get(i));
                    node.add(nodeThema);
                    popups.put(nodeThema, new ThematicPopupMenu(this, tddl.getThematicArray().get(i).isVisible()));

                }
            } else {
                popups.put(node, new LayerPopupMenu(this, l.isVisible()));
            }
        }
    }

    /**
     * Add a node to the tree for a thematic
     * @param tv
     */
    @SuppressWarnings("unchecked")
    public void addThematicNode(Thematic tv) {

//        DefaultMutableTreeNode nodeThema = new DefaultMutableTreeNode(tv);
        DefaultMutableTreeNode nodeThema = new DefaultMutableTreeNode(tv);
        lastThematicDataDisplayLayerNodeSelected = getObjectNode(tv.getLayer(), root);
        /*
         * add new thematic node as the first child of the parent node
         */
        DefaultTreeModel model = (DefaultTreeModel) getModel();
        model.insertNodeInto(nodeThema, lastThematicDataDisplayLayerNodeSelected, 0);
        popups.put(nodeThema, new ThematicPopupMenu(this, tv.isVisible()));
        setSelectionPath(new TreePath(nodeThema.getPath()));
        updateUI();
        //model.removeNodeFromParent(fake);

    }

    public DefaultMutableTreeNode addLayerNode(Layer layer) {
        DefaultMutableTreeNode layerNode = new DefaultMutableTreeNode(layer);
        DefaultTreeModel model = (DefaultTreeModel) getModel();
        model.insertNodeInto(layerNode, root, 0);
        if (layer instanceof ThematicDataDisplayLayer) {
            ThematicDataDisplayLayer tddl = (ThematicDataDisplayLayer) layer;
            tddl.addThematicListener(this);
            for (Thematic tv : tddl.getThematicArray()) {
                addThematicNode(tv);
            }
        }
        if (layer instanceof ThematicDataDisplayLayer) {
            popups.put(layerNode, new ThematicDataDisplayLayerPopupMenu(this,
                    (ThematicDataDisplayLayer) layer));
        } else {
            popups.put(layerNode, new LayerPopupMenu(this, layer.isVisible()));
        }
        setSelectionPath(new TreePath(layerNode.getPath()));
        nLayers++;
        return layerNode;
    }

    public void removeSelectedLayer(Layer l) {
        DefaultMutableTreeNode n = getObjectNode(l, root);
        n.removeFromParent();
        nLayers--;
        layerHandler.removeLayer(l);//not clean, gui should not set the model
        updateUI();
    }

    public void removeLayer(Layer l) {
        DefaultMutableTreeNode n = getObjectNode(l, root);
        n.removeFromParent();
        nLayers--;
        updateUI();
    }

    public void removeLayers() {
        root.removeAllChildren();
        nLayers = 0;
        updateUI();
    }

    public DefaultMutableTreeNode getObjectNode(Object obj, DefaultMutableTreeNode node) {
        if (node.getUserObject() == obj) {
            return node;
        } else {
            for (int i = 0; i < node.getChildCount(); i++) {
                DefaultMutableTreeNode n = getObjectNode(obj, (DefaultMutableTreeNode) node.getChildAt(i));
                if (n != null) {
                    return n;
                }
            }
        }
        return null;

    }

    public boolean removeSelectedNode() {
        DefaultMutableTreeNode n = (DefaultMutableTreeNode) getLastSelectedPathComponent();
        if (n != null) {
            DefaultMutableTreeNode ps = n.getPreviousSibling();
            DefaultMutableTreeNode ns = n.getNextSibling();
            n.removeFromParent();
            if (ps != null) {
                setSelectionPath(new TreePath(ps.getPath()));
            } else {
                if (ns != null) {
                    setSelectionPath(new TreePath(ns.getPath()));
                }
            }
            if (n.getUserObject() instanceof Layer) {
                nLayers--;
            }
            if (nLayers == 0) {
                firePropertyChange(LayersPanelTree.LayerDeselectedCmd, null, n.getUserObject());
            }
            updateUI();
            return true;
        }
        return false;
    }

    public JComponent getLayerGUIPanel() {
        return layerGUIPanel;
    }

    public LayerHandler getLayerHandler() {
        return layerHandler;
    }

    public void setLayerGUIComp(JComponent cp) {
        layerGUIPanel = cp;
    }

    @Override
    public void valueChanged(TreeSelectionEvent e) {
//        try{
//            throw new Exception();
//        }
//        catch(Exception ex){ex.printStackTrace();}
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) getLastSelectedPathComponent();
        if (node != null) {
            String command = "";
            if (node.getUserObject() instanceof Layer) {
                if (node.getUserObject() instanceof ThematicDataDisplayLayer) {
                    lastThematicDataDisplayLayerNodeSelected = node;
                }
                command = LayersPanelTree.LayerSelectedCmd;
            } else {
                command = LayersPanelTree.ThematicSelectedCmd;
            }
            if (node != null) {
                showPalette(node.getUserObject());
                firePropertyChange(command, null, node.getUserObject());
            }
        }
        //updateUI();
    }

    /*public void moveNode(int indexFrom, int indexTo, DefaultMutableTreeNode node) {
    int diff = indexFrom - indexTo;
    int absDiff = Math.abs(diff);
    // System.out.println("from "+indexFrom +" - to "+ indexTo);
    for (int i = 0; i < absDiff; i++) {
    if (diff > 0) {
    moveUpNode(node);
    } else {
    moveDownNode(node);
    }
    }
    }*/
    public int  moveUpSelectedNode() {
        return moveUpNode((DefaultMutableTreeNode) getLastSelectedPathComponent());
    }

    public int moveDownSelectedNode() {
        return moveDownNode((DefaultMutableTreeNode) getLastSelectedPathComponent());
    }

    public int moveUpNode(DefaultMutableTreeNode node) {
        DefaultMutableTreeNode parent = (DefaultMutableTreeNode) node.getParent();       
        int i = parent.getIndex(node);
        //int newIndex = parent.getIndex(node.getPreviousSibling());
        //System.out.println("i = " + i);
        //System.out.println("previous sible = " + newIndex);
        if (i < 1)// already at the top level
        {
            return -1;
        }
        moveGraphics(node.getUserObject(), i, -1);
        return parent.getIndex(node.getPreviousNode());
     
    }

    public boolean moveGraphics(Object obj, int index, int offset) {

        if (obj instanceof Thematic) {
            Thematic thema = (Thematic) obj;
            if (offset == -1) //move up
            {
                thema.getLayer().moveUpThematicVisu(index);
            } else //move down
            {
                thema.getLayer().moveDownThematicVisu(index);
            }
            layerHandler.setLayers();
            return true;
        } else {
            if (obj instanceof Layer) {
                Layer l = layerHandler.getLayers()[index + offset];
                if (!(l instanceof LegendLayer) && !(obj instanceof LegendLayer))// Legend always at the top
                // level
                {
                    layerHandler.moveLayer((Layer) obj, index + offset);
                    return true;
                } else {
                    JOptionPane.showMessageDialog(this,
                            i18n.get(LayersTree.class, "cantmove", "Cant move a layer above a legend"),
                            i18n.get(LayersTree.class, "notallowed", "Operation ot allowed"),
                            JOptionPane.ERROR_MESSAGE);
                    return false;
                }
            }
        }
        return false;
    }

//    private void setSelectedNode(int index){
//        setSelectionRow(index);
//    }

    public int moveDownNode(DefaultMutableTreeNode node) {

        DefaultTreeModel model = (DefaultTreeModel) getModel();
        DefaultMutableTreeNode parent = (DefaultMutableTreeNode) node.getParent();
        int i = parent.getIndex(node);
        if (i >= parent.getChildCount() - 1) {
            return -1;
        }

        // move layers/visualisations
        moveGraphics(node.getUserObject(), i, 1);
        return i;
//
//            // update tree
//            model.removeNodeFromParent(node);
//            model.insertNodeInto(node, parent, i + 1);
//            model.reload();
//            expandPath(new TreePath(parent.getPath()));
//            TreePath tp = new TreePath(node.getPath());
//            expandPath(tp);
//            setSelectionPath(tp);
//            updateUI();
//        }
    }

    public void removeLayerGUI(Layer l) {
        layerGUIPanel.remove(l.getGUI());
        layerGUIPanel.updateUI();

    }

    protected void showPalette(Object obj) {
        if (layerGUIPanel != null) {
            layerGUIPanel.setVisible(false);
            layerGUIPanel.removeAll();
            if (obj instanceof ThematicDataDisplayLayer || obj instanceof LegendLayer) {
                Component gui = ((Layer) obj).getGUI();
                if (gui == null) {
                    return;
                }
                layerGUIPanel.add(gui, BorderLayout.CENTER);
            } else {
                if (obj instanceof Layer) {
                    Component gui = ((Layer) obj).getGUI();
                    if (gui != null) {
                        layerGUIPanel.add(gui,
                                BorderLayout.NORTH);
                    }
                } else {
                    if (obj instanceof Thematic) {
                        Component gui = ((Thematic) obj).getLayer().getGUI();
                        if (gui == null) {
                            return;
                        }
                        layerGUIPanel.add(gui, BorderLayout.CENTER);
                        firePropertyChange(LayersPanelTree.ThematicSelectedCmd,
                                null, (Thematic) obj);
                        ((Thematic) obj).getLayer().getThematicPalette().showCard(((Thematic) obj).getName());
                    }

                }
            }
            layerGUIPanel.setVisible(true);
            layerGUIPanel.updateUI();
        }

    }

    @Override
    public void thematicAdded(Thematic thema) {
        Layer layer = thema.getLayer();
        ((ThematicDataDisplayLayer) layer).getThematicPalette().addThematicPane(thema, (ThematicDataDisplayLayer) layer, this);
        addThematicNode(thema);
    }

    @Override
    public void thematicChanged(Thematic thema) {
    }

    @Override
    public void thematicRemoved(Thematic thema) {
            if (ThemaMapPanelsManager.MODE == ThemaMapPanelsManager.MODE.CLASSIC) {
                removeSelectedNode();
            } else {
                //System.out.println("THEMATIC REMOVED = " + thema);
                DefaultMutableTreeNode n = getObjectNode(thema, root);
                n.removeFromParent();
                updateUI();
            }
    }

    @Override
    public void mouseClicked(MouseEvent arg0) {
        // TODO Auto-generated method stub
    }

    @Override
    public void mouseEntered(MouseEvent arg0) {
        // TODO Auto-generated method stub
    }

    @Override
    public void mouseExited(MouseEvent arg0) {
        // TODO Auto-generated method stub
    }

    @Override
    public void mousePressed(MouseEvent evt) {
        if (SwingUtilities.isRightMouseButton(evt)) {
            int selRow = getRowForLocation(evt.getX(), evt.getY());
            //unselect selected rows
            if (getSelectionCount() > 0) {
                for (int i : getSelectionRows()) {
                    removeSelectionRow(i);
                }
            }
            //select clicked one
            addSelectionRow(selRow);
            //updateUI();
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) getLastSelectedPathComponent();
            if (popups.get(node) != null) {
                popups.get(node).show(this, evt.getX(), evt.getY(), node);
            }
        }
    }

    @Override
    public void mouseReleased(MouseEvent arg0) {
        // TODO Auto-generated method stub
    }

    public LayerHandler getHandlerLayer() {
        return layerHandler;
    }

    public int getNLayers() {
        return nLayers;
    }

    public DefaultMutableTreeNode getRootNode(){return root;}

    public void setLayers(LayerEvent le) {
        if(layerHandler.getLayers().length == 0)
            removeLayers();//clear panel
    }

    public void layerUpdate() {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    public void titleUpdate(String title) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    public void layerRemoved(Layer l) {
        removeLayer(l);
        removeLayerGUI(l);
    }
}
