// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.gui.dnd;

import java.awt.Component;
import java.awt.LayoutManager;
import java.util.ArrayList;


import javax.swing.Box;

import fr.unicaen.openmap.gui.component.ThemaMapSplitPane;

/**
 * 
 * @author jchauvea
 *
 */
public class MovableBoxesContainer extends Box {

    protected ArrayList<Component> elements;
    protected ArrayList<String> boxesList;
    protected int type;
    protected int nbBox;
    protected int nbDrop;
    protected LayoutManager defaultLayout;

    public MovableBoxesContainer(int arg0) {
        super(arg0);
        this.type = arg0;
        elements = new ArrayList<Component>();
        boxesList = new ArrayList<String>();
        nbBox = 0;
        nbDrop = 0;
        defaultLayout = getLayout();
    }

    /**
     * Add a component to this container at the index
     * @param c
     * @param index
     */
    public void addElement(Component c, int index) {
        elements.add(index, c);
        if (c instanceof MovableBox) {
            nbBox++;
            ((MovableBox) c).setParentBoxContainer(this);
            boxesList.add(((MovableBox) c).getPropertyPrefix());
        } else {
            if (c instanceof DropArea) {
                //dropAreas[nbDrop] = (DropArea)c;
                nbDrop++;
            }
        }
        showElements();
    }

    /**
     * Add a component to this container
     * @param c
     */
    public void addElement(Component c) {
        elements.add(c);
        if (c instanceof MovableBox) {
            nbBox++;
            ((MovableBox) c).setParentBoxContainer(this);
            boxesList.add(((MovableBox) c).getPropertyPrefix());
        } else {
            if (c instanceof DropArea) {
                //dropAreas[nbDrop] = (DropArea)c;
                nbDrop++;
            }
        }

        showElements();
    }

    /**
     * Remove component element
     * @param c
     */
    public void removeElement(Component c) {
        if (elements.remove(c)) {
            if (c instanceof MovableBox) {
                nbBox--;
                boxesList.remove(((MovableBox) c).getPropertyPrefix());
            } else if (c instanceof DropArea) {
                nbDrop--;
            }
        }
        if (nbBox <= 0) {
            showOnlyOneDropArea();
        }


    }

    /**
     * Show only one drop area(reception area) when severals
     */
    public void showOnlyOneDropArea() {
        for (Component c : elements) {
            c.setVisible(false);
        }
        if (!elements.isEmpty()) {
            elements.get(0).setVisible(true);
        }
    }

    /**
     * Show all elements
     */
    public void showElements() {
        for (Component c : elements) {
            c.setVisible(true);
        }
    }

    /**
     * Retrun the index of the given component
     * @param c
     * @return
     */
    public int getElementIndex(Component c) {
        return elements.indexOf(c);
    }

    /**
     * Update/Refresh the GUI
     */
    public void updateGUI() {
        removeAll();
        for (Component c : elements) {
            add(c);
        }
        updateUI();
        setVisible(false);
        setVisible(true);
    }

    /**
     * Close all elements of the component
     */
    public void close() {
        for (Component c : elements) {
            if (c instanceof MovableBox) {
                ((MovableBox) c).close();
            } else {
                if (c instanceof MovableBoxesContainer) {
                    ((MovableBoxesContainer) c).close();
                }
            }
        }
    }

    /**
     * Remove all elements from the component
     */
    public void removeAllElements() {
        elements.clear();
        boxesList.clear();
        nbDrop = 0;
        nbBox = 0;
    }

    public int getType() {
        return type;
    }

    public void setType(int type) {
        this.type = type;
    }

    public boolean isEmpty() {
        return elements.isEmpty();
    }

    public boolean containsBoxes() {
        return nbBox > 0;
    }

    public int getNumElements() {
        return elements.size();
    }

    public int getNbBox() {
        return nbBox;
    }

    public void setNbBox(int nbBox) {
        this.nbBox = nbBox;
    }

    @Override
    public void setVisible(boolean b) {
        super.setVisible(b);
        if (getParent() instanceof ThemaMapSplitPane) {
            ((ThemaMapSplitPane) getParent()).hasChanged();
        }
    }

    public ArrayList<String> getBoxesList() {
        return boxesList;
    }
}
