// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.dataDisplayLayer.gui;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.Debug;
import fr.unicaen.openmap.util.Util;
import java.awt.*;
import java.util.ArrayList;
import java.util.Locale;
import javax.swing.*;
import javax.swing.plaf.metal.MetalTabbedPaneUI;


/**
 * This class is used to display a basic palette.
 * First  : Configuration of the layer and DataStorages ...
 * Second : Data Gui ...
 * Third  : Color layer properties.Color layer properties.
 */

public class SimplePalette extends JPanel {
    protected SimplePaletteTabbedPane tabs = null;
    protected JToolBar toolBar = null;

    
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    protected I18n i18n = new BasicI18n(new Locale("fr","FR"));
    /**
     * A model of empty palette (with empty JTabbedPane). Use to create your palette as you like.
     */
    public SimplePalette() {
        super();
        setDoubleBuffered(true);
        setLayout(new BorderLayout());
        tabs = new SimplePaletteTabbedPane();
        add(tabs, BorderLayout.CENTER);
        toolBar = new JToolBar();
        add(toolBar, BorderLayout.NORTH);
    }
    
    /**
     * Creates a palette with two tools : request panel and graphics panel.
     * @param left JComponent layed in first
     * @param graphicsPanel JComponent to have colors choice
     */
    public SimplePalette(Component left, GraphicsPanel graphicsPanel) {
        this();
        if (left !=null) tabs.addTab(i18n.get(this,"Configuration","Configuration"), null, left);
        tabs.addTab(i18n.get(this,"Coloration","Coloration"), null, graphicsPanel);
    }
    
    /**
     * Creates a palette with two tools : request panel and graphics panel.
     * @param left Component layed in first
     * @param graphicsPanel Component to have colors choice
     */
    public SimplePalette(Component left, GraphicsPanel graphicsPanel, JComponent storagePanel) {
        this();
        tabs.addTab(i18n.get(this,"Data","Data"),null,storagePanel);
        if(graphicsPanel!=null) tabs.addTab(i18n.get(this,"Coloration","Coloration"), null, graphicsPanel);
        if (left!=null) tabs.addTab(i18n.get(this,"Configuration","Configuration"), null, left);
    }
    
    public void addToToolBar(Component button){
        try {
            Debug.message(Util.className(this), this + " addButtonToToolBar()");
            toolBar.add(button);
        } catch (Exception e){
            Debug.error("===> [ "+Util.className(this)+" ] :  Error "+e.toString());
            e.printStackTrace();
        }
    }
       
    public SimplePaletteTabbedPane getTabs(){return tabs; }
    
    public static void customTabbedPaneUI(JTabbedPane tabbed,final int index){
    	tabbed.setUI(new MetalTabbedPaneUI(){
    		protected void paintTabBorder(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected){
    			if(tabIndex == index){
    				g.setColor(new Color(100,100,100));
    				g.drawLine(x,y,(x),y+h);
    			}
    			else{
    				super.paintTabBorder(g, tabPlacement, tabIndex, x, y, w, h, isSelected);
    			}
    		}
    		protected void paintTabBackground(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected) {
    			if(tabIndex != index)
    				super.paintTabBackground(g, tabPlacement, tabIndex, x, y, w, h, isSelected);
    		}

    });
    
    }


    public static void customTabbedPaneUI(JTabbedPane tabbed,final ArrayList<Integer> indices, final int withRightBorderIndex){
    	tabbed.setUI(new MetalTabbedPaneUI(){

    		protected void paintTabBorder(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected){
    			if(!indices.contains(tabIndex)){
                            super.paintTabBorder(g, tabPlacement, tabIndex, x, y, w, h, isSelected);
    			}
    			else{
    				if(tabIndex == withRightBorderIndex){
                                    g.setColor(new Color(100,100,100));
                                    g.drawLine(x,y,(x),y+h);
                                }
    			}
    		}
    		protected void paintTabBackground(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected) {
    			if(!indices.contains(tabIndex))
    				super.paintTabBackground(g, tabPlacement, tabIndex, x, y, w, h, isSelected);
    		}

    });

    }
}
