// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

/*
 * @author Jacques.Madelaine@info.unicaen.fr
 * code adapted from a code developped at IRD by Nicolas Antoine
 */

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.*;

import java.util.*;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import com.bbn.openmap.omGraphics.OMColorChooser;

import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.util.classification.Classification;

import com.bbn.openmap.I18n;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.util.PropUtils;

public class ColorScalePanel2 extends JPanel implements ActionListener, ChangeListener {

    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     */
    protected static I18n i18n = new BasicI18n();

    /**
     * Default property name for color scale.
     */
    public static final String CUSTOM_PROPERTY_NAME = "custom";
    public static final String CRAYOLA8_PROPERTY_NAME = "crayola8";
    public static final String CRAYOLA10_PROPERTY_NAME = "crayola10";
    public static final String CRAYOLARED_PROPERTY_NAME = "crayolared";
    public static final String CRAYOLABLUE_PROPERTY_NAME = "crayolablue";
    public static final String PASTEL_PROPERTY_NAME = "pastel";
    public static final String WATER_PROPERTY_NAME = "water";
    public static final String SAND_PROPERTY_NAME = "sand";
    protected Border border;

    public static final String CAMOUFLAGE_PROPERTY_NAME = "camouflage";
    public static final String WARM_PROPERTY_NAME = "warm";
    public static final String DESERT1_PROPERTY_NAME = "desert1";
    public static final String DESERT2_PROPERTY_NAME = "desert2";
    public static final String FOREST_PROPERTY_NAME = "forest";
    public static final String COLD_PROPERTY_NAME = "cold";
    public static final String FRUIT_PROPERTY_NAME = "fruit";
    public static final String BLUE_GREEN_PROPERTY_NAME = "blueGreen";
    public static final String FLESH_PROPERTY_NAME = "flesh";
    public static final String MINERAL_PROPERTY_NAME = "mineral";

    public static final String TITLE_PROPERTY_NAME = "title";
    public static final String SHADED_PROPERTY_NAME = "shaded";
    public static final String COLOR_CHOOSER_PROPERTY_NAME = "colorChooser";
    public static final String COLOR_SCALE_PROPERTY_NAME = "colorScale";

    /**
     * Default color scale in Hex.
     */
    public static final String crayola8InHex = 
	"ddb00000 ddeb5300 ddffdc00 dd125e14 dd111171 dd53006f dd653425 dd000000";
    public static final String crayola10InHex = 
	"ddb00000 ddeb3200 ddeb5300 ddff8400 ddffdc00 dd3e9300 dd125e14 dd004b45 dd111171 dd53006f";
    // Crayola 10 does not exists : it is a subjective subset of Crayola 16
    public static final String crayolaRedInHex =  // Crayola 16 (red subset)
	"ddffdc00 ddffba00 ddff9540 ddff8400 ddeb5300 ddeb3200 ddd00000 ddb00000 dd8b0038 dd8b0052";

    public static final String crayolaBlueInHex = // Crayola 16 (blue subset)
	"ddffdc00 ddbaff00 dd3e9300 dd125e14 dd004b45 dd121298 dd111171 dd34006f dd270053 dd53006f";

    public static final String waterColorScaleInHex = "cddffe7ba  dd66cdaa  dde0eeee  ddffebcd  dd8b3e2f  ddff34b3  dd8b668b  ddffc1c1  dd6ca6cd  dd00ff7f"; 
    public static final String sandColorScaleInHex = "dd8b7d6b  ddffe7ba  dd008b8b  dd006400  ddee7600  ddeec900  ddfff68f  ddffa07a";
    public static final String pastelColorScaleInHex = "ddffb300 ddedd3c1 dd74c476 dd80a6ce ddffff4d ddf4c6de dda6dba0 ddbfd1e5";
    public static final String camouflageColorScaleInHex = "523A1C 967E4D 76945F CABA89 F0EBE2 1C4761 4981A5 90B8D3 413A2C 224A1E";
    public static final String warmColorScaleInHex = "FADB00 FFCF00 F4B618 EFA500 EA7825 E5562C DF001C CB312F B12925 972520";
    public static final String desert1ColorScaleInHex = "E3DCD3 C0B1A1 AA9886 967E69 795F48 563E29 AFBBA0 8D9D7A 5E7646 315B27";
    public static final String desert2ColorScaleInHex = "EEE6D6 E1D4BC D3C09C C5A87B B68F62 A3774E 6F4919 EEEC00 FCC300 F29400";
    public static final String forestColorScaleInHex = "94A653 6B8F24 48651C 2C4311 B39A12 8E7C14 665A10 4A420B 60410B 432D07	";
    public static final String coldColorScaleInHex = "E5E9F4 C8D6EC 799FCF 4380BD 0E5FA7 004988 91D4F2 00ADE1 0096D9 0080BA";

    public static final String fruitColorScaleInHex = "DC3628 A47E58 E2873F D4002E 8CB03A 1D7951 9E0C46 DBE9C8 621F62 FDE077";
    public static final String blueGreenColorScaleInHex =    "E8ECCB B8C763 BDCB1E 3DA231 00883C 0095A8 4DC0E6 00A1D9 007DBA 1E3789";// "1E3789 007DBA 00A1D9 4DC0E6 0095A8 00883C 3DA231 BDCB1E B8C763 E8ECCB ";
 
    public static final String fleshColorScaleInHex = "F8E8D7 F3D9BE EECAA5 D4A87E B9937C 977663 7B6152 5B443B F2C8BE E5B6AC";
    public static final String mineralColorScaleInHex = "A12116 7A1114 7E6666 6C4A4D 544045 4F0B0A 889EA9 5D7784 3F5E65 173542";
 
    /**
     * Customized color scale in hex.
     */
    private String customColorScaleInHex = "ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff ddffffff";

    /**
     * Customized ColorScale.
     */
    protected ColorScale customColorScale = new ColorScale(customColorScaleInHex, i18n.get(ColorScalePanel2.class, CUSTOM_PROPERTY_NAME, CUSTOM_PROPERTY_NAME));

    /**
     * Current ColorScale.
     */
    protected ColorScale colorScale;

    protected String stringNumberFormat = "%5.2f";

    protected JButton colorButtons[];
    protected JComboBox colorCombos[];
    //    protected JButton refreshPanelB;
    protected JCheckBox shadedB;
    private boolean shaded = false;
    protected JSpinner colorNumber;
    protected JButton selectedButton;
    private final String predefinedColorMapText[] = {
	//	i18n.get(ColorScalePanel.class, CUSTOM_PROPERTY_NAME, CUSTOM_PROPERTY_NAME),
	customColorScale.toString(),
	i18n.get(ColorScalePanel2.class, CRAYOLA8_PROPERTY_NAME, CRAYOLA8_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, CRAYOLA10_PROPERTY_NAME, CRAYOLA10_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, CRAYOLARED_PROPERTY_NAME, CRAYOLARED_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, CRAYOLABLUE_PROPERTY_NAME, CRAYOLABLUE_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, PASTEL_PROPERTY_NAME, PASTEL_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, WATER_PROPERTY_NAME, WATER_PROPERTY_NAME ),
	i18n.get(ColorScalePanel2.class, SAND_PROPERTY_NAME, SAND_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, CAMOUFLAGE_PROPERTY_NAME, CAMOUFLAGE_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, WARM_PROPERTY_NAME, WARM_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, DESERT1_PROPERTY_NAME, DESERT1_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, DESERT2_PROPERTY_NAME, DESERT2_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, FOREST_PROPERTY_NAME, FOREST_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, COLD_PROPERTY_NAME, COLD_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, FRUIT_PROPERTY_NAME, FRUIT_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, BLUE_GREEN_PROPERTY_NAME, BLUE_GREEN_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, FLESH_PROPERTY_NAME, FLESH_PROPERTY_NAME),
	i18n.get(ColorScalePanel2.class, MINERAL_PROPERTY_NAME, MINERAL_PROPERTY_NAME)
    };

    private String predefinedColorMapInHex[] = {
	customColorScaleInHex,
	crayola8InHex,
	crayola10InHex,
	crayolaRedInHex,
	crayolaBlueInHex,
	pastelColorScaleInHex, 
	waterColorScaleInHex, 
	sandColorScaleInHex,
	camouflageColorScaleInHex,
	warmColorScaleInHex,
	desert1ColorScaleInHex,
	desert2ColorScaleInHex,
	forestColorScaleInHex,
	coldColorScaleInHex,
	fruitColorScaleInHex,
	blueGreenColorScaleInHex,
	fleshColorScaleInHex,
	mineralColorScaleInHex
    };

    protected JComboBox predefinedColorMapButtons = new JComboBox(predefinedColorMapText);
    
    public ColorScalePanel2(ColorScale colorScale) {
	this.colorScale = colorScale;


	customize();
    }
 
    /** Crude way to refresh: empty the panel and re-create the content! */
    public void refreshPanel() {
	setVisible(false);
	removeAll();
	createOptionPanel();
	setVisible(true);
	fireColorScaleChanged();
    }

    public void createOptionPanel() {
	//setBorder(BorderFactory.createTitledBorder(""/*i18n.get(this, TITLE_PROPERTY_NAME, "Color Scale")*/));
	//setPreferredSize(new Dimension(200, 200));
	setLayout(new GridLayout(1, 2));

	// A bunch of sub panels
	//JPanel colorPanel = new JPanel(new GridLayout(11, 1, 5, 5));
        Box colorPanel = Box.createVerticalBox();
	Box optionSetPanel = Box.createVerticalBox();
	colorNumber = new JSpinner(new SpinnerNumberModel(colorScale.getColorNumber(), 1, 10, 1));
	colorNumber.addChangeListener(this);	
	
	optionSetPanel.add(colorNumber);

	//	predefinedColorMapButtons = new JComboBox(predefinedColorMapText);
	ActionListener predefinedActionListener = new ActionListener() {
		// Predefined colormaps
		public void actionPerformed(ActionEvent ae) {

		    int index = ((JComboBox)ae.getSource()).getSelectedIndex();
		    if (index < 0) 
			return;
		    colorScale.initFromColorScaleInHex(predefinedColorMapInHex[index]);
		    shadedB.setSelected(shaded = false);
		    refreshColor();
		}
	    };
	predefinedColorMapButtons.addActionListener(predefinedActionListener);
	optionSetPanel.add(predefinedColorMapButtons);


	// Shaded colormap 
	shadedB = new JCheckBox(i18n.get(this, SHADED_PROPERTY_NAME, "Shaded"));
	shadedB.setSelected(shaded);
	shadedB.addActionListener(this);

	optionSetPanel.add(shadedB); 
	// refresh button: should be useless!!!
	// for testing, I keep this code...
	/*	constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.fill = GridBagConstraints.NONE;
		refreshPanelB = new JButton("Refresh Panel");
		refreshPanelB.addActionListener(this);
		refreshPanelButtonPanel.add(refreshPanelB, constraints);*/

	// Colormap buttons, one for each color
	colorButtons = new JButton[colorScale.getColorNumber()];
        colorCombos = new JComboBox[colorScale.getColorNumber()];
	// Left clic will show a color chooser
	MouseAdapter ma = new MouseAdapter() {

		public void mouseClicked(MouseEvent me) {
		    if ((me.getButton() == MouseEvent.BUTTON1)/* && (me.getClickCount() == 2)*/) {
			//		    if ((me.getButton() == MouseEvent.BUTTON3) && (me.getClickCount() == 1)) {
			for(int i = 0; i < colorButtons.length; i++) {
			    if (colorButtons[i] != null && me.getSource() == colorButtons[i]) {
			    	if(me.getClickCount() == 2)
			    	{
				    Color newButtonColor = OMColorChooser.showDialog(colorButtons[i], i18n.get(this, COLOR_CHOOSER_PROPERTY_NAME, "Choose a color..."), colorButtons[i].getBackground());
					if (newButtonColor != null) {
					    colorButtons[i].setBackground(newButtonColor);
                                            colorCombos[i].setBackground(newButtonColor);
					    colorScale.setColor(i, newButtonColor);
					    customize();
					    //		    refreshPanel();	    
					    return;
				    }
				}
			    	else{
			    		if(selectedButton == null){
			    			border = colorButtons[i].getBorder();
			    			colorButtons[i].setBorder(BorderFactory.createLineBorder(Color.black));
			    			selectedButton = colorButtons[i];
			    		}
			    		else{
			    		selectedButton.setBorder(border);
			    		colorButtons[i].setBorder(BorderFactory.createLineBorder(Color.black));
			    		selectedButton = colorButtons[i];
			    		}
						
					}
			    }
			}
		    }	    	

		}
	    };

	for(int i = 0 ; i < colorButtons.length ; i++) {
            JPanel p = new JPanel();
	    colorButtons[i] = new JButton();
	    colorButtons[i].addMouseListener(ma);
	    colorButtons[i].setToolTipText(i18n.get(this, COLOR_SCALE_PROPERTY_NAME, I18n.TOOLTIP, "button 1 to change color"));
	    colorButtons[i].setBackground(colorScale.getColor(i));
	    //	    colorButtons[i].setText("");
	    //colorButtons[i].setText(getColorButtonText(i));
            colorCombos[i] = new JComboBox();
            colorCombos[i].setBackground(colorScale.getColor(i));
            p.add(colorButtons[i]);
            p.add(colorCombos[i]);
            p.setBorder(BorderFactory.createEmptyBorder(-5,-5,-5,-5));
	    colorPanel.add(p);
	}

	//Box box = Box.createVerticalBox();
        JPanel pwrap = new JPanel();
        pwrap.add(optionSetPanel);
	add(pwrap);
	add(colorPanel);
	
    }

    protected String getColorButtonText(int i) {
	return " ";
    }    

    public void stateChanged(ChangeEvent e) {
	if(e.getSource() == colorNumber) { //We set the number of colors
	    int n = ((Integer) colorNumber.getValue()).intValue();
	    colorScale.setColorNumber(n);
	    refreshPanel();	    
	    return;
	}
    }
 
    public void actionPerformed(ActionEvent event) {
	if (event.getSource() == shadedB) { // Do shaded color map
	    if (shadedB.isSelected()) {
		shaded = true;
		colorScale.doShade();
	    } else {
		colorScale.copy(new ColorScale(predefinedColorMapInHex[predefinedColorMapButtons.getSelectedIndex()], ""));
		shaded = false;
	    }
	    refreshColor();
	} /*else if (event.getSource() == refreshPanelB) { // Should be useless!!!!
	    refreshPanel();	    
	    } 
	  */
    }
    
    /**
     * Just refresh color scale buttons.
     */ 
    public void refreshColor() {
	for(int i = 0 ; i < colorButtons.length ; i++) {
	    colorButtons[i].setBackground(colorScale.getColor(i));
            colorCombos[i].setBackground(colorScale.getColor(i));
	}
	fireColorScaleChanged();
    }

    /**
     * Used when a color is changed on color scale. Enable customized color scale.
     */
    private void customize() {
	customColorScale.copy(colorScale);
	customColorScaleInHex = customColorScale.getProperties("", new Properties()).getProperty(ColorScale.colorScalePropertyString);
	predefinedColorMapInHex[0] = customColorScaleInHex;
	predefinedColorMapButtons.setSelectedIndex(0);
	//	System.out.println(predefinedColorMapInHex[0]));
    }


    //working....
    //sur le modèle sympa de Colorizer pour la gestion d'événements
    //pourrait utiliser Object & faire directos une classe, mais moins sympa
    private ArrayList<ColorScaleListener> listenerList = new ArrayList<ColorScaleListener>();
    public void addColorScaleListener(ColorScaleListener l) { listenerList.add(l); }
    public void removeColorScaleListener(ColorScaleListener l) { listenerList.remove(l); }
    
    protected void fireColorScaleChanged() {
	//for (int i = 0; i < listenerList.size(); i++)
	    //listenerList.get(i).colorScaleChanged(this);
    }
}

