// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.event.ItemEvent;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ArrowHead;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMFlow;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Colorizer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Area;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Flow;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import java.awt.event.ItemListener;
import javax.swing.JScrollPane;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

/**
 * Flow visualization and management
 * 
 * @author Jerome Chauveau
 */
public class FlowPanel extends JPanel {

    private JPanel panFromTo, panStyle;
    private FlowOverviewPanel ovPanel;
    private JComboBox comboToCol, comboSort;
    private Flow flow;
    protected I18n i18n = new BasicI18n();
    static final int TYPE_START = 0;
    static final int TYPE_END = 1;
    ThematicGUI thematicGUI;
    protected JComboBox dataList;
    private ThematicConfPanel confp;
    //private JPanel cardPanel;
    private JPanel dataPanel;
    private ColorizerPanel colorizerPanel;
    private JCheckBox checkVisibility;
    private Box toBox;

    //private JLabel labIdentification;
    public FlowPanel(final Flow flow,
            final DataStorageIterator dsi,
            int startModel,
            int endModel,
            int axis,
            ThematicGUI tgui,
            Colorizer colorizer) {
        this.flow = flow;
        this.thematicGUI = tgui;
        setLayout(new BoxLayout(this, axis));
        initDataList(dsi);

        JPanel panType = new JPanel();

        final SpinnerNumberModel model =
                new SpinnerNumberModel(flow.getNbStep(), //initial value
                0, //min
                100, //max
                1);//step

        JSpinner stepSpinner = new JSpinner(model);
        panType.add(new JLabel("Nombres d'étapes"));
        panType.add(stepSpinner);
        stepSpinner.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent arg0) {
                int val = model.getNumber().intValue();
                flow.setNbStep(val);
                refreshToBox(dsi);
                flow.setChosenType(val == 0 ? Flow.type.PATH : Flow.type.FLOW);
            }
        });
        //panel for FROM/TO Columns choice

        panFromTo = new JPanel();


        //JPanel to = new JPanel();
        toBox = Box.createVerticalBox();
        refreshToBox(dsi);

        //panFromTo.setBorder(BorderFactory.createTitledBorder(i18n.get(this, "from_to", "From/To Columns")));
        panFromTo.add(toBox);

        //dep = arr
        final JCheckBox checkShowStartEqualsEnd = new JCheckBox("Afficher les boucles");
        final JSpinner spinnerDepArrRadius = new JSpinner();
        checkShowStartEqualsEnd.setSelected(flow.isShowStartEqualsEnd());
        spinnerDepArrRadius.setEnabled(flow.isShowStartEqualsEnd());
        spinnerDepArrRadius.setValue(flow.getLoopRadius());
        checkShowStartEqualsEnd.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent arg0) {
                flow.setShowStartEqualsEnd(checkShowStartEqualsEnd.isSelected());
                spinnerDepArrRadius.setEnabled(checkShowStartEqualsEnd.isSelected());
            }
        });
        spinnerDepArrRadius.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent arg0) {
                flow.setLoopRadius(Integer.parseInt(spinnerDepArrRadius.getValue().toString()));
            }
        });

        //panFromTo.add(checkShowStartEqualsEnd);
        JPanel panSort = new JPanel();
        comboSort = new JComboBox(dsi.getDataColumnNames());
        panSort.add(comboSort);
        ComponentSizer.fixComponentSize(comboSort, new Dimension(150, 25));
        /*themaConfSort = new ThematicConfigurationPanel(i18n.get(this,"sort","Sort column"),
        panSort, this);*/

        panStyle = new JPanel();
        Box leftStyleBox = Box.createVerticalBox();
        Box arrowBox = Box.createHorizontalBox();
        //panStyle.setLayout(new GridLayout(2, 1));
        panStyle.setBorder(BorderFactory.createTitledBorder(""));
        JPanel pModels = new JPanel(new GridLayout(2, 1));
        Box startBox = Box.createHorizontalBox();
        Box endBox = Box.createHorizontalBox();
        pModels.add(startBox);
        pModels.add(endBox);
        arrowBox.add(pModels);
        //JPanel p = new JPanel();
        //p.setLayout(new GridLayout(2, 1));
        startBox.add(new JLabel(i18n.get(this, "start_arrow",
                "Start")));
        startBox.add(comboBoxForArrowHeadModelChoice(startModel,
                TYPE_START));
        endBox.add(new JLabel(i18n.get(this, "end_arrow",
                "End")));
        endBox.add(comboBoxForArrowHeadModelChoice(endModel,
                TYPE_END));

        //overview panel
        ovPanel = new FlowOverviewPanel();
        ovPanel.setMaximumSize(new Dimension(120, 30));
        ovPanel.setPreferredSize(new Dimension(120, 30));
        ovPanel.setMinimumSize(new Dimension(120, 30));
        arrowBox.add(ovPanel);
        leftStyleBox.add(arrowBox);

        ButtonGroup bGroup = new ButtonGroup();

        final JRadioButton radioManual = new JRadioButton();
        bGroup.add(radioManual);
        radioManual.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioManual.isSelected()) {
                    flow.setControlPointsVisibility(true);
                    OMFlow.BENDED_MODE = OMFlow.BENDED_MANUAL_MODE;
                    flow.setBadCache(true);
                    checkVisibility.setEnabled(true);
                    checkVisibility.setSelected(true);
                }

            }
        });
        radioManual.setSelected(OMFlow.BENDED_MODE == OMFlow.BENDED_MANUAL_MODE);
        JPanel pl2 = new JPanel(new BorderLayout());
        pl2.add(radioManual, BorderLayout.WEST);
        pl2.add(new JLabel(i18n.get(this, "manual", "manual")));
        checkVisibility = new JCheckBox(i18n.get(this, "handles", "handles"));
        checkVisibility.setSelected(flow.isControlPointsVisibility());
        checkVisibility.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                flow.setControlPointsVisibility(checkVisibility.isSelected());
            }
        });
        pl2.add(checkVisibility, BorderLayout.EAST);
        leftStyleBox.add(pl2);
        //spinners.add(new JPanel());

        JPanel pl1 = new JPanel();
        pl1.setLayout(new BorderLayout());
        final JRadioButton radioAuto = new JRadioButton();

        radioAuto.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (radioAuto.isSelected()) {
                    flow.setControlPointsVisibility(false);
                    OMFlow.BENDED_MODE = OMFlow.BENDED_AUTO_MODE;
                    flow.setBadCache(true);
                    checkVisibility.setEnabled(false);
                }

            }
        });
        radioAuto.setSelected(OMFlow.BENDED_MODE == OMFlow.BENDED_AUTO_MODE);
        checkVisibility.setEnabled(OMFlow.BENDED_MODE == OMFlow.BENDED_MANUAL_MODE);
        bGroup.add(radioAuto);
        pl1.add(radioAuto, BorderLayout.WEST);
        pl1.add(new JLabel(i18n.get(this, "bended", "Bended size")));
        pl1.add(bendedSpinner(), BorderLayout.EAST);
        leftStyleBox.add(pl1);
        //JPanel pb = new JPanel();

        //spinners.add(pb);

        JPanel pl3 = new JPanel();
        pl3.add(new JLabel(i18n.get(this, "line_arc", "Arcs length") + " :"));
        pl3.add(nbLineSpinner());
        leftStyleBox.add(pl3);


        //Box wrapType = Box.createVerticalBox();
        Box wrapType = Box.createVerticalBox();
        wrapType.add(panType);
        wrapType.add(panFromTo);
        JPanel panLoopsAndRadius = new JPanel();
        wrapType.add(panLoopsAndRadius);
        panLoopsAndRadius.add(checkShowStartEqualsEnd);
        panLoopsAndRadius.add(new JLabel(" - Rayon"));
        panLoopsAndRadius.add(spinnerDepArrRadius);
        ThematicConfPanel confType = ThematicConfPanel.createThematicConfPanel(i18n.get(this, "type", "Representation type"), false);
        JPanel wrapPanType = new JPanel();
        wrapPanType.add(confType);
        confType.setContent(wrapType);
        tgui.addThematicConfigurationPanel(confType);
        add(wrapPanType);

        panStyle.add(leftStyleBox);
        wrapType.setBorder(BorderFactory.createEmptyBorder(0, 25, 0, 0));
        /*ThematicConfPanel themaConfFromTo = ThematicConfPanel.createThematicConfPanel(i18n.get(this, "from_to", "From/To Columns"));
        themaConfFromTo.setContent(panFromTo);
        tgui.addThematicConfigurationPanel(themaConfFromTo);
        add(themaConfFromTo);*/

        ThematicConfPanel vt = ThematicConfPanel.createThematicConfPanel(i18n.get(this, "arrow_style", "Arrows' Style"));
        vt.setContent(new JScrollPane(panStyle));
        JPanel wrapStyle = new JPanel();
        wrapStyle.add(vt);
        add(wrapStyle);
        tgui.addThematicConfigurationPanel(vt);
        ThematicConfPanel panResizer = (ThematicConfPanel) flow.getResizer().getGUI(dsi);
        JPanel wrapSizer = new JPanel();
        wrapSizer.add(panResizer);
        add(wrapSizer);
        tgui.addThematicConfigurationPanel(panResizer);
        //final JPanel coloredAreaPanel = new JPanel();
        final JCheckBox classifCB = new JCheckBox(
                i18n.get(this, "classified", "Use the classification"));
        //coloredAreaPanel.setLayout(new BorderLayout());
        //classifCB.setBorder(BorderFactory.createEmptyBorder(-1,0,-2,0));
        colorizerPanel = colorizer.getGUI();
        //colorizerPanel.setScrollDimension(new Dimension(280,50));
        //colorizerPanel.add(classifCB, 0);
        //JPanel colorDataPanel = new JPanel();
        dataPanel = new JPanel();
        dataPanel.setLayout(new BoxLayout(dataPanel, BoxLayout.X_AXIS));

        //cardPanel = new JPanel(new CardLayout());


        //colorDataPanel.add(classifCB);
        //colorDataPanel.add(dataPanel);
        dataPanel.add(new JLabel(i18n.get(Area.class, "colorDataColumnName", "Color Data Column Name: ")));
        dataPanel.add(dataList);
        dataPanel.add(colorizerPanel.getClassifList());
        ComponentSizer.fixComponentSize(dataList, new Dimension(120, 20));
        //colorizerPanel.add(dataPanel, 0);
        //colorDataPanel.add(Box.createHorizontalGlue());

        JPanel noClassifPan = new JPanel();
        noClassifPan.add(new JLabel(i18n.get(this, "configure_color", "Colors must be set in Geographical Entity tab")));

        //coloredAreaPanel.add(colorDataPanel);

        //cardPanel.add(colorizerPanel, CLASSIF_PAN);
        //cardPanel.add(noClassifPan, NO_CLASSIF_PAN);
        JPanel wrapClassifCB = new JPanel(new BorderLayout());
        wrapClassifCB.add(classifCB, BorderLayout.WEST);
        JPanel wrap = new JPanel(new BorderLayout());
        wrap.add(wrapClassifCB, BorderLayout.NORTH);
        JPanel wrapCenter = new JPanel(new BorderLayout());
        wrapCenter.add(dataPanel, BorderLayout.NORTH);
        wrapCenter.add(colorizerPanel, BorderLayout.CENTER);
        wrap.add(wrapCenter, BorderLayout.CENTER);
        //coloredAreaPanel.add(wrapClassifCB, BorderLayout.NORTH);
        //coloredAreaPanel.add(colorizerPanel, BorderLayout.CENTER);

        confp = ThematicConfPanel.createThematicConfPanel(i18n.get(this, "classif", "Classification"), false);
        /*confp.setContent(new JScrollPane(
        wrap,
        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER
        ));*/
        confp.setContent(wrap);
        JPanel wrapClassif = new JPanel();
        wrapClassif.add(confp);
        classifCB.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent ie) {
                setClassified(classifCB.isSelected());
                setClassifPanEnabled(classifCB.isSelected());
            }
        });
        classifCB.setSelected(flow.isClassified());
        //setClassified(classifCB.isSelected());

        confp.updateUI();

        //tgui.add(confp);
        tgui.addThematicConfigurationPanel(confp);

        add(wrapClassif);
        ////colorizerPanel.setScrollDimension(new Dimension(310,90));
        //showFlowPanel(false);
    }

    private void refreshToBox(DataStorageIterator dsi) {
        toBox.removeAll();
        
        for (int i = 0; i < flow.getNbStep(); i++) {
            final int index = i+1;
            JComboBox cb = new JComboBox(dsi.getOMColumnNames());
            String stepCol = flow.getStepColumnAt(index);
            if(stepCol != null)
                cb.setSelectedItem(stepCol);
            flow.setStepColumn(index,cb.getSelectedItem().toString());
            toBox.add(cb);
            cb.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent ie) {
                    flow.setStepColumn(index,((JComboBox)ie.getSource()).getSelectedItem().toString());
                }
            });
        }
        panFromTo.updateUI();
    }

    private void setClassifPanEnabled(boolean b) {
        dataList.setEnabled(b);
        colorizerPanel.setEnabled(b);

    }

    private void setClassified(boolean b) {
        flow.setClassified(b);
        confp.updateUI();
    }

    private void initDataList(DataStorageIterator dsi) {
        if (dataList == null) {
            dataList = new JComboBox(dsi.getDataColumnNames());
        } else {
            for (ActionListener a : dataList.getActionListeners()) {
                dataList.removeActionListener(a);
            }
            dataList.removeAllItems();
            for (String item : dsi.getDataColumnNames()) {
                dataList.addItem(item);
            }
        }
        //System.err.println("DATA COLUMN NAME" + getDataColumnName());
        if (!"".equals(flow.getColorDataColumnName()) && flow.getColorDataColumnName() != null) {
            dataList.setSelectedIndex(
                    dsi.getIndexForColumnName(flow.getColorDataColumnName()));
        } else {
            if (dsi.getDataColumnNames().length > 0) {
                flow.setColorDataColumnName(dsi.getDataColumnNames()[0]);
            }
        }

        dataList.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    int index = ((JComboBox) e.getSource()).getSelectedIndex();
                    flow.setColorDataColumnName(flow.getLayer().getDataStorageIterator().getRealColumnName(index));
                } catch (ArrayIndexOutOfBoundsException ex) {
                }
            }
        });
    }

    public void refreshGUI(DataStorageIterator dsi) {
        if (!flow.isDragging()) {
            refreshToBox(dsi);
//            comboToCol = new JComboBox(dsi.getOMColumnNames());
//
//            int index = dsi.getIndexForOMColumnName(flow.getToColumnName());
//            if (index == -1 && dsi.getOMColumnNames().length > 0) {
//                index = 0;
//            }
//
//            comboToCol.setSelectedIndex(index);
//
//            comboToCol.addActionListener(new ActionListener() {
//
//                public void actionPerformed(ActionEvent event) {
//                    try {
//                        flow.setToColumnName(((JComboBox) event.getSource()).getSelectedItem().toString());
//                        flow.setBadCache(true);
//                    } catch (NullPointerException exc) {
//                    }
//                }
//            });
        }

        //primary keys list
			/*if(columnList == null)
        {
        columnListModel = new DefaultListModel();
        for(String s:dsi.getDataColumnNames())
        columnListModel.addElement(s);
        columnList = new JList(columnListModel);
        selectedListModel = new DefaultListModel();
        selectedColumnList = new JList(selectedListModel);
        }
        else{

        columnListModel.removeAllElements();
        selectedListModel.removeAllElements();
        for(String s:dsi.getDataColumnNames())
        columnListModel.addElement(s);


        }*/

    }

    /**
     * Build combo box for an arrow model choice
     *
     * @param selectedName
     * @param type
     * @param typeName
     * @return
     */
    private JComboBox comboBoxForArrowHeadModelChoice(
            final int selectedIndex, final int type) {
        //ButtonGroup res = new ButtonGroup();
        final JComboBox cb = new JComboBox();
        //JPanel panel = new JPanel();
        //panel.add(new JLabel(typeName + ": "));
        //panel.add(cb);
        for (ArrowHead.ArrowHeadType arrowHeadType : ArrowHead.ArrowHeadType.values()) {
            cb.addItem(i18n.get(this, arrowHeadType.getName(), arrowHeadType.getName()));
        }
        final Flow myFlow = this.flow;
        ItemListener listener = new ItemListener() {

            public void itemStateChanged(ItemEvent arg0) {
                if (type == TYPE_START) {
                    //startIcon = ArrowHead.getModel(cb.getSelectedIndex()).getName();
                    myFlow.setStartArrowModel(cb.getSelectedIndex());
                } else {
                    //endIcon = ArrowHead.getModel(cb.getSelectedIndex()).getName();
                    myFlow.setEndArrowModel(cb.getSelectedIndex());
                }
                setOverviewIcon();

                //flow.getLayer().refresh();
                flow.setBadCache(true);
            }
        };
        cb.setSelectedIndex(selectedIndex);
        cb.addItemListener(listener);
        return cb;
    }

    /**
     * Build radio button group for an arrow model choice
     *
     * @param selectedName
     * @param type
     * @param typeName
     * @return
     */
//    private JPanel buttonGroupForArrowHeadModelChoice(
//            final String selectedName, final int type, String typeName) {
//        ButtonGroup res = new ButtonGroup();
//        JPanel panel = new JPanel();
//        panel.add(new JLabel(typeName + ": "));
//
//        for (int i = 0; i < ArrowHead.getArrowHeadModelsNames().length; i++) {
//            JRadioButton b = new JRadioButton(
//                    i18n.get(this, ArrowHead.getArrowHeadModelsNames()[i], ArrowHead.getArrowHeadModelsNames()[i]));
//            // select the button of the current drawing type of the flowlayer
//            if (selectedName.equals(ArrowHead.getArrowHeadModelsNames()[i])) {
//                b.setSelected(true);
//            }
//            res.add(b);
//            panel.add(b);
//            final int a = i;
//            final Flow myFlow = this.flow;
//            b.addActionListener(new ActionListener() {
//
//                public void actionPerformed(ActionEvent evt) {
//                    if (type == TYPE_START) {
//                        startIcon = ArrowHead.getArrowHeadModelsNames()[a];
//                        myFlow.setStartArrowModel(ArrowHead.getArrowHeadModelsNames()[a]);
//                    } else {
//                        endIcon = ArrowHead.getArrowHeadModelsNames()[a];
//                        myFlow.setEndArrowModel(ArrowHead.getArrowHeadModelsNames()[a]);
//                    }
//                    setOverviewIcon();
//
//                    //flow.getLayer().refresh();
//                    flow.setBadCache(true);
//                }
//            });
//        }
//        return panel;
//    }
    /**
     * build the bended value JSpiner
     *
     * @return
     */
    private JSpinner bendedSpinner() {
        // JPanel panSpinner = new JPanel();
        JSpinner spin = new JSpinner();
        //final Flow myFlow = flow;
        spin.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent event) {
                String val = (((JSpinner) event.getSource()).getValue()).toString();
                OMFlow.BENDED_DISTANCE = Integer.parseInt(val);
                //myFlow.getLayer().refresh();
                flow.setBadCache(true);
            }
        });
        JComponent editor = spin.getEditor();
        ((JSpinner.DefaultEditor) editor).getTextField().setColumns(5);
        spin.setValue(OMFlow.BENDED_DISTANCE);
        return spin;
    }

    /**
     * build the number of line per arc JSpiner
     *
     * @return
     */
    private JSpinner nbLineSpinner() {
        // JPanel lineSpinner = new JPanel();
        JSpinner spin = new JSpinner();
        //final Flow myFlow = flow;
        spin.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent event) {
                String val = (((JSpinner) event.getSource()).getValue()).toString();
                OMFlow.ARC_LENGTH = Integer.parseInt(val);
                flow.setBadCache(true);
                //myFlow.getLayer().
                //myFlow.getLayer().refresh();
            }
        });
        JComponent editor = spin.getEditor();
        ((JSpinner.DefaultEditor) editor).getTextField().setColumns(5);
        spin.setValue(OMFlow.ARC_LENGTH);
        return spin;

    }

    /**
     * set the overview arrow model icon
     */
    private void setOverviewIcon() {
        //overviewIcon.setIcon(Util.loadIcon(startIcon + endIcon
        //		+ ".png"));
        ovPanel.repaint();
    }

//    private void showFlowPanel(boolean b) {
//        panFromTo.setVisible(b);
//    }
    class FlowOverviewPanel extends JPanel {

        /*public FlowOverviewPanel(){
        //setPreferredSize(new Dimension(220,150));
        //setBorder(BorderFactory.createLineBorder(Color.BLACK));
        }*/
        public void paintComponent(Graphics g) {
            g.clearRect(0, 0, getWidth(), getHeight());
            super.paintComponent(g);

            OMFlow sample = new OMFlow(flow,
                    new Point(getWidth() / 2 - 30, getHeight() / 2),
                    new Point(getWidth() / 2 + 30, getHeight() / 2),
                    ArrowHead.getModel(flow.getStartArrowModel()),
                    ArrowHead.getModel(flow.getEndArrowModel()),
                    5);
            if (flow.getDrawingAttributes().getFillPaint() != null) {
                g.setColor((Color) flow.getDrawingAttributes().getFillPaint());
                g.fillPolygon(sample.getXBorderAsInt(), sample.getYBorderAsInt(),
                        sample.getXBorder().length);
            }
            g.setColor((Color) flow.getDrawingAttributes().getLinePaint());
            g.drawPolygon(sample.getXBorderAsInt(), sample.getYBorderAsInt(),
                    sample.getXBorder().length);

        }
    }
//    public JPanel getTypePanel() {
//        return panType;
//    }
}
