// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.event.*;

import com.bbn.openmap.event.*;
import com.bbn.openmap.util.*;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.MapBean;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import java.awt.Cursor;


public class MouseControlPointsListener implements MapMouseListener {

    protected OMGraphicList omcps;
    protected MapBean mapBean;


    public MouseControlPointsListener(OMGraphicList omcps) {
        this.omcps = omcps;

    }

    public MouseControlPointsListener() {
        this.omcps = new OMGraphicList();
    }

    public void addOMControlPoints(OMGraphicList omcp) {
	omcps.add(omcp);
	// set the vivibility according to getOMControlPointVisible() 
	//                              and the visibility of the controlled OMResizeableGraphic 
	
	// Debug.message("Legend", "ml@"+hashCode()+" AddOMControlPoints " + omcp.hashCode() + " -> " + omcps.size() /*+ "\n" + omcpsToString()*/);
    }

    public void removeAllOMControlPoints(){
        omcps.clear();
    }

    public void setOMControlPoints(OMGraphicList omcp){
        omcps = omcp;
    }

    public OMGraphicList getOMControlPoints(){
        return omcps;
    }

    private boolean omControlPointsVisible = false; 
    public boolean getOMControlPointVisible() { return omControlPointsVisible; }
    public void setOMControlPointsVisible(boolean v) {
        // Debug.message("Legend", "ml@"+hashCode()+" Set OMCP visibility to "+ v +" - nb OMCP = "+omcps.size() + "\n" + omcpsToString());
	omControlPointsVisible = v;
	if (omControlPointsVisible) {
	    // set the visibility according to the visibility of the controlled resizeable graphics
	    for(int i = 0; i < omcps.size() ; i++)  
		omcps.getOMGraphicAt(i).setVisible(v);
	} else // turn them off
	    omcps.setVisible(false);
    }

    public String[] getMouseModeServiceList() {
        return new String[]{SelectMouseMode.modeID};
    }
    protected ThemaMapControlPoint omcpSelected = null;

    /**
     * Invoked when a mouse button has been pressed on a component. 
     * Find closest OMGraphicList from MouseEvent and find index of this list in LegendableLayers Vector.
     * @param e MouseEvent
     * @return true (the listener is always able to process the event).
     */
    public boolean mousePressed(MouseEvent e) {
	Debug.message("LegendMouse", ">> mousePressed("+e.getX()+", "+e.getY()+") " + omcps.size());

        OMGraphic omg = omcps.findClosest(e.getX(), e.getY(), 2);
        if (!(omg instanceof ThemaMapControlPoint)) {
            return false;
        }
        omcpSelected = (ThemaMapControlPoint) omg;

        if (omcpSelected != null) {
            Debug.message("LegendMouse", omcpSelected + ".doActionPressed");
            omcpSelected.doActionPressed(e);
            return true;
        }
        return false;
    }

    /**
     * Invoked when a mouse button has been released on a component. 
     * A change (move all - resize border) is over.
     * @param e MouseEvent
     * @return true if the listener was able to process the event.
     */
    public boolean mouseReleased(MouseEvent e) {
        Debug.message("LegendMouse", "<< mouseReleased("+e.getX()+", "+e.getY()+") "+omcpSelected);

        if (omcpSelected != null) {
            Debug.message("LegendMouse", omcpSelected + ".doActionReleased");
            omcpSelected.doActionReleased(e);
            omcpSelected = null;
            return true;
        }
        return false;
    }

    public boolean mouseClicked(MouseEvent e) {
        return true;
    }

    /**
     * Invoked when a mouse button is pressed on a component and then dragged.
     */
    public boolean mouseDragged(MouseEvent e) {
        // Debug.message("LegendMouse", this+" MapBean = " + mapBean);
	Debug.message("LegendMouse", ">> Legend mouseDragged(" + e.getX() + ", " + e.getY() + ") "+omcpSelected);
	// forward the event only if a point is selected and 
        //         either the mapbean is null or the point is outside the mapbean's window
        if (omcpSelected != null && (mapBean == null || mapBean.contains(e.getX(), e.getY()))) {
	    Debug.message("LegendMouse", omcpSelected + ".doActionDragged");
	    omcpSelected.doActionDragged(e);
	    return true;
	}
	return false;
    }

    /**
     * Invoked when the mouse enters a component.
     * @param e MouseEvent
     */
    public void mouseEntered(MouseEvent e) {
        //System.out.println("mouse entered");
    }

    /**
     * Invoked when the mouse exits a component.
     * @param e MouseEvent
     */
    public void mouseExited(MouseEvent e) {
        // Debug.message("Legend", "Legend mouseExited("+e.getX()+", "+e.getY()+")");
    }

    /**
     * Handle a mouse cursor moving without the button being pressed. 
     */
    public void mouseMoved() {
        // Debug.message(Util.className(this), this + " mouseMoved()");
    }

    /**
     * Invoked when the mouse button has been moved on a component (with no buttons down).
     * @param e MouseEvent
     * @return true if the listener was able to process the event.
     */
    public boolean mouseMoved(MouseEvent e) {
        if(mapBean == null)
            return false;
        else{
        OMControlPoint omcp = (OMControlPoint)omcps.findClosest(e.getX(), e.getY(), 2);
        if(omcp !=null ){
                if(omcp instanceof OMControlAngle){
                   mapBean.setCursor(new Cursor(((OMControlAngle)omcp).getCursor()));
                   return true;
                }
                else{
                    mapBean.setCursor(new Cursor(Cursor.MOVE_CURSOR));
                    return true;
                }
            }
        else
            mapBean.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            return false;
        }
        
    }
//        return true;
//    }

    public MapBean getMapBean() {
        return mapBean;
    }

    public void setMapBean(MapBean mapBean) {
        this.mapBean = mapBean;
    }

    public String omcpsToString() {
	StringBuffer ret = new StringBuffer();
    	for (OMGraphic omg : omcps) {
	    if (omg instanceof OMGraphicList) {
		ret.append(((OMGraphicList)omg).size());
		for (OMGraphic omg2 : ((OMGraphicList)omg))
		    ret.append(" " + omg2);
	    } else
		ret.append(" " + omg); 
	    ret.append("\n");
	}
	return ret.toString();
    }

}
