// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;

import fr.unicaen.openmap.event.GUIChangeEvent;
import fr.unicaen.openmap.event.GUIChangeListener;
import fr.unicaen.openmap.gui.FileNamePanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMRegularPolygon;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.ProportionalSymbol;
import fr.unicaen.openmap.util.gui.ComponentSizer;
import fr.unicaen.openmap.util.propertyEditor.FileTypeDescription;
import java.awt.GridLayout;
import java.awt.RenderingHints;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

/**
 * GUI Panel for ProportionalSymbol Layer
 * 
 * @author jerome chauveau
 * 
 */
public class ProportionalSymbolPanel extends JPanel implements GUIChangeListener {

    private ProportionalSymbol proportionalSymbol;
    private JPanel sliderBox = new JPanel(new GridLayout(3, 2)); // left panel for sliders
    private JPanel regularPolygonSamplePanel; // right panel for sample drawing
    private JPanel cards, typePanel, cardPict;
    private ImgPanel imgPanel;
    private JSpinner nbEdgesField, startAngleField, starCoefField;
    private JRadioButton rbDisc, rbPoly, rbPict;
    private FileNamePanel imgFilePanel;
    protected I18n i18n = new BasicI18n();
    private ProportionalSymbol.type mode;
    final static String POLYPANEL = "card with poly spinners";
    final static String PICTOPANEL = "card with picto panel";

    public ProportionalSymbolPanel(ProportionalSymbol l) {
        proportionalSymbol = l;
        //setLayout(new BorderLayout());
        final JPanel mainPan = new JPanel();
        add(mainPan);
        mainPan.setLayout(new BorderLayout());
        //mainPan.setBorder(BorderFactory.createTitledBorder(""));
        typePanel = buildTypePanel();
        regularPolygonSamplePanel = new RegularPolygonSamplePanel(25);
        SpinnerModel model =
                new SpinnerNumberModel(l.getNbEdges()/*ProportionalSymbol.nbDiscEdges*/, //initial value
                3, //min
                21, //max
                1);//step
        nbEdgesField = new JSpinner(model);
        Dimension dim = new Dimension(50, 25);

        nbEdgesField.setMaximumSize(dim);
        nbEdgesField.setPreferredSize(dim);
        nbEdgesField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                proportionalSymbol.setNbEdges(((Integer) nbEdgesField.getValue()).intValue());
                repaintSamplePanel();
                proportionalSymbol.getLayer().refreshGraphicsAndGUI();
            }
        });
        JLabel lab1 = new JLabel(i18n.get(this, "edgesLabel", "Nb edges"));// "label"
        sliderBox.add(lab1);
        sliderBox.add(nbEdgesField);

        model =
                new SpinnerNumberModel(proportionalSymbol.getStartAngle(), //initial value
                0, //min
                180, //max
                5);//step
        startAngleField = new JSpinner();
        startAngleField.setValue(proportionalSymbol.getStartAngle());
        startAngleField.setMaximumSize(dim);
        startAngleField.setPreferredSize(dim);

        startAngleField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                proportionalSymbol.setStartAngle(((Integer) startAngleField.getValue()).intValue());
                repaintSamplePanel();
            }
        });
        JLabel lab2 = new JLabel(i18n.get(this, "rotationLabel", "Rotation"));
        lab2.setVerticalAlignment(JLabel.TOP);
        sliderBox.add(lab2);
        sliderBox.add(startAngleField);
        model =
                new SpinnerNumberModel(proportionalSymbol.getStarCoef(), //initial value
                0, //min
                100, //max
                1);//step
        starCoefField = new JSpinner();
        starCoefField.setMaximumSize(dim);
        starCoefField.setPreferredSize(dim); //	starCoefField.setMajorTickSpacing(25);
        starCoefField.addChangeListener(
                new ChangeListener() {

                    public void stateChanged(ChangeEvent e) {
                        proportionalSymbol.setStarCoef(((Integer) starCoefField.getValue()).intValue());
                        repaintSamplePanel();
                    }
                });
        JLabel lab3 = new JLabel(i18n.get(this, "starCoefLabel", "Star coef."));
        lab3.setVerticalAlignment(JLabel.TOP);
        sliderBox.add(lab3);
        sliderBox.add(starCoefField);
        cards = new JPanel();
        cards.setLayout(new CardLayout());
        JPanel card1 = new JPanel();
        card1.add(sliderBox);
        card1.add(regularPolygonSamplePanel);
        cardPict = new JPanel(new BorderLayout());
        imgFilePanel = new FileNamePanel(FileTypeDescription.IMG_DESCRIPTION,
                "", FileTypeDescription.IMG_PATTERN, this);
        cardPict.add(imgFilePanel, BorderLayout.NORTH);
        JScrollPane sp = new JScrollPane(buildImgPanel(150),
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        ComponentSizer.fixComponentSize(sp, new Dimension(280, 65));
        cardPict.add(sp, BorderLayout.CENTER);
        cards.add(card1, POLYPANEL);
        cards.add(cardPict, PICTOPANEL);
        mainPan.add(typePanel, BorderLayout.NORTH);
        mainPan.add(cards, BorderLayout.CENTER);
        setMode(proportionalSymbol.getMode());

    }

    /**
     * Build panel with type radio buttons
     * 
     * @return
     */
    private JPanel buildTypePanel() {
        JPanel tPanel = new JPanel();
        ButtonGroup bgroup = new ButtonGroup();
        rbDisc = new JRadioButton(i18n.get(this, "disc", "Disc"));

        rbPoly = new JRadioButton(i18n.get(this, "poly", "Polygon"));

        rbPict = new JRadioButton(i18n.get(this, "picto", "Pictogram"));

        if (proportionalSymbol.getMode() == ProportionalSymbol.type.DISC) {
            rbDisc.setSelected(true);
        } else {
            if (proportionalSymbol.getMode() == ProportionalSymbol.type.POLY) {
                rbPoly.setSelected(true);
            } else {
                rbPict.setSelected(true);
            }
        }

        rbPoly.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    setMode(ProportionalSymbol.type.POLY);
                }
            }
        });

        rbPict.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    setMode(ProportionalSymbol.type.PICTO);
                }
            }
        });

        rbDisc.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    setMode(ProportionalSymbol.type.DISC);
                }
            }
        });
        bgroup.add(rbDisc);
        bgroup.add(rbPoly);
        bgroup.add(rbPict);
        tPanel.add(rbDisc);
        tPanel.add(rbPoly);
        tPanel.add(rbPict);

        return tPanel;
    }

    public void repaintSamplePanel() {
        regularPolygonSamplePanel.repaint();
    }

    /**
     * Set GUI aspect according to representation selected mode
     * 
     * @param t
     */
    private void setMode(ProportionalSymbol.type t) {
        CardLayout cl = (CardLayout) (cards.getLayout());
        mode = t;
        proportionalSymbol.setMode(t);

        switch (t) {
            case DISC:
                cl.show(cards, POLYPANEL);
                nbEdgesField.setValue(ProportionalSymbol.nbDiscEdges);
                starCoefField.setValue(0);
                nbEdgesField.setEnabled(false);
                startAngleField.setEnabled(false);
                starCoefField.setEnabled(false);
                break;
            case PICTO:
                cl.show(cards, PICTOPANEL);
                break;
            case POLY:
                cl.show(cards, POLYPANEL);
                nbEdgesField.setValue(proportionalSymbol.getNbEdges()/*ProportionalSymbol.nbPolyEdges*/);
                nbEdgesField.setEnabled(true);
                startAngleField.setEnabled(true);
                starCoefField.setEnabled(true);
                break;
        }

    }

    public void removePicto() {
        typePanel.remove(rbPict);
        CardLayout cl = (CardLayout) cards.getLayout();
        cl.removeLayoutComponent(cardPict);
        updateUI();
    }

    private JComponent buildImgPanel(int size) {
        imgPanel = new ImgPanel(size, "", false);
        imgPanel.setPreferredSize(new Dimension(size, size));
        imgPanel.setMaximumSize(new Dimension(size, size));
        imgPanel.setBorder(BorderFactory.createTitledBorder("Image"));
        return imgPanel;
    }

    public void reloadFromGUI(GUIChangeEvent event) {
        Object source = event.getSource();
        if (source == imgFilePanel) {
            String imgFile = imgFilePanel.getFilePath();
            imgPanel.setImg(imgFile);
            imgPanel.setAsImg(true);
            imgPanel.repaint();
            proportionalSymbol.setPictogram(imgFile);
        }

    }

    /**
     * Inner class: Panel in which is drawn Proportional symbol overview
     * 
     */
    class RegularPolygonSamplePanel extends JPanel {

        int size;

        public RegularPolygonSamplePanel(int s) {
            size = s;
            setPreferredSize(new Dimension(2 * s, 2 * s));
        }

        public void paintComponent(Graphics g) {

            int xOffset = size, yOffset = size;
            Graphics2D g2 =(Graphics2D) g;
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                        RenderingHints.VALUE_ANTIALIAS_ON);
            //g2.setColor(new Colo); // colors.getLineColor());
            g2.clearRect(0, 0, getWidth(), getHeight());
            //g2.fillRect(5, 5, 2 * size - 5, 2 * size - 5);
            //g2.setColor(Color.black);
            //g2.drawRect(5, 5, 2 * size - 6, 2 * size - 6);
           // g2.setColor(Color.blue);
            if (mode != ProportionalSymbol.type.DISC) {
//		g.drawString(String.format("%d "
//					   + i18n.get(RegularPolygonPanel.class, "edges", "edges")
//					   + ", %d "
//					   + i18n.get(RegularPolygonPanel.class, "rotation",
//						      "rotation") + ", %d%% "
//					   + i18n.get(RegularPolygonPanel.class, "star", "star"),
//					   layer.getNbEdges(), layer.getStartAngle(), layer
//					   .getStarCoef()), 5, 15);
            }
            OMRegularPolygon sample = new OMRegularPolygon(null, "", 0F, 0F, proportionalSymbol.getNbEdges(), proportionalSymbol.getElemRadius(), xOffset, yOffset,
                    (float) (proportionalSymbol.getStartAngle() * Math.PI / 180.0), proportionalSymbol.getStarCoef(), false, false);
            // We don't draw directly the OMPoly in g but we borrow its Xs and
            // Ys !
            if (proportionalSymbol.getDrawingAttributes().getFillPaint() != null) {
                g2.setColor((Color)proportionalSymbol.getDrawingAttributes().getFillPaint());
                g2.fillPolygon(sample.getXs(), sample.getYs(),
                        sample.getXs().length);
            }
            g2.setColor((Color)proportionalSymbol.getDrawingAttributes().getLinePaint());
            g2.drawPolygon(sample.getXs(), sample.getYs(),
                    sample.getXs().length);
        }
    }

    /**
     * Inner class: Panel in which is drawn selected pictogram overview
     * 
     */
    class ImgPanel extends JPanel {

        private String img;
        private boolean asImg;
        private int size;

        ImgPanel(int size, String img, boolean b) {
            this.size = size;
            asImg = b;
            this.img = img;
        }

        public void paintComponent(Graphics g) {
            g.clearRect(0, 0, getWidth(), getHeight());
            if (!asImg) {
                g.drawString("-" + i18n.get(this, "noimage", "-no image-") + "-", 10, 40);
            } else {
                ImageIcon imgc = new ImageIcon(this.img);
                Image img = imgc.getImage();
                g.drawImage(img, 5, 10, size - 10, size - 20, this);
            }
        }

        public void setAsImg(boolean b) {
            this.asImg = b;
        }

        public void setImg(String img) {
            this.img = img;
        }
    }
}
