// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import fr.unicaen.openmap.util.Colors;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.OMRegularPolygon;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.OMResizeableRegularPolygon;
import com.bbn.openmap.util.Debug;
import java.util.Locale;
import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;

public class RegularPolygonPanel extends JPanel {

    private RegularPolygon layer;
    // protected Thematic getLayer() { return layer; }
    private JPanel sliderBox = new JPanel(); // left panel for sliders
    private JPanel regularPolygonSamplePanel; // right panel for sample drawing
    /**
     * Use I18N mecanism to allow internationalization. Use I18N.properties file
     * or I18N_fr_FR.properties for french version
     */
    protected I18n i18n = new BasicI18n();
    private boolean multi;
    private static final int nbPolys = 3;

    public RegularPolygonPanel(RegularPolygon l, boolean multi) {
        this(l);
        this.multi = multi;
    }

    public RegularPolygonPanel(RegularPolygon l) {
        layer = l;

        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        setBorder(BorderFactory.createTitledBorder(""/* i18n.get(this,"title","Polygon") */));
        regularPolygonSamplePanel = new RegularPolygonSamplePanel(60);

        sliderBox.setLayout(new GridLayout(3,2));

        Box sbox;

        sbox = Box.createHorizontalBox();
//        final JSlider nbEdgesField = new JSlider(JSlider.HORIZONTAL, 3, 21,
//                layer.getNbEdges());
        SpinnerModel model =
                new SpinnerNumberModel(layer.getNbEdges(), //initial value
                3, //min
                21, //max
                1);//step
        final JSpinner nbEdgesField = new JSpinner(model);
//        nbEdgesField.setMajorTickSpacing(3);
//        nbEdgesField.setMinorTickSpacing(1);
//        nbEdgesField.setSnapToTicks(true);
//        nbEdgesField.setPaintTicks(true);
//        nbEdgesField.setPaintLabels(true);
        nbEdgesField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                layer.setNbEdges(((Integer) nbEdgesField.getValue()).intValue());
                repaintSamplePanel();
            }
        });
        sliderBox.add(new JLabel(i18n.get(this, "edgesLabel", "Nb edges")), "label");
        //JPanel wrapNbEdges = new JPanel();
        //wrapNbEdges.add(nbEdgesField);
        sbox.add(nbEdgesField, "field");
        sliderBox.add(sbox);

        sbox = Box.createHorizontalBox();
//        final JSlider startAngleField = new JSlider(JSlider.HORIZONTAL, 0, 180,
//                layer.getStartAngle());
        model =
                new SpinnerNumberModel(layer.getStartAngle(), //initial value
                0, //min
                180, //max
                5);//step
        final JSpinner startAngleField = new JSpinner(model);
//        startAngleField.setMajorTickSpacing(45);
//        startAngleField.setMinorTickSpacing(15);
//        startAngleField.setPaintTicks(true);
//        startAngleField.setPaintLabels(true);
        startAngleField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                layer.setStartAngle(((Integer) startAngleField.getValue()).intValue());
                repaintSamplePanel();
            }
        });
        sliderBox.add(new JLabel(i18n.get(this, "rotationLabel", "Rotation")),
                "label");
        //JPanel wrapStartAngle = new JPanel();
       // wrapStartAngle.add(startAngleField);
        sbox.add(startAngleField, "field");
        sliderBox.add(sbox);

        sbox = Box.createHorizontalBox();
//        final JSlider starCoefField = new JSlider(JSlider.HORIZONTAL, 0, 100,
//                layer.getStarCoef());
        model =
                new SpinnerNumberModel(layer.getStarCoef(), //initial value
                0, //min
                180, //max
                5);//step
        final JSpinner starCoefField = new JSpinner(model);
//        starCoefField.setMajorTickSpacing(25);
//        starCoefField.setMinorTickSpacing(5);
//        starCoefField.setSnapToTicks(true);
//        starCoefField.setPaintTicks(true);
//        starCoefField.setPaintLabels(true);
        starCoefField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                layer.setStarCoef(((Integer) starCoefField.getValue()).intValue());
                repaintSamplePanel();
            }
        });
        sliderBox.add(new JLabel(i18n.get(this, "starCoefLabel", "Star coef.")),
                "label");
        sbox.add(starCoefField, "field");
        sliderBox.add(sbox);

        add(sliderBox);

        add(regularPolygonSamplePanel);

    }

    public void addToSliderBox(JComponent clab, JComponent cslid) {
        GridLayout gLayout = (GridLayout)sliderBox.getLayout();
        gLayout.setRows(gLayout.getRows() + 1);
        sliderBox.add(clab);
        sliderBox.add(cslid);
        sliderBox.updateUI();
    }

    public void repaintSamplePanel() {
        regularPolygonSamplePanel.repaint();
    }

    class RegularPolygonSamplePanel extends JPanel {

        int size;

        public RegularPolygonSamplePanel(int s) {
            size = s;
            setPreferredSize(new Dimension(2 * s, 2 * s));
        }

        private void drawRegularPoly(Graphics g, int nbEdges, float radius,
                int xOffset, int yOffset, float startAngle, int starCoef) {
            OMRegularPolygon sample =
                    new OMRegularPolygon(null, "", 0F, 0F, nbEdges, radius, xOffset, yOffset, startAngle, starCoef, false, false);
            // We don't draw directly the OMPoly in g but we borrow its Xs
            // and Ys !
            if (layer.getDrawingAttributes().getFillPaint() != null) {
                g.setColor((Color)layer.getDrawingAttributes().getFillPaint());
                g.fillPolygon(sample.getXs(), sample.getYs(), sample.getXs().length);
            }
            g.setColor((Color)layer.getDrawingAttributes().getLinePaint());
            g.drawPolygon(sample.getXs(), sample.getYs(), sample.getXs().length);
        }

        public void paintComponent(Graphics g) {
            int xOffset = size, yOffset = size;
            g.setColor(Color.white); // colors.getLineColor());
            g.clearRect(0, 0, getWidth(), getHeight());
            g.fillRect(5, 5, 2 * size - 5, 2 * size - 5);
            g.setColor(Color.blue);

            if (!multi) {
//                g.drawString(String.format("%d " + i18n.get(RegularPolygonPanel.class, "edges", "edges") + ", %d " + i18n.get(RegularPolygonPanel.class, "rotation",
//                        "rotation") + ", %d%% " + i18n.get(RegularPolygonPanel.class, "star", "star"),
//                        layer.getNbEdges(), layer.getStartAngle(), layer.getStarCoef()),
//                        5, 15);
                drawRegularPoly(g, layer.getNbEdges(), layer.getElemRadius(),
                        xOffset, yOffset,
                        (float) (layer.getStartAngle() * Math.PI / 180.0),
                        layer.getStarCoef());
            } else {
//                g.drawString(String.format("%d " + i18n.get(RegularPolygonPanel.class, "edges", "edges") + ", %d " + i18n.get(RegularPolygonPanel.class, "rotation", "rotation") + ", %d%% " + i18n.get(RegularPolygonPanel.class, "star", "star") + ", %d%% " + i18n.get(RegularPolygonPanel.class, "spacing", "spacing"),
//                        layer.getNbEdges(), layer.getStartAngle(), layer.getStarCoef(),
//                        layer.getGapPercentage()),
//                        5, 15);
                int radius = layer.getElemRadius();
                int gap = (int) (radius * layer.getGapPercentage() / 100.0);
                int width = (int) (2 * radius + gap); // place needed for one poly
                xOffset -= width * ((int) (nbPolys / 2.0 - 0.5));
                yOffset -= width * ((int) (nbPolys / 2.0 - 0.5));
                int xOffsetTmp = xOffset;

                for (int i = 0; i < nbPolys; i++) {
                    for (int j = 0; j < nbPolys; j++) {
                        drawRegularPoly(g, layer.getNbEdges(), layer.getElemRadius(),
                                xOffsetTmp, yOffset,
                                (float) (layer.getStartAngle() * Math.PI / 180.0),
                                layer.getStarCoef());
                        xOffsetTmp += width;
                    }
                    xOffsetTmp = xOffset;
                    yOffset += width;

                }
            }
        }
    }
}
