// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui;

import com.bbn.openmap.MapBean;
import java.awt.event.MouseEvent;

import com.bbn.openmap.event.MapMouseListener;
import com.bbn.openmap.omGraphics.OMGraphicList;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.ThemaMapControlPoint;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Thematic;
import java.awt.Cursor;

/**
 * Listener for OMFlowControlPoints
 * The user can select an OMflow (by a click on its OMControlPoint) and move the control point
 * to change curvature of the arrow
 * @author jerome chauveau
 *
 */
public class ThematicMouseControlPointsListener implements MapMouseListener {

    private OMGraphicList controlPoints;
    private ThemaMapControlPoint selected = null;
    private Thematic thematic;
    private MapBean mapBean;

    public ThematicMouseControlPointsListener(Thematic tv) {
        this.controlPoints = new OMGraphicList();
        this.thematic = tv;
    }

    public void clearControlPoints() {
        controlPoints.clear();
    }

    public void addOMControlPoints(OMGraphicList omcp) {
        controlPoints.add(omcp);
    }

    public String[] getMouseModeServiceList() {
        return null;
    }

    public boolean mouseClicked(MouseEvent arg0) {
        return false;
    }

    public boolean mouseDragged(MouseEvent arg0) {
        if (selected != null) {
            thematic.setBadCache(true);

            selected.doActionDragged(arg0);
            thematic.getLayer().repaint();
            //thematic.getLayer().refreshInteractiveAndBadCacheTVS();
            return true;
        }

        return false;
    }

    public void mouseEntered(MouseEvent arg0) {
    }

    public void mouseExited(MouseEvent arg0) {
    }

    public void mouseMoved() {
    }

    private MapBean getMapBean() {
        if (mapBean == null) {
            mapBean = thematic.getLayer().getMapBean();
        }
        return mapBean;
    }

    public boolean mouseMoved(MouseEvent e) {
        //System.out.println(thematic.getName() + "Mouse Moved");
        ThemaMapControlPoint tcp = (ThemaMapControlPoint) controlPoints.findClosest(e.getX(), e.getY(), 2);
        if (getMapBean() == null) {
            return false;
        } else {
            // System.out.println(controlPoints.size());
            if (tcp != null) {
                getMapBean().setCursor(new Cursor(Cursor.MOVE_CURSOR));
                //System.out.println(thematic.getName() + "MOVE CURSORRR-----");
                return true;
            } else {
                //System.out.println(thematic.getName() + "defaut CURSORRR");
                getMapBean().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                return false;
            }
        }

    }

    public boolean mousePressed(MouseEvent e) {

        selected = (ThemaMapControlPoint) controlPoints.findClosest(e.getX(), e.getY(), 2);

        if (selected != null) {

            selected.doActionPressed(e);

            return true;
        }
        return false;

    }

    public boolean mouseReleased(MouseEvent arg0) {
        //System.out.println("RELEASEDDDDDDDDDDDDDDDDDDDDdd");
        if (selected != null) {
            selected.doActionReleased(arg0);
//            for (OMGraphic cp : controlPoints) {
//            }
            selected = null;
            return true;
        }
        return false;
    }
}
