// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics;

import java.awt.event.*;
import java.awt.Color;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.MapBean;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend.*;

public class OMControlPoint extends OMPoly implements ThemaMapControlPoint{

    private static final String  classNameForDebug = "OMControlPoint";
    
    /** Used to stuck the control point inside the MapBean window */
    protected MapBean mapBean;

    protected OMControlPoint(int[] points, MapBean mb) {
	super(points);
	setColor(Color.black);
	mapBean = mb;	
    }

    protected OMControlPoint(double[] points, MapBean mb){
	super(points,OMGraphic.DECIMAL_DEGREES, OMGraphic.LINETYPE_STRAIGHT);	
	mapBean = mb;
	setColor(Color.black);	
    }
    /** (x0, y0) defines the anchor */
    protected int x0, y0;
    protected int radius;

    public OMControlPoint(int x, int y, int radius, MapBean mb) {
	this(new int[] {}, mb); // to set RENDERXY type
 	this.x0 = x; this.y0 = y;
	this.radius = radius;
	setXYs();
    }	
	
    public OMControlPoint(int x, int y, int radius, Color c, MapBean mb) {
	this(x, y, radius, mb);
	setColor(c);
	setSelectedColor(Color.red);
    }
	
    public OMControlPoint(int x, int y, int radius, Color c, Color sc, MapBean mb) {
	this(x, y, radius, mb);
	setColor(c);
	setSelectedColor(sc);
    }

    protected void setXYs() {
	setXs(new int[] { 
		x0-radius,
		x0+radius,
		x0+radius,
		x0-radius,
		x0-radius });
	setYs(new int[] { 
		y0+radius,
		y0+radius,
		y0-radius,
		y0-radius,
		y0+radius });
    }
	
    public void setColor(Color c) {
	setFillPaint(c);
    }

    public void setSelectedColor(Color c) {
	setSelectPaint(c);
    }
    
    /** translate the point */
    public void setAnchor(int x, int y) {
	// System.out.println("### " + this + ".moveto("+x+", "+y+")");
	x0 = x; y0 = y;
	setXYs();
    }

    public void doActionPressed(MouseEvent e) {
	// Debug.message(classNameForDebug, "[OMControlPoint] : Mouse Pressed");
	// Graphic is selected, should change its color
	select();
       ((OMResizeableGraphic)getAppObject()).addToInteractiveGraphic();
	// refresh the app object
	//((OMResizeableGraphic)getAppObject()).refreshInteractiveGraphics();
    }
	
    public void doActionReleased(MouseEvent e) {
        // Debug.message(classNameForDebug, "[OMControlPoint] : Mouse Released");
	// Graphic is deselected, should change its color
	deselect();
        ((OMResizeableGraphic)getAppObject()).released();
	((OMResizeableGraphic)getAppObject()).refreshGraphics();
    }

    public void doActionDragged(MouseEvent e) {
	// Debug.message(classNameForDebug, "[OMControlPoint] : Mouse Dragged");
	if (! isInMapBeanBounds(e.getX(), e.getY()))
	    return;

	int dx = e.getX() - x0;
	int dy = e.getY() - y0;

	OMResizeableGraphic omrg = (OMResizeableGraphic)getAppObject();
	// Debug.message("Legend2", "translate "+omrg);
	omrg.translate(dx, dy);
	omrg.refreshInteractiveGraphics();

    }

    public boolean isInMapBeanBounds(int x, int y){
	return mapBean == null || mapBean.contains(x, y);
    }

    public void translate(int dx, int dy) {
	setAnchor(x0+dx, y0+dy);
    }

    /** respond to the extension of a border */
    public void extend(int dw, int dh) {
    }

    java.awt.Paint nonSelectedFillPaint = fillPaint;
    public void select() {
	nonSelectedFillPaint = getFillPaint();
	setFillPaint(getSelectPaint());
	Object appObject = getAppObject();
	super.select();
	//System.out.println("APP OBJECT="+appObject);
    }

    public void deselect() {
	setFillPaint(nonSelectedFillPaint);
	Object appObject = getAppObject();
	super.deselect();
    }

    public String toString() {
	return fr.unicaen.openmap.util.Util.className(this)+"@"+Integer.toHexString(hashCode())+" ("+x0+", "+y0+") "+(isVisible()?" v":"");
    }
	
    public void setVisible(boolean b) {
	// System.out.print(this+" visibility to "+ b+" ");
	if (b) { // check if the app object is visible
	    Object appObject = getAppObject();
	    // System.out.print(appObject);
	    if (appObject instanceof OMResizeableGraphic &&
		! ((OMResizeableGraphic)appObject).getVisible())
		b = false;	   
	}
	// System.out.println(" -> "+b);
	super.setVisible(b);
    }
}
