// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.MinMeanMaxValuesAndSizes;
import java.awt.Paint;
import java.util.Properties;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/** Disc for DiscLegend */
public class OMResizeableDisc extends OMResizeableGraphic {

    private OMCircle omDisc;
    private OMLine omLine;
    private MinMeanMaxValuesAndSizes mmmValuesAndSizes;
    private MinMeanMaxValuesAndSizes.typeValue type;

    /*private Resizer resizer;
    private double value;*/
    public OMResizeableDisc(MinMeanMaxValuesAndSizes.typeValue t, Refreshable r, MouseControlPointsListener ml, MinMeanMaxValuesAndSizes mmmValuesAndSizes,/*Resizer resizer,*/ int px1, int py1, int radius, int lineLength/*, double value*/) {
        this(t, r, ml, mmmValuesAndSizes, px1, py1, radius, lineLength, null, null);
    }

    public OMResizeableDisc(MinMeanMaxValuesAndSizes.typeValue t, Refreshable r, MouseControlPointsListener ml, MinMeanMaxValuesAndSizes mmmValuesAndSizes, int px1, int py1, int radius, int lineLength, Paint lineColor, Paint fillColor) {
        super(r, ml, ""); // empty name as we don't want a GUI for it.
        this.mmmValuesAndSizes = mmmValuesAndSizes;
        this.type = t;
//        this.resizer = resizer;
//        this.value = value;
        omDisc = new OMCircle(px1, py1, 2 * radius, 2 * radius);
        if (lineColor != null) {
            omDisc.setLinePaint(lineColor);
        }
        if (fillColor != null) {
            omDisc.setFillPaint(fillColor);
        }
//        addOMControlPoint(new OMControlNPoint(px1, py1,
//                OMResizeableGraphic.OMControlPointRadius,
//                Color.blue, Color.red, refreshable.getMapBean()));
        // System.out.println("OMResizeableDisc ["+px1+", "+py1+"] "+radius);
        omLine = new OMLine(px1, py1 - radius, px1 + lineLength, py1 - radius);
        if (lineColor != null) {
            omLine.setLinePaint(lineColor);
        }
        add(omDisc);
        add(omLine);

        //System.out.println("TAILLE DISC = " + omDisc.getWidth());
    }

    public void translate(int dx, int dy) {
        //System.out.println("Trasnlate: " + dx + ";" + dy);
        omDisc.setX(omDisc.getX() + dx);
        omDisc.setY(omDisc.getY() + dy);

        int[] linePts = omLine.getPts();
        linePts[0] += dx;
        linePts[2] += dx;
        linePts[1] += dy;
        linePts[3] += dy;

//        for (int i = 0; i < omControlPoints.size(); i++) {
//            ((OMControlPoint) omControlPoints.getOMGraphicAt(i)).translate(dx, dy);
//        }
    }

    public void extend(int dw, int dh) {
        omDisc.setWidth(omDisc.getWidth() + dw);
        omDisc.setHeight(omDisc.getHeight() + dh);
        ////translate(dw, dh);
    }

//     public Properties removeAnchorFromProperties(String prefix, Properties props) {
//
//        return props;
//    }
    public int getXAnchor() {
        return omDisc.getX() - omDisc.getWidth();
    }

    public int getYAnchor() {
        return omDisc.getY() - omDisc.getHeight();
    }

    public int getWidth() {
        return omDisc.getWidth();
    }

    public int getHeight() {
        return omDisc.getHeight();
    }

//<<<<<<< .mine
    public void setRadius(int r) {
        omDisc.setWidth(r * 2);
        omDisc.setHeight(r * 2);
    }

//    @Override
//    public JComponent getGUI() {
//
//
//
//        JPanel p = new JPanel();
//        SpinnerNumberModel spModel = new SpinnerNumberModel();
//        final JSpinner spinValue = new JSpinner(spModel);
//        spinValue.setValue(mmmValuesAndSizes.getNumericValue(type.getIndex()));
//
//        spinValue.addChangeListener(new ChangeListener() {
//
//            public void stateChanged(ChangeEvent arg0) {
//                int value = (Integer) spinValue.getValue();
//                setRadius((int) mmmValuesAndSizes.getResizer().getSize(value));
//                //System.out.println("SET OMDISC WIDTH = " + (int) mmmValuesAndSizes.getResizer().getSize(value) + ", value = " + value + ", type=" + type);
//                //omDisc.setHeight((int)mmmValuesAndSizes.getResizer().getSize(value) * 2);
//                mmmValuesAndSizes.setValue(type, value);
//
//                refreshGraphics();
//            }
//        });
//
//        p.add(new JLabel("Value: "));
//        p.add(spinValue);
//        return p;
//    }
//=======
//>>>>>>> .r489

        public void setProperties(String prefix, Properties props) {
        if (props == null) {
            return;
        }
        prefix = PropUtils.getScopedPropertyPrefix(prefix);
        //Position
        translate(
                PropUtils.intFromProperties(props, prefix + X_ANCHOR_PROPERTY_NAME, getXAnchor()) - getXAnchor(),
                PropUtils.intFromProperties(props, prefix + Y_ANCHOR_PROPERTY_NAME, getYAnchor()) - getYAnchor());


        Boolean v = PropUtils.booleanFromProperties(props, prefix + VISIBILITY_PROPERTY_NAME, getVisible());
        //System.out.println("(SET) "+prefix + VISIBILITY_PROPERTY_NAME+" "+v);
        setVisible(v);
        // System.out.println("ResizeableGraphic setProps " + this);
    }

    public Properties getProperties(String prefix, Properties props) {
        if (props == null) {
            props = new Properties();
        }
        prefix = PropUtils.getScopedPropertyPrefix(prefix);

        //Position
        props.put(prefix + X_ANCHOR_PROPERTY_NAME, Integer.toString(getXAnchor()));
        props.put(prefix + Y_ANCHOR_PROPERTY_NAME, Integer.toString(getYAnchor()));


        //visibility
        // System.out.println("(GET) "+prefix + VISIBILITY_PROPERTY_NAME+" "+getVisible());
        props.put(prefix + VISIBILITY_PROPERTY_NAME, String.valueOf(getVisible()));

       // System.out.println(getClass()+ " " +prefix + WIDTH_PROPERTY_NAME+"="+Integer.toString(getWidth()));
        return props;
    }
}
