// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.legend;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.*;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.omGraphics.*;
import java.awt.Paint;
import java.util.Properties;

public class OMResizeableMultiRegularPolygonLegend extends OMResizeableFrame {

    public OMResizeableMultiRegularPolygonLegend(Refreshable r, MouseControlPointsListener ml, int x0, int y0, int nbEdges, float radius, float startAngle, int starCoef, Paint lineColor, Paint fillColor, String dataColumnName, double valuePerUnit, String name) {
	super(r, ml, x0, y0, 0, 0, name); // we'll set width and height at the end of the ctor.

	init(r, ml, x0, y0, nbEdges, radius, startAngle, starCoef, lineColor, fillColor, dataColumnName, valuePerUnit);
    }

    public void init(Refreshable r, MouseControlPointsListener ml, int x0, int y0, int nbEdges, float radius, float startAngle, int starCoef, Paint lineColor, Paint fillColor, String dataColumnName, double valuePerUnit) {

	Properties props = new Properties();
	props = getProperties(" ", props);
	clear(); // resizeableGraphics.clear();	graphics.clear();
	int offset = 5;
	omTitle = new OMResizeableText(refreshable, ml, 
				       x0, y0, 
				       dataColumnName, OMText.JUSTIFY_CENTER, "title");
	omTitle.setFontStyle(java.awt.Font.BOLD);
	y0 += omTitle.getHeight();
	omTitle.translate(0, y0);
	this.add(omTitle);


	x0 += 1.2*radius;
	y0 += 1.5*radius;
	// int x0 = getAnchor() + 1.2*radius;
	// int y0 = getYAnchor() + 1.5*radius;
	
	// OMResizeableRegularPolygon 
	omPoly = 
	    new OMResizeableRegularPolygon(r, ml, x0, y0, nbEdges, 
					   radius, startAngle, starCoef, lineColor, fillColor);
	this.add(omPoly);
	x0 += 1.2*radius;
	
	//	OMResizeableText 
	omText = 
	    new OMResizeableText(r, ml, x0, y0,
				     String.valueOf(valuePerUnit), //String.format(" %3g %s", valuePerUnit, dataColumnName),
				     OMText.JUSTIFY_LEFT, dataColumnName);
	y0 += omText.getHeight()/2;
	omText.translate(0, omText.getHeight()/2);
	
	int gap = omText.getHeight() - (y0 - getYAnchor()); 
	if (gap > 0) { // the text will be outside of the top frame 
	    omPoly.translate(0, gap); 
	    omText.translate(0, gap); 
	    y0 += gap;
	}

	this.add(omText);
	this.extendBorder((int)Math.max(omTitle.getWidth(), x0+omText.getWidth()+.2*radius), (int)(y0+1.1*radius));

	omTitle.translate(this.getWidth()/2, 0);
    }


    //    public void extend(int dw, int dh) {}


    //===============================================================
    // PROPERTIES
    //===============================================================
    private OMResizeableRegularPolygon omPoly;
    private OMResizeableText omText;
    private OMResizeableText omTitle;

    public static final String POLY_PROPERTY_NAME = "multipoly.poly";
    public static final String TEXT_PROPERTY_NAME = "multipoly.text";
    public static final String TITLE_PROPERTY_NAME = "multipoly.title";


    public void setProperties(String prefix, Properties properties) {
	super.setProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);

	if (omPoly != null)
	    omPoly.setProperties(prefix + POLY_PROPERTY_NAME, properties);

	if (omText != null)
	    omText.setProperties(prefix + TEXT_PROPERTY_NAME, properties);
    }

    public Properties getProperties(String prefix, Properties properties) {
	properties = super.getProperties(prefix, properties);
	prefix = PropUtils.getScopedPropertyPrefix(prefix);

	if (omPoly != null)
	    properties = omPoly.getProperties(prefix + POLY_PROPERTY_NAME, properties);

	if (omText != null)
	    properties = omText.getProperties(prefix + TEXT_PROPERTY_NAME, properties);

	return properties;
    }


}
