/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu;

import com.bbn.openmap.event.MapMouseListener;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.PropUtils;
import fr.unicaen.openmap.dataStorage.DataStorageIterator;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicConfPanel;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicGUI;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ThematicMouseControlPointsListener;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.Offseter;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.util.OffseterListener;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.Properties;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;

/**
 *Offsetable thematic
 * @author jchauvea
 */
public class OffsetableRaw extends Raw implements OffseterListener {

    protected Offseter offseter;
    protected ThematicMouseControlPointsListener mouseControlPointsListener;
    protected boolean offsetsOn;
    protected Box box;
    public static String offsetsOnPropertyString = "offsetsOn";

    public OffsetableRaw(ThematicDataDisplayLayer layer, String name) {
        this(layer, name, "");

    }

    public OffsetableRaw(ThematicDataDisplayLayer layer, String name, String typeName) {
        super(layer, name, typeName);
        offseter = new Offseter(0, 0);
        mouseControlPointsListener = new ThematicMouseControlPointsListener(this);
    }

    public void offseterChanged(Offseter offseter) {
        if (this.offseter == offseter) {
            refresh();
        }
    }

    public void enableOffsets(DataStorageIterator dsi) {
        offsetsOn = true;
        offseter.setColumnNames(dsi.getDataColumnNames());
        offseter.addOffseterListener(this);
        updateGUI();


    }

    public void disableOffsets() {
        offsetsOn = false;
        offseter.removeOffseterListener(this);
        updateGUI();
    }

    @Override
    public OMGraphicList getGraphics(DataStorageIterator dsi, Projection proj) {
        return super.getGraphics(dsi, proj);
    }

    @Override
    public Properties getProperties(String prefix, Properties properties) {
        properties = super.getProperties(prefix, properties);
        properties.put(prefix + offsetsOnPropertyString, String.valueOf(offsetsOn));
        properties = offseter.getProperties(prefix, properties);
        return properties;
    }

    @Override
    public void setProperties(String prefix, Properties properties) {
        super.setProperties(prefix, properties);
        offsetsOn = PropUtils.booleanFromProperties(properties, prefix + offsetsOnPropertyString, false);
        if (offsetsOn) {
            offseter.setProperties(prefix, properties);
        }
    }

    @Override
    public JComponent getGUI(final DataStorageIterator dsi) {
        thematicGUI = (ThematicGUI) super.getGUI(dsi);
        box = Box.createVerticalBox();
        final JCheckBox checkOffset = new JCheckBox(i18n.get(Offseter.class, "enabledManual", "Enable manual placement"));
        checkOffset.setSelected(offsetsOn);
        checkOffset.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                if (checkOffset.isSelected()) {
                    enableOffsets(dsi);
                } else {
                    disableOffsets();
                }
            }
        });
        JPanel pCheck = new JPanel();
        pCheck.add(checkOffset);
        box.add(pCheck);
        ThematicConfPanel confPan =
                ThematicConfPanel.createThematicConfPanel(i18n.get(this, "Positions", "Positions"));
        confPan.collapse();
        confPan.setContent(box);
        thematicGUI.addThematicConfigurationPanel(confPan);
        JPanel wrap = new JPanel();
        wrap.add(confPan);
        thematicGUI.add(wrap);
        if (offsetsOn) {
            offseter.setColumnNames(dsi.getDataColumnNames());
        }
        if (offsetsOn) {
            enableOffsets(dsi);
        } else {
            disableOffsets();
        }
        return thematicGUI;
    }

    public void updateGUI() {
        if (offsetsOn) {
            box.add(offseter.getGUI());
        } else {
            box.remove(offseter.getGUI());
        }
        box.setVisible(false);
        box.setVisible(true);
        thematicGUI.setVisible(false);
        thematicGUI.setVisible(true);
    }

    public ThematicMouseControlPointsListener getMouseControlPointsListener() {
        return mouseControlPointsListener;
    }

    @Override
    public MapMouseListener getMapMouseListener() {
        //super.getMapMouseListener()
        return mouseControlPointsListener;
    }

    public Offseter getOffseter() {
        return offseter;
    }
//    public void setProjection(Projection proj){
//        super.setProjection(proj);
//        offseter.setProjection(proj);
//    }
}
