package fr.unicaen.openmap.layer.wms;

import com.bbn.openmap.plugin.wms.WMSPlugIn;

import com.bbn.openmap.MapBean;
import com.bbn.openmap.MouseDelegator;
import com.bbn.openmap.gui.DefaultHelpMenu;
import java.util.Properties;
import com.bbn.openmap.layer.OMGraphicHandlerLayer;
import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.proj.*;
import com.bbn.openmap.proj.coords.LatLonPoint;
import com.bbn.openmap.util.*;

import fr.unicaen.openmap.util.Util;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import javax.imageio.ImageIO;
import java.awt.image.*;
import java.awt.geom.Point2D;
import java.beans.beancontext.BeanContext;
import javax.swing.JOptionPane;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.image.WMTConstants;
import com.bbn.openmap.proj.Projection;
import com.bbn.openmap.util.Debug;
import java.util.StringTokenizer;
import java.awt.geom.Point2D;
import java.util.NoSuchElementException;

/**
 * Sample implementation of what we need to display a raster image of the Earth...
 */
public class WMSLayer extends WMSPlugIn {
 
    /**
     * Use I18N mecanism to allow internationalization.
     * Use I18N.properties file or I18N_fr_FR.properties for french version
     **/
    protected I18n i18n = new BasicI18n();
    public WMSLayer() {
        Debug.message("WMSLayer", this + " new WMSLayer()");
    }
    
    /**
     * Layer class name, found unsing java.lang.reflect.
     */
    protected String classNameForDebug = Util.className(this);
    
  
    /**
     * Sample function to render the layer as a string. You don't need
     * to override this method to match the name of your subclass since
     * we use java.lang.reflect to find the class name.
     */
    public String toString() { return "[" + classNameForDebug + ":" + getName() + "]"; }
    
//   /**
//    * Basic sample implementation. You should override this
//    * function to match your layer interfaces need.
//    */
//   public Component getGUI() {
//	return null;
//   }

//    /**
//     * Used to redraw all elements.
//     */
//    public void refresh() {
//        Debug.message(classNameForDebug, this + " refresh()");
//        // use Swing Worker ...
//        doPrepare();        
//    }
//    
//    /**
//     * Create the OMGraphics to be displayed by the MapBean.
//     *
//     * @return OMGraphicList
//     */
//    public synchronized OMGraphicList prepare() {
//        Projection proj = getProjection();
//	if (proj == null) {
//            Debug.message(classNameForDebug, this + " no projection yet, skipping prepare()");
//            return null;
//        }
//        OMGraphicList omgl = null;
//        setList(null);
//        Debug.message(classNameForDebug, this + " prepare() used !");
//
//        omgl = generateGraphics();
//	omgl.generate(proj);
//
//        setList(omgl);
//	Debug.message(classNameForDebug, this + " " + omgl.size() + " OMGraphics");
//	return omgl;
//    }
//
//    protected BufferedImage image = null;
//
//    protected void initImage() {
//	if (image != null)
//	    return;
//	try {
//	    image = ImageIO.read(Util.open(imageFileName));
//	} catch (java.io.IOException ioe) {
//	    System.err.println("RasterLayer: can't open "+imageFileName);
//	}
//    }
//    
//
//    protected Projection proj = null;
//
//    protected Point2D p = null; // will be allocated at the first call of inverse and then reused.
//    protected int getPixel(int x, int y) {
//	p = proj.inverse(x, y, p);
//	double lon = p.getX();
//	double lat = p.getY();
//
//	try {
//	    return image.getRGB
//		((int)Math.round(image.getMinX() + image.getWidth() * (lon - lonMin) / (lonMax - lonMin)),
//		 (int)Math.round(image.getMinY() + image.getHeight() * (lat - latMin) / (latMax - latMin)));
//	} catch (ArrayIndexOutOfBoundsException e) {
//		return 0; // black
//	}
//    }
//
////    protected int filter(int i){
////        return i + 80 << 8;
////    }
//
//    public OMGraphicList generateGraphics() {
//	
//	initImage();
//	if (image == null) 
//	    return null;
//
//	proj = getProjection();
//	if (proj == null) {
//	    System.err.println("RasterLayer: no projection has been set");
//	    return null;
//	}
//
//	int w = proj.getWidth();
//	int h = proj.getHeight();
//
//	BufferedImage bim = new BufferedImage (w, h, BufferedImage.TYPE_INT_ARGB);
//	for(int x = 0; x < w; x++)
//	    for (int y = 0; y < h; y++)
//		bim.setRGB(x, y, getPixel(x, y));
//	    
//	OMRaster omr = new OMRaster(0, 0, bim);
//	OMGraphicList omgl = new OMGraphicList();
//	omgl.add(omr);
//
//	return omgl;
//    }

    private double  lonMin = -180 , lonMax = +180, latMin = 90, latMax = -90;

       /**
     * Create the query to be sent to the server, based on current settings.
     */
    public String createQueryString(Projection p) {

        if (queryHeader == null) {
            return null;
        }

        String bbox = "-180,-90,180,-90";
        String height = "1000";
        String width = "600";

        if (p != null) {
            Point2D ul = p.getUpperLeft();
            Point2D lr = p.getLowerRight();
	    lonMin = ul.getX();
	    lonMax = lr.getX();
	    latMin = ul.getY();
	    latMax = lr.getY();
            bbox = Double.toString(lonMin) + ","
                    + Double.toString(latMax) + ","
                    + Double.toString(lonMax) + ","
                    + Double.toString(latMin);
            height = Integer.toString(p.getHeight());
            width = Integer.toString(p.getWidth());
        }

        StringBuffer buf = new StringBuffer(queryHeader);
        buf.append("?" + WMTConstants.VERSION + "=" + wmsVersion + "&"
                + WMTConstants.REQUEST + "=" + mapRequestName + "&"
                + WMTConstants.SRS + "=" + "EPSG:4326" + "&"
                + WMTConstants.BBOX + "=" + bbox + "&" + WMTConstants.HEIGHT
                + "=" + height + "&" + WMTConstants.WIDTH + "=" + width + "&"
                + WMTConstants.EXCEPTIONS + "=" + errorHandling);

        if (imageFormat != null) {
            buf.append("&" + WMTConstants.FORMAT + "=" + imageFormat);

            String baseImageFormat = imageFormat;
            if (baseImageFormat.indexOf('/') > 0)
                baseImageFormat = baseImageFormat.substring(baseImageFormat.indexOf('/'));
            if (baseImageFormat.equals(WMTConstants.IMAGEFORMAT_JPEG)) {
                buf.append("&quality=" + imageQuality);
            }
        }

        if (transparent != null) {
            buf.append("&" + WMTConstants.TRANSPARENT + "=" + transparent);
        }

        if (backgroundColor != null) {
            buf.append("&" + WMTConstants.BGCOLOR + "=" + backgroundColor);
        }

        if (layers != null) {
            buf.append("&" + WMTConstants.LAYERS + "=" + layers);
        }

        if (styles != null) {
            buf.append("&" + WMTConstants.STYLES + "=" + styles);
        }

        if (Debug.debugging("wms")) {
            Debug.output("query string: " + buf);
        }

        /*
         * Included to allow for one or more vendor specific parameters to be
         * specified such as ESRI's ArcIMS's "ServiceName" parameter.
         */
        if (vendorSpecificNames != null) {
            if (vendorSpecificValues != null) {
                StringTokenizer nameTokenizer = new StringTokenizer(vendorSpecificNames, ",");
                StringTokenizer valueTokenizer = new StringTokenizer(vendorSpecificValues, ",");
                String paramName = null;
                String paramValue = null;
                while (nameTokenizer.hasMoreTokens()) {
                    try {
                        paramName = nameTokenizer.nextToken();
                        paramValue = valueTokenizer.nextToken();
                        buf.append("&" + paramName + "=" + paramValue);
                    } catch (NoSuchElementException e) {
                        if (Debug.debugging("wms")) {
                            Debug.output("WMSPlugIn.getRectangle(): "
                                    + "parameter \"" + paramName
                                    + "\" has no value");
                        }
                    }
                }
            }
        }
        return buf.toString();
    }

    /**
     * A simple rewrite of WebImagePlugIn.getRectangle() 
     * that transforms the rectangle image usig the current projection.
     * Don't use super.getRectangle() to avoid an extra call to generate().
     */
    public OMGraphicList getRectangle(Projection p) {
	// super.getRectangle(p);
	currentProjection = p;

	BufferedImage image = getImage(p);
	Debug.message("wms", "WMS proj:"+p+" image: "+image);
	if (image == null)
	    return new OMGraphicList();

	int w = p.getWidth();
	int h = p.getHeight();
	
	// pb if image has not w x h dimensions
	if (w != image.getWidth() || h != image.getHeight()) {
	    OMRaster omr = new OMRaster(0, 0, image);
	    OMGraphicList omgl = new OMGraphicList();
	    omgl.add(omr);
	    omgl.generate(p);
	    return omgl;	
	}

	BufferedImage bim = new BufferedImage (w, h, BufferedImage.TYPE_INT_ARGB);
	for(int x = 0; x < w; x++)
	    for (int y = 0; y < h; y++)
		bim.setRGB(x, y, getPixel(x, y, image));
	    
	OMRaster omr = new OMRaster(0, 0, bim);
	OMGraphicList omgl = new OMGraphicList();
	omgl.add(omr);
	omgl.generate(p);
	return omgl;	
    }
    
    private Point2D p = null; // will be allocated at the first call of inverse and then reused.
    protected int getPixel(int x, int y, BufferedImage image) {
	p = currentProjection.inverse(x, y, p);
	double lon = p.getX();
	double lat = p.getY();

	try {
	    return image.getRGB
		((int)Math.round(image.getMinX() + image.getWidth() * (lon - lonMin) / (lonMax - lonMin)),
		 (int)Math.round(image.getMinY() + image.getHeight() * (lat - latMin) / (latMax - latMin)));
	} catch (ArrayIndexOutOfBoundsException e) {
		return 0; // black
	}
    }


    public BufferedImage getImage(Projection p) {

        String urlString = createQueryString(p);

        if (Debug.debugging("plugin")) {
            Debug.output("WebImagePlugIn.getRectangle() with \"" + urlString
                    + "\"");
        }

        if (urlString == null) {
            return null;
        }

        java.net.URL url = null;

        try {
            url = new java.net.URL(urlString);
            java.net.HttpURLConnection urlc = (java.net.HttpURLConnection) url.openConnection();

            if (Debug.debugging("plugin")) {
                Debug.output("url content type: " + urlc.getContentType());
            }

            if (urlc == null || urlc.getContentType() == null) {
                if (layer != null) {
                    layer.fireRequestMessage(getName()
                            + ": unable to connect to " + getServerName());
                } else {
                    Debug.error(getName() + ": unable to connect to "
                            + getServerName());
                }
                return null;
            }

            // text
            if (urlc.getContentType().startsWith("text")) {
                java.io.BufferedReader bin = new java.io.BufferedReader(new java.io.InputStreamReader(urlc.getInputStream()));
                String st;
                String message = "";
                while ((st = bin.readLine()) != null) {
                    message += st;
                }

                // Debug.error(message);
                // How about we toss the message out to the user
                // instead?
                if (layer != null) {
                    layer.fireRequestMessage(message);
                }

                // image
            } else if (urlc.getContentType().startsWith("image")) {
		return ImageIO.read(urlc.getInputStream());
            } // end if image
        } catch (java.net.MalformedURLException murle) {
            Debug.error("WebImagePlugIn: URL \"" + urlString
                    + "\" is malformed.");
        } catch (java.io.IOException ioe) {
            JOptionPane.showMessageDialog(null,
                    getName() + ":\n\n   Couldn't connect to "
                            + getServerName(),
                    "Connection Problem",
                    JOptionPane.INFORMATION_MESSAGE);

        }
	return null;
    }

}
