// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.tools.drawing;

import com.bbn.openmap.InformationDelegator;
import com.bbn.openmap.MapHandler;
import com.bbn.openmap.omGraphics.GraphicAttributes;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.tools.drawing.DrawingTool;
import com.bbn.openmap.tools.drawing.DrawingToolRequestor;
import com.bbn.openmap.tools.drawing.EditToolLoader;
import com.bbn.openmap.tools.drawing.OMDrawingTool;
import com.bbn.openmap.tools.drawing.OMDrawingToolLauncher;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PaletteHelper;
import fr.unicaen.openmap.dataStorage.EditableDataStorage;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

/**
 *
 * @author jchauvea
 */
public class TMDrawingToolLauncher extends OMDrawingToolLauncher {


    public TMDrawingToolLauncher(){
        super();
        defaultGraphicAttributes.setLineType(OMGraphic.LINETYPE_STRAIGHT);
    }


    /*String[] rtc = {
        i18n.get(OMDrawingToolLauncher.class,
        "renderingType.LatLon",
        "Lat/Lon"),
        i18n.get(OMDrawingToolLauncher.class, "renderingType.XY", "X/Y"),
        i18n.get(OMDrawingToolLauncher.class,
        "renderingType.XYOffset",
        "X/Y Offset")};*/

    /**
     * Set the component that will receive the new/edited OMGraphic from the
     * DrawingTool. Does not change the GUI. Called when the combo box changes.
     *
     * @param name GUI pretty name of requestor.
     */
    public void setCurrentRequestor(DrawingToolRequestor r) {
        currentRequestor = r;
    }

    /**
     * Build the stuff that goes in the launcher.
     */
    public void resetGUI() {
        removeAll();

        JPanel palette = new JPanel();
        palette.setLayout(new BoxLayout(palette, BoxLayout.Y_AXIS));
        palette.setAlignmentX(Component.CENTER_ALIGNMENT); // LEFT
        palette.setAlignmentY(Component.CENTER_ALIGNMENT); // BOTTOM

        String[] requestorNames = new String[drawingToolRequestors.size()];

        if (Debug.debugging("omdtl")) {
            Debug.output("Have " + requestorNames.length + " REQUESTORS");
        }

        for (int i = 0; i < requestorNames.length; i++) {
            requestorNames[i] = ((DrawingToolRequestor) drawingToolRequestors.elementAt(i)).getName();
            if (requestorNames[i] == null) {
                Debug.output("OMDrawingToolLauncher has a requestor that is unnamed.  Please assign a name to the requestor");
                requestorNames[i] = "-- Unnamed --";
            }
            if (Debug.debugging("omdtl")) {
                Debug.output("Adding REQUESTOR " + requestorNames[i] + " to menu");
            }
        }


        JPanel panel = PaletteHelper.createPaletteJPanel(i18n.get(EditableDataStorage.class,
                "graphicType",
                "Graphic Type:"));
        panel.add(getToolWidgets(useTextEditToolTitles));
        palette.add(panel);


        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.CENTER;

        panel3 = new JPanel();
        add(palette);


    }

    @Override
    protected JComponent getToolWidgets(boolean useText) {

        if (useText) {
            // Set editables with all the pretty names.
            Vector<String> editables = new Vector<String>();

            for (LoaderHolder lh : loaders) {
                editables.add(lh.prettyName);
            }

            return null;//createToolOptionMenu(editables);
        } else {
            return createToolButtonPanel();
        }
    }

    
    private JPanel createToolButtonPanel() {
        // Otherwise, create a set of buttons.
        JPanel panel = new JPanel();
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;

        panel.setLayout(gridbag);

        ButtonGroup bg = new ButtonGroup();

        int toolbarCount = 0;
        boolean limitWidth = false;
        if (maxHorNumLoaderButtons >= 0) {
            limitWidth = true;
        }

        JToggleButton btn;
        JToolBar iconBar = null;
        boolean activeSet = false;
        for (LoaderHolder lh : loaders) {

            if (toolbarCount == 0) {
                iconBar = new JToolBar();
                iconBar.setFloatable(false);
                gridbag.setConstraints(iconBar, c);
                panel.add(iconBar);
            }

            String pName = lh.prettyName;
            EditToolLoader etl = lh.loader;
            ImageIcon icon = etl.getIcon(getEditableClassName(pName));

            btn = new JToggleButton(icon, !activeSet);
            btn.setToolTipText(pName);
            btn.setActionCommand(pName);
            btn.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent ae) {
                    setCurrentCreation(ae.getActionCommand());
                    createGraphic();
                }
            });
            bg.add(btn);

            if (iconBar != null) {
                iconBar.add(btn);
            }

            toolbarCount++;

            // Just set one as active, the first one.
            if (!activeSet) {
                setCurrentCreation(pName);
                activeSet = true;
            }

            if (limitWidth && toolbarCount >= maxHorNumLoaderButtons) {
                toolbarCount = 0;
            }
        }

        return panel;
    }

    /**
     * Create graphic
     */
    private void createGraphic() {
        // Get the active EditToolLoader
        DrawingTool dt = getDrawingTool();
        if (dt instanceof OMDrawingTool) {
            OMDrawingTool omdt = (OMDrawingTool) dt;

            if (omdt.isActivated()) {
                omdt.deactivate();
            }
        }

        if (dt != null && currentCreation != null && currentRequestor != null) {
            // Copy the default GraphicAttributes into another
            // copy...
            GraphicAttributes ga = (GraphicAttributes) defaultGraphicAttributes.clone();

            // fire it up!
            dt.setBehaviorMask(OMDrawingTool.DEFAULT_BEHAVIOR_MASK);
            dt.create(currentCreation, ga, currentRequestor);
        } else {

            StringBuffer sb = new StringBuffer();
            StringBuffer em = new StringBuffer();

            if (dt == null) {
                sb.append("   No drawing tool is available!\n");
                em.append(i18n.get(OMDrawingToolLauncher.class,
                        "noDrowingTool",
                        "   No drawing tool is available!\n"));
            } else {
                sb.append("   Drawing tool OK.\n");
            }

            if (currentCreation == null) {
                sb.append("   No valid choice of graphic to create.\n");
                em.append(i18n.get(OMDrawingToolLauncher.class,
                        "noValidChoice",
                        "   No valid choice of graphic to create.\n"));
            } else {
                sb.append("   Graphic choice OK.\n");
            }

            if (currentRequestor == null) {
                sb.append("   No valid receiver for the created graphic.\n");
                em.append(i18n.get(OMDrawingToolLauncher.class,
                        "noValidReceiver",
                        "   No valid receiver for the created graphic.\n"));
            } else {
                sb.append("   Graphic receiver OK.\n");
            }

            Debug.output("OMDrawingToolLauncher: Something is not set:\n" + sb.toString());

            MapHandler mapHandler = (MapHandler) getBeanContext();
            if (mapHandler != null) {
                InformationDelegator id = (InformationDelegator) mapHandler.get("com.bbn.openmap.InformationDelegator");
                if (id != null) {

                    id.displayMessage(i18n.get(OMDrawingToolLauncher.class,
                            "problem",
                            "Problem"),
                            i18n.get(OMDrawingToolLauncher.class,
                            "problemCreatingGraphic",
                            "Problem creating new graphic:\n") + em.toString());
                }
            }
        }
    }

    @Override
    protected void updateDrawingAttributesGUI() {
    }
}
