// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.util;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.JOptionPane;

/**
 *Generate HTML Code in order to visualize generated images from ThemaMap
 * @author jchauvea
 */
public class HtmlGenerator {

    //public static int nbStep;
    public final static String animationPrefix = "AnimationExport-";
    public final static String benchtestPrefix = "Benchtest-";

    public static String getHTML(ArrayList<String> images, ImageExporter.ImageType t) {
        String tab ="images = [";
        for(String im : images){
            tab += "'"+im + "',";
        }
        tab = tab.substring(0, tab.length() - 1);//remove last comma
        tab += "];";

        String src;
        if(t == ImageExporter.ImageType.JPG){
            src = "\"<img src='\"+images[index]+\"'/>\"";
        }
        else{
           src="\"<embed width='1024px' height='768px' src='\"+images[index]+\"' type='image/svg+xml' />\"";
        }

        return "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n<html>\n"
                + "<head>\n"
                + "<title></title>\n"
                + "</head>\n"
                + "<script>\n"
                + "index = 0;\n"
                + tab + "\n"
                + "function next(){ \n"
                + " document.getElementById(\"carte\").innerHTML="+src+";"
                + "index = index + 1;\n"
                + "}\n"
                + "function animation(){\n"
                //+ "document.getElementById(\"compteur\").innerHTML = index;\n"
                //+ "document.getElementById(\"carte\").innerHTML="+src+";\n"
                //+ "index = index + 1;\n"
                + "next();"
                + "time = parseInt(document.getElementById(\"select\").value) * 1000;"
                + "if(index<" + images.size() + "){\n"
                + "setTimeout(\"animation()\",time);\n"
                + "}\n}\n"
                + "</script>\n"
                + "<body>\n"
                + "<h1>ThemaMap</h1>\n"
                + "<input type=\"button\" value=\"Start\" onClick=\"animation()\"/>\n "
                + "<select id=\"select\"><option>1</option><option>2</option><option>3</option><option>4</option><option>5</option></select>"
                + "<input type=\"button\" value=\"next >>\" onClick=\"next();\" /> \n "
                + "<div id=\"carte\"></div>\n"
                + "</body>\n"
                + "</html>\n";
    }

    public static void generateHTML(String folder, String fileName, ImageExporter.ImageType t) {
        ArrayList<String> images = new ArrayList<String>();
        File fileFolder = new File(folder);
        String files[] = fileFolder.list();
        for(String f : files){
            if (f.endsWith(t.toString().toLowerCase())){
                images.add(f);
            }
        }
        writeHTML(folder+"/"+fileName, getHTML(images, t));
    }
//
//    public static void generateBenchTestHTML(String folderPath, String htmlFile, ImageExporter.ImageType t) {
//        File folder = new File(folderPath);
//        if (!folder.isDirectory()) {
//            return;
//        }
//        ArrayList<String> images = new ArrayList<String>();
//        String files[] = folder.list();
//        for (String f : files) {
//            if (f.endsWith(t.toString().toLowerCase())) {
//                images.add(f);
//            }
//        }
//        writeHTML(folderPath+ "/" + htmlFile, getHTML(images, t));
//    }

    public static void writeHTML(String fileName, String html) {
        try {
            FileWriter writer = new FileWriter(fileName);
            writer.write(html, 0, html.length());
            writer.flush();
            JOptionPane.showMessageDialog(null, "Génération terminée");
        } catch (IOException ex) {
            ex.printStackTrace();
        }

    }
}


