// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors
 */
// </header>
package fr.unicaen.openmap.util;

import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.MapBean;
import com.bbn.openmap.MapHandler;
import com.bbn.openmap.image.SunJPEGFormatter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;

/**
 * Export map's OMgraphics from Themamap into an image file
 * @author jchauvea
 */
public class ImageExporter {

    protected static MapHandler mapHandler;

    public static enum ImageType {

        JPG, SVG
    }
    //protected static SVGFormatter formatter = new SVGFormatter();

    /**
     * map handler initialization
     * @param mh
     */
    public static void initMapHandler(MapHandler mh) {
        mapHandler = mh;
    }

    /**
     * Export into the specified fileName in SVG or JPEG format
     * @param fileName
     * @param type
     */
    public static void exportInFile(String fileName, ImageType type) {

            LayerHandler lh = (LayerHandler) mapHandler.get("com.bbn.openmap.LayerHandler");
            MapBean mb = (MapBean) mapHandler.get("com.bbn.openmap.MapBean");
            Layer[] layers = lh.getMapLayers();
            JFrame frame = new JFrame();
            frame.setSize(mb.getSize());
            mb.paint(mb.getGraphics());
            exportLayersInFile(fileName, type, layers, mb);

    }

    /**
     * Export a list of layers
     * @param fileName
     * @param type
     * @param layers
     * @param mb
     */
    public static void exportLayersInFile(String fileName, ImageType type, Layer[] layers, MapBean mb) {
        try {
            byte[] imageBytes = null;
            File f = new File(fileName);
            if (f.exists()) {
                f.delete();
            }
            switch (type) {
                case SVG:

                    SVGFormatter formatter = new SVGFormatter();
                    imageBytes = formatter.getTextFromLayers(layers,
                            mb.getWidth(),
                            mb.getHeight(),
                            mb.getBackground());
                    break;
                case JPG:
                    imageBytes = new SunJPEGFormatter().getImageFromMapBean(mb, mb.getWidth(), mb.getHeight());
                    break;
            }
            Logger.getLogger(ImageExporter.class.getName()).log(Level.INFO, null, "---------->EXPORT IMAGE " + type);
            FileOutputStream binFile = new FileOutputStream(fileName);
            binFile.write(imageBytes);
            binFile.close();
        } catch (IOException e) {
        } catch (NullPointerException ne) {
            ne.printStackTrace();
            exportLayersInFile(fileName, type, layers, mb);
        }

    }

    /**
     * 
     */
    public static void closeCurrentProject() {
        LayerHandler layerHandler = (LayerHandler) mapHandler.get("com.bbn.openmap.LayerHandler");
        for (Layer l : layerHandler.getLayers()) {

            layerHandler.removeLayer(l);
        }
    }
}
