// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util;

import java.awt.*;
import java.awt.print.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

/**
 * Print utilities.
 */
public class PrintTools {
    /**
     *
     */
    public static Font FONTBASE = new Font("Times New Roman", Font.PLAIN, 10);
    
    /**
     *
     */
    public static void printHeader(Graphics g, int page) {
	PrintJob job = ((PrintGraphics) g).getPrintJob();
	int pw = job.getPageDimension().width;
	int ph = job.getPageDimension().height;
	int pres = job.getPageResolution();
	int marge = pres * 2 / 5; // 1 cm
	int x = 0, y = 0;
	g.setColor(Color.black);
	Font ft = new Font(FONTBASE.getFamily(), Font.BOLD, FONTBASE.getSize());
	g.setFont(ft);
	FontMetrics fm= g.getFontMetrics();
	y += fm.getMaxAscent() + 10;
	g.drawString("OpenMap / Postgis Extension 0.1 (c) 2002 D.Gigan", x, y);
	y += fm.getMaxDescent();
	ft = FONTBASE;
	g.setFont(ft);
	fm = g.getFontMetrics();
	y+= fm.getMaxAscent();
	g.drawString(getDateString(), x, y);
	y += fm.getMaxDescent();
    }
    
    /**
     *
     */
    public static void printFooter(Graphics _g) {
	PrintJob pjob=((PrintGraphics)_g).getPrintJob();
	int pw = pjob.getPageDimension().width;
	int ph = pjob.getPageDimension().height;
	int pres = pjob.getPageResolution();
	int marge = pres * 2 / 5; // 1 cm
	int x = 0;
	int y = 0;
	Font ft;
	FontMetrics fm;
	_g.setColor(Color.black);
	ft = new Font(FONTBASE.getFamily(), Font.ITALIC, FONTBASE.getSize());
	_g.setFont(ft);
	fm = _g.getFontMetrics();
	y += fm.getMaxAscent();
	_g.drawString("http://www.info.unicaen.fr", x, y);
	y += fm.getMaxAscent() + fm.getMaxDescent() + 2;
	_g.drawString("http://www.etu.info.unicaen.fr", x, y);
    }
    
    public static void printComponent(Graphics _g, Component _c) {
	PrintJob pjob=((PrintGraphics)_g).getPrintJob();
	int      pw  =pjob.getPageDimension().width;
	int      ph  =pjob.getPageDimension().height;
	int      pres=pjob.getPageResolution();

	int      marge=pres*2/5; // 1 cm

	_g.setFont(FONTBASE);
	_g.setColor(Color.black);

	FontMetrics fm=_g.getFontMetrics();
	int x,y;
	int h=fm.getMaxAscent()+fm.getMaxDescent();

	x=marge;
	y=marge;
	_g.translate(x,y);
	printHeader(_g,1);
	_g.translate(-x,-y);
    
	x=marge;
	y=ph-marge-64;
	_g.translate(x,y);
	printFooter(_g);
	_g.translate(-x,-y);

	x=marge;
	y=marge+64+2*h;
	_g.translate(x,y);

	Shape     s=_g.getClip();
	Dimension d=_c.getSize();
	_g.setClip(0,0,d.width,d.height);
	_c.paint(_g);
	_g.setClip(s);
	_g.translate(-x,-y);
    }
    /**
     *
     */
    public static void printImage(Graphics _g, Image _c) {
	PrintJob pjob = ((PrintGraphics) _g).getPrintJob();
	int pw = pjob.getPageDimension().width;
	int ph = pjob.getPageDimension().height;
	int pres= pjob.getPageResolution();
	int marge = pres * 2 / 5; // 1 cm
	_g.setFont(FONTBASE);
	_g.setColor(Color.black);
	FontMetrics fm = _g.getFontMetrics();
	int x, y;
	int h = fm.getMaxAscent() + fm.getMaxDescent();
	x = marge;
	y = marge;
	_g.translate(x,y);
	printHeader(_g,1);
	_g.translate(-x,-y);
	x = marge;
	y = ph - marge - 64;
	_g.translate(x,y);
	printFooter(_g);
	_g.translate(-x,-y);
	x = marge;
	y = marge+64+2*h;
	_g.translate(x,y);
	Shape s = _g.getClip();
	int wi = _c.getWidth(null);
	int hi =_c.getHeight(null);
	boolean reduite = false;
	int wmax = pw - 2 * marge;
	int hmax = ph - 2 * (marge + 128 + 2 * h);
	if( wi > wmax) {
	    hi = hi * wmax / wi;
	    wi = wmax;
	    reduite = true;
	}
	if (hi > hmax) {
	    wi = wi * hmax / hi;
	    hi = hmax;
	    reduite = true;
	}
	_g.setClip(0, 0, wi, hi);
	if(reduite) _g.drawImage(_c, 0, 0, wi, hi,null);
	else _g.drawImage(_c, 0, 0, null);
	_g.setClip(s);
	_g.translate(-x,-y);
    }
    
    /**
     *
     */
    public static void printString(Graphics _g, String _t) {
	PrintJob pjob = ((PrintGraphics) _g).getPrintJob();
	int pw = pjob.getPageDimension().width;
	int ph = pjob.getPageDimension().height;
	int pres = pjob.getPageResolution();
	int marge = pres * 2 / 5; // 1 cm
	int page = 1;
	_g.setFont(FONTBASE);
	_g.setColor(Color.black);
	FontMetrics fm = _g.getFontMetrics();
	int x, y;
	int h = fm.getMaxAscent()+fm.getMaxDescent();
	x = marge;
	y = marge;
	_g.translate(x, y);
	printHeader(_g, 1);
	_g.translate(-x, -y);
	x = marge;
	y = ph - marge - 64;
	_g.translate(x, y);
	printFooter(_g);
	_g.translate(-x, -y);
	x = marge;
	y = marge + 64 + h;
	while(!_t.equals("")) {
	    int i=_t.indexOf("\n");
	    String s;
	    if(i >= 0) {
		s = _t.substring(0,i);
		_t = _t.substring(i+1);
	    } else {
		s = _t;
		_t = "";
	    }
	    _g.drawString(s, x, y);
	    y += h;
	    if( y > ph - marge - 64 -h) {
		_g.dispose();
		_g = pjob.getGraphics();
		_g.setFont(FONTBASE);
		_g.setColor(Color.black);
		fm = _g.getFontMetrics();
		h = fm.getMaxAscent() + fm.getMaxDescent();
		page++;
		x = marge;
		y = marge;
		_g.translate(x, y);
		printHeader(_g, 1);
		_g.translate(-x, -y);
		x = marge;
		y = ph - marge - 64;
		_g.translate(x,y);
		printFooter(_g);
		_g.translate(-x, -y);
		x = marge;
		y = marge + 64 + h;
	    }
	}
    }

    /**
     * Used to print a table.
     */
    public static void printTable(Graphics _g, TableModel _t) {
	PrintJob pjob=((PrintGraphics) _g).getPrintJob();
	int pw = pjob.getPageDimension().width;
	int ph = pjob.getPageDimension().height;
	int pres= pjob.getPageResolution();
	int marge = pres * 8 / 5; // 4 cm
	int page = 1;
	_g.setFont(FONTBASE);
	FontMetrics fm = _g.getFontMetrics();
	int x, y;
	int h1 = fm.getMaxAscent() + fm.getMaxDescent() + 4;
	int w1 = (pw - 2 * marge) / _t.getColumnCount();
	x = marge;
	y = marge;
	_g.translate(x, y);
	printHeader(_g, 1);
	_g.translate(-x, -y);
	x = marge;
	y = ph - marge - 64;
	_g.translate(x, y);
	printFooter(_g);
	_g.translate(-x, -y);
	x = marge;
	y = marge + 64 + h1;
	final Color GRIS1 = new Color(192,192,192);
	// final Color GRIS2 = new Color(224,224,224);
	for(int j = 0 ; j < _t.getColumnCount() ; j++) {
	    _g.setColor(GRIS1);
	    _g.fillRect(x, y, w1, h1);
	    _g.setColor(Color.black);
	    _g.drawRect(x, y, w1, h1);
	    String s = _t.getColumnName(j);
	    _g.drawString(s, x + 2, y + 2 + fm.getMaxAscent());
	    x += w1;
	}
	x = marge;
	y += h1;
	for(int i = 0 ; i < _t.getRowCount() ; i++) {
	    for(int j = 0 ; j < _t.getColumnCount() ; j++) {
		// _g.setColor(i % 2 == 1 ? GRIS2 : Color.white);
		// _g.fillRect(x, y, w1, h1);
		_g.setColor(Color.black);
		_g.drawRect(x, y, w1, h1);
		String s = _t.getValueAt(i,j).toString();
		_g.drawString(s, x + 2,y + 2 + fm.getMaxAscent());
		x += w1;
	    }
	    x = marge;
	    y += h1;
	    if( y > ph - marge - 64 - h1) {
		_g.dispose();
		_g = pjob.getGraphics();
		_g.setFont(FONTBASE);
		_g.setColor(Color.black);
		fm = _g.getFontMetrics();
		h1 = fm.getMaxAscent() + fm.getMaxDescent() + 4;
		w1 = (pw - 2 * marge) / _t.getColumnCount();
		page++;
		x = marge;
		y = marge;
		_g.translate(x, y);
		printHeader(_g, 1);
		_g.translate(-x, -y);
		x = marge;
		y = ph-marge-64;
		_g.translate(x,y);
		printFooter(_g);
		_g.translate(-x,-y);
		x = marge;
		y = marge + 64 + h1;
		for(int j = 0 ; j < _t.getColumnCount() ; j++) {
		    _g.setColor(GRIS1);
		    _g.fillRect(x,y,w1,h1);
		    _g.setColor(Color.black);
		    _g.drawRect(x, y, w1, h1);
		    String s = _t.getColumnName(j);
		    _g.drawString(s, x + 2, y + 2 + fm.getMaxAscent());
		    x += w1;
		}
		x = marge;
		y += h1;
	    }
	}
    }

    /**
     * 
     */
    public static String getDateString() {
	String [] ids = TimeZone.getAvailableIDs(+1 * 60 * 60 * 1000);
	SimpleTimeZone cet = new SimpleTimeZone(+1 * 60 * 60 * 1000, ids[0]);
	Date now = new Date();
	Calendar cal;
	DateFormat fmt;
	cet.setStartRule(Calendar.APRIL, 1, Calendar.SUNDAY, 2 * 60 * 60 * 1000);
	cet.setEndRule  (Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * 60 * 60 * 1000);
	cal=new GregorianCalendar(cet);
	cal.setTime(now);
	fmt=DateFormat.getDateInstance(DateFormat.LONG, Locale.getDefault());
	fmt.setCalendar(cal);
	return fmt.format(now);
    }
}
