// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.classification;

import java.util.ArrayList;
import java.util.Collections;
import com.bbn.openmap.util.*;


/**
 * Equal Step Classification : each class has the same width.
 * Try to fix min and max such as they appear to be "rounded" data.
 */
public class StepClassification extends ClassificationOfDouble {

    public StepClassification(ArrayList<Double> data, int nbClasses) {
	super(data, nbClasses);
    }
    
    public StepClassification(Classification c, int nbClasses) {
	super(c, nbClasses);
    }

    /** rounded values for min and max data values. */
    protected double min, max; 

    protected void init() {
	if (data == null) return;
	Collections.sort(data);
	double maxDataValue = data.get(data.size()-1);
	double minDataValue = data.get(0);
	Debug.message("Classification", "StepClassification maxDataValue = " + maxDataValue + " minDataValue = " + minDataValue);

	// let's round up max to a power of ten 
	int decimalOrderOfMax = maxDataValue > 0
	    ? (int)Math.ceil(Math.log10(maxDataValue)) 
	    : (int)Math.floor(Math.log10(-maxDataValue));
	double maxBasePower10 = (maxDataValue > 0 ? 1 : -1) * Math.pow(10, decimalOrderOfMax);
	max = maxBasePower10;
	// System.out.println("max -> " + max);
	// take care that (max - maxDataValue) is less than 10% of (max - min)
	while ((max - maxDataValue) > .1 * (maxDataValue - minDataValue)) {
	    maxBasePower10 /= 10;
	    max = Math.ceil(maxDataValue/maxBasePower10)*maxBasePower10; 
	    //	    System.out.println("max -> " + max);
	}

	// let's round down min to a power of ten
	int decimalOrderOfMin = minDataValue > 0
	    ? (int)Math.floor(Math.log(minDataValue)/Math.log(10))
	    : (int)Math.ceil(Math.log(-minDataValue)/Math.log(10));
	double minBasePower10 = (minDataValue > 0 ? 1 : -1) * Math.pow(10, decimalOrderOfMin);
	min = minBasePower10;
	//	System.out.println("min -> " + min);
	// take care that (min - minDataValue) is less than 10% of (max - min)
	while ((minDataValue - min) > .1 * (maxDataValue - minDataValue)) {
	    minBasePower10 /= 10;
	    min = Math.floor(minDataValue/minBasePower10)*minBasePower10; 
	    //	    System.out.println("min -> " + min);
	}
	
	// round down min to zero if necessary
	if (min > 0 && min < .1 * (maxDataValue - minDataValue))
	    min = 0;

	// at this point, min and max are rounded
	Debug.message("Classification", "StepClassification max = "+ max + " min = " + min + " maxDataValue = " + maxDataValue + " minDataValue = " + minDataValue + "\n   10% of interval = " + .1*(maxDataValue - minDataValue) + " max end width = " + (max - maxDataValue) + " min end width = " + (minDataValue - min));

	setBreaks();
	initCountsFromBreaks();
    }

    protected void setBreaks() {	
	int step = (int)Math.ceil((max-min)/nbClasses);
	nbClasses = (int)Math.ceil((max-min)/step);
	breaks = new double[nbClasses+1];
	for(int i = 0; i <= nbClasses; i++)
	    breaks[i] = min + i * step;
    }
}
