// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.propertyEditor;

import java.awt.Component;
import java.beans.PropertyEditorSupport;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;

import com.bbn.openmap.util.Debug;

import fr.unicaen.openmap.dataStorage.shapefile.ShapeFileDataStorage;
import fr.unicaen.openmap.event.GUIChangeEvent;
import fr.unicaen.openmap.event.GUIChangeListener;
import fr.unicaen.openmap.gui.FileNamePanel;

/**
 * This PropertyEditor provide two FileNamePanels.
 * @see fr.unicaen.openmap.gui.FileNamePanel
 */
public class ShapeFileDataStoragePropertyEditor extends PropertyEditorSupport implements GUIChangeListener {

    /**
     * Separator used in getAsText() between property and value.
     */
    public final static String keyValueSeparator = "\t";//!= DataStoragePropertyEditor.keyValueSeparator;

    /**
     * Separator used in getAsText() between properties.
     */
    public final static String propertySeparator = "\r"; //!= DataStoragePropertyEditor.propertySeparator;

    private String prefix = " ";
    private FileNamePanel shpPanel;
    private FileNamePanel dbfPanel;

 

    //    public ShapeFileDataStoragePropertyEditor() {    }



    public boolean supportsCustomEditor() {
	return true;
    }

    public Component getCustomEditor() {
	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

	shpPanel = new FileNamePanel(FileTypeDescription.SHP_DESCRIPTION, "", FileTypeDescription.SHP_PATTERN, this); 
	dbfPanel = new FileNamePanel(FileTypeDescription.DBF_DESCRIPTION, "", FileTypeDescription.DBF_PATTERN, this);          

	panel.add(shpPanel); 
	panel.add(dbfPanel); 

	return panel;
    }

    /**
     * Get the property value can be used. Warning : contains 2 properties : shape file path and dbf file path.
     * @see fr.unicaen.openmap.dataStorage.shapefile.ShapeFileDataStorage#getProperties(java.lang.String prefix, java.util.Properties properties) 
     * @see fr.unicaen.openmap.dataStorage.shapefile.ShapeFileDataStorage#getPropertyInfo(java.util.Properties info) 
     * @return a String representation for these properties
     */
    public String getAsText() {
	return 
	    prefix +
	    ShapeFileDataStorage.PARAM_SHP + keyValueSeparator + shpPanel.getFilePath()
	    + propertySeparator + prefix +
	    ShapeFileDataStorage.PARAM_DBF + keyValueSeparator + dbfPanel.getFilePath()
	    + propertySeparator;

    }

    public void setValue(Object value) {

	if (value instanceof ShapeFileDataStorage) {
	    ShapeFileDataStorage ds = (ShapeFileDataStorage) value;
	    shpPanel.setFilePath(ds.getShapeFile());
	    dbfPanel.setFilePath(ds.getDbfFile());
	}
    }

    public void reloadFromGUI(GUIChangeEvent event) {

	Object source = event.getSource();
        if (source == shpPanel) {
            String shpFile = shpPanel.getFilePath();
	    dbfPanel.setFilePath(shpFile.replaceFirst("\\.shp$",".dbf"));
	} else if (source == dbfPanel) {
	    //nothing
	} else {
            Debug.message("DataStorage", "[ShapeFileDataStorage] GUIChangeEvent not processed");
	}
    }
}
