// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util.propertyEditor;

import java.awt.Component;
import java.beans.PropertyEditorSupport;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import java.util.List;
import java.awt.event.ActionListener;

import com.bbn.openmap.util.Debug;

import fr.unicaen.openmap.layer.thematicDataDisplayLayer.ThematicDataDisplayLayer;
import fr.unicaen.openmap.dataStorage.DataStorage;
import fr.unicaen.openmap.dataStorage.SharedDataStorage;
import fr.unicaen.openmap.event.GUIChangeEvent;
import fr.unicaen.openmap.event.GUIChangeListener;

/**
 * 
 */
public class SharedDataStoragePropertyEditor extends PropertyEditorSupport implements GUIChangeListener {

    /**
     * Separator used in getAsText() between property and value.
     */
    public final static String keyValueSeparator = "\t";//!= DataStoragePropertyEditor.keyValueSeparator;

    /**
     * Separator used in getAsText() between properties.
     */
    public final static String propertySeparator = "\r"; //!= DataStoragePropertyEditor.propertySeparator;

    private String prefix = " ";
    private LayersPanel layersPanel;
    private DSPathPanel dsPanel;

//  public SharedDataStoragePropertyEditor() {
//    }

    public boolean supportsCustomEditor() {
	return true;
    }

    public Component getCustomEditor() {
	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

	layersPanel = new LayersPanel(this);
	panel.add(layersPanel); 
	
	dsPanel = new DSPathPanel();
	panel.add(dsPanel); 

	return panel;
    }

    /**
     * Get the property value can be used. Warning : contains 2 properties : layer name and datastorage  path.
     * @see fr.unicaen.openmap.dataStorage.shapefile.SharedDataStorage#getProperties(java.lang.String prefix, java.util.Properties properties) 
     * @see fr.unicaen.openmap.dataStorage.shapefile.SharedDataStorage#getPropertyInfo(java.util.Properties info) 
     * @return a String representation for these properties
     */
    public String getAsText() {
	return 
	    prefix +
	    SharedDataStorage. foreignLayerNamePropertyString + keyValueSeparator + layersPanel.getForeignLayerName()
	    + propertySeparator + prefix +
	    SharedDataStorage.dataStoragePathPropertyString + keyValueSeparator + dsPanel.getDSPath()
	    + propertySeparator;

    }

    public void setValue(Object value) {
	// System.out.println("->SharedDataStoragePropertyEditor.setValue("+value+")");
	if (value instanceof SharedDataStorage) {
	    SharedDataStorage ds = (SharedDataStorage) value;
	    layersPanel.setLayer(ds.getLayer());
	    layersPanel.setForeignLayer(ds.getForeignLayer());
	    dsPanel.setForeignLayer(ds.getForeignLayer());
	    dsPanel.setDS(ds.getDataStorage());
	}
	// System.out.println("<-SharedDataStoragePropertyEditor.setValue() => layer:"+layersPanel.getForeignLayer() + " dsp:" + dsPanel.getDSPath());
    }

    public void reloadFromGUI(GUIChangeEvent event) {

	Object source = event.getSource();
        if (source == layersPanel) {
	    dsPanel.setForeignLayer(layersPanel.getForeignLayer());
	} else if (source == dsPanel) {
	    // nothing
	} else {
            Debug.message("DataStorage", "*** [SharedDataStorage] GUIChangeEvent not processed from "+ source + " "+ source.hashCode()+" <> "+ layersPanel + " "+ this);
	}
    }
}

class LayersPanel extends JPanel {
    private JComboBox list;
    /** the layer of the SharedDataStorage to be excluded from the list */
    private ThematicDataDisplayLayer layer;
    private GUIChangeListener changeListener;
    public LayersPanel(GUIChangeListener changeListener) {       
	this.changeListener = changeListener;
	list = new JComboBox();
	add(list);
	init();
    }

    public void setLayer(ThematicDataDisplayLayer layer) {
	this.layer = layer;
    }

    public ThematicDataDisplayLayer getLayer() {
	return layer;
    }

    private void init() {
	for (ActionListener a : list.getActionListeners()) {
	    list.removeActionListener(a);
	}
	list.removeAllItems();
	List<ThematicDataDisplayLayer> layers = 
	    SharedDataStorage.getThematicDataDisplayLayers();
	for (ThematicDataDisplayLayer l : layers)
	    if (l != getLayer()) // exclude the layer of the SharedDataStorage
		list.addItem(l.getName());

	list.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent evt) {
		    changeListener.reloadFromGUI(new GUIChangeEvent(LayersPanel.this));
		}
	    });
	// list.setSelectedIndex(-1);
    }
	
    public String getForeignLayerName() {
	if (list.getSelectedIndex() < 0)
	    return null;
	return list.getSelectedItem().toString();
    }

    public ThematicDataDisplayLayer getForeignLayer() {
	if (list.getSelectedIndex() < 0)
	    return null;
	return  SharedDataStorage.getLayerByName(list.getSelectedItem().toString());
    }
    
    public void setForeignLayer(ThematicDataDisplayLayer l) {
	init();
	if (l == null) {
	    list.setSelectedIndex(-1);
	    return;
	}
	for (int i = 0; i < list.getItemCount(); i++) 
	    if (list.getItemAt(i).toString().equals(l.getName())) {
		list.setSelectedIndex(i);
		return;
	    }
    }
}

class DSPathPanel extends JPanel {
    private JComboBox list;
    private ThematicDataDisplayLayer foreignLayer;
    private List<DataStorage> dss; 

    public DSPathPanel() {
	list = new JComboBox();
	add(list);
	init();
    }

    private void init() {
	for (ActionListener a : list.getActionListeners()) {
	    list.removeActionListener(a);
	}
	list.removeAllItems();
	dss = SharedDataStorage.getDataStorageList(foreignLayer);
	Debug.message("DataStorage", "[SharedDataStoragePropertyEditor] DSS for : "+ foreignLayer + " => " + dss);
	if (foreignLayer == null) {
	    list.setEnabled(false);
	    return;
	}
	for (DataStorage ds : dss) {
	    list.addItem(ds.getName());
	}
	list.setEnabled(true);
    }

    public void setDS(DataStorage ds) {
	int index = -1;
	for (DataStorage _ds : dss) {
	    ++index;
	    if (_ds == ds)
		break;
	}
	list.setSelectedIndex(index);
    }
	
    public void setForeignLayer(ThematicDataDisplayLayer layer) {
	this.foreignLayer = layer;
	init();
    }

    public String getDSPath() {
	int index = list.getSelectedIndex();
	if (index== -1)
	    return "-";
	return SharedDataStorage.getDataStoragePath(foreignLayer, dss.get(index));
    }
}

